(   Title:  A utiilty for testing words that use the bstack for
	    Zettel bignums, or for GMP multiple precision integers
     File:  btester.fs
 Log file:  gmpfr.log
   Author:  David N. Williams
  License:  Public Domain, John Hayes
  Version:  1.5.0
  Started:  February 5, 2011
  Revised:  March 19, 2011

For any code derived from John Hayes' tester program:
)
\ (C) 1995 JOHNS HOPKINS UNIVERSITY / APPLIED PHYSICS LABORATORY
\ MAY BE DISTRIBUTED FREELY AS LONG AS THIS COPYRIGHT NOTICE REMAINS.
(
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

The rest is public domain.
)
BASE @ DECIMAL

\ *** ERRORS

VARIABLE BT-ERROR-XT
: BT-ERROR BT-ERROR-XT @ EXECUTE ;  \ vectored error reporting

VARIABLE bt-error-index  \ holds array index|-1 after error

: BT-ERROR-DEFAULT   ( c-addr u -- )
(
Display an error message followed by the line that had the
error.
)
  TYPE SOURCE TYPE CR ;
' BT-ERROR-DEFAULT BT-ERROR-XT !

\ *** ARRAYS
(
The first array slot has index zero.
)

\ USER CONFIG DEFAULTS
[UNDEFINED] MAX-BT-INITIALS [IF] 32 CONSTANT MAX-BT-INITIALS [THEN]
[UNDEFINED] MAX-BT-RESULTS  [IF] 32 CONSTANT MAX-BT-RESULTS  [THEN]

\ Arrays begin with a cell-sized count.
CREATE bt-initials MAX-BT-INITIALS 1+ CELLS ALLOT
CREATE bt-results  MAX-BT-RESULTS  1+ CELLS ALLOT
CREATE bt-goals    MAX-BT-RESULTS  1+ CELLS ALLOT

: 0BT-A  ( +n 'array -- )  \ assume n<>0
(
Initialize array b variables for garbage collection.
)
  swap 1+ cells over + swap DO 0 i ! 1 cells +LOOP ;

MAX-BT-INITIALS bt-initials 0BT-A
MAX-BT-RESULTS  bt-results  0BT-A
MAX-BT-RESULTS  bt-goals    0BT-A

\ Overflow checking to be done elsewhere.
: bt-a!   ( +n 'array -- ) ( b: b_[n-1] ... b_0 -- )
  2dup ! cell+ swap ( 'array n) dup
  IF   cells over + swap
       DO i b! 1 cells +LOOP 
  ELSE 2drop THEN ;

: bt-a@   ( 'array -- +n ) (b: -- b_[n-1] ... b_0 )
  dup @ dup >r ( 'array n r: n)
  IF   cell+  r@ 1- cells over + 
       DO i b@ -1 cells +LOOP
  ELSE drop THEN r> ;

[UNDEFINED] NIP [IF] : NIP  ( a b -- b)  swap drop ; [THEN]

: bt-a<>  ( 'array1 'array2 -- [index+1]|-1|0 )
(
Nonzero return for errors:
  -1       unequal array size
  index+1  deepest unequal array elements
)
  dup @ swap cell+ >r >r  ( 'array1 r: 'b2 #left )
  dup @ r@ <> IF 2r> 2drop drop -1 EXIT THEN
  ( 'array1) dup cell+
  BEGIN  r@ ( 'array1 'b1 #left)
  WHILE  ( 'b1) dup b@ cell+
         r> ( #left) 1- r>
         ( 'b2) dup b@ cell+ >r ( #left) >r
         b=
  WHILE
  REPEAT ( 'array1 'b1) drop @ ( len) 2r> nip - ( index+1)
  ELSE   ( 'array1 'b1) 2drop 2r> 2drop 0
  THEN ;

\ *** TESTS

: B{   ( <binitials> -- )
(
Save the big number stack <binitials> for restoration by }B. 
Clear <bresults> and <bgoals> storage to put it in a defined
state for }B in case B-> is missing.
)
  bdepth dup MAX-BT-INITIALS > ABORT" TOO MANY B INITIALS"
  bt-initials bt-a!   0 bt-results !   0 bt-goals ! ;

: B->  ( <bresults> -- )
(
Record the depth and contents of the big number stack.
)
  bdepth dup MAX-BT-RESULTS > ABORT" TOO MANY B RESULTS"
  bt-results bt-a! ;

: }B   ( <bgoals> -- <binitials> )
(
Save the big number stack <bgoals>, compare them with the saved
<bresults>, and restore the <binitials>.
)
  bdepth dup MAX-BT-RESULTS > ABORT" TOO MANY B GOALS"
  bt-goals bt-a!
  bt-results bt-goals bt-a<> dup bt-error-index !
  ?DUP IF -1 =
    IF   S" WRONG NUMBER OF B RESULTS: "
    ELSE -1 bt-error-index +! S" INCORRECT B RESULT: " THEN
    BT-ERROR
  THEN
  bt-initials bt-a@ ( len) drop ;

BASE !

\ *** END of btester.fs
