(       Title:  Complex Reliable Float Word Set Tests
         File:  complex-rf-test.fs
       Author:  David N. Williams
      Version:  1.0.5
      License:  LGPL
Last revision:  May 12, 2011
)
\ Copyright (C) 2002, 2003, 2005-2011 David N. Williams
(
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or at your option any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA 02111-1307 USA.

This code tests rfcomplex.fs, a reliable floating-point [mpfr]
port of revisions of Julian V. Noble's complex arithmetic
lexicon in complex.fs [FSL Algorithm #60].

It is intended to test for formal correctness, not high
accuracy.  It does not include any tests involving IEEE 754-2008
style floating-point values for signed zero, NaN, or infinity.

"Gauge functions" are functions that we test against.  They are
defined independently here, sometimes in terms of already tested
functions.

This code is compatible with ANS Forth 200x, with an
environmental dependence on lower case.

Unattributed changes are by David N. Williams.  The last
revision date above may reflect cosmetic changes not logged
here.

Version 1.0.5
 9May11 * Reverted mixed complex/real nomenclature to use "f"
	  instead of "rf.  Reverted "-z" to "znegate".

Version 1.0.4
 6May11 * Started port from complex-test-xf.fs 1.0.3.
 7May11 * Released.
)

\ debugging
[UNDEFINED] \\ [IF]
: \\  ( -- )  -1 parse 2drop BEGIN refill 0= UNTIL ;
[THEN]
: .#  ( n -- )  cr ." #" . cr ;

(
The nonprincipal argument is used by default in complex-rf.fs,
but only for arg, log, and nonintegral powers.  Uncomment the
line below to use principal argument for those operations as
well.
)
s" complex-rf.fs" included 
\ true to PRINCIPAL-ARG

0 value BITS/FLOAT
: binary64  ( -- )  53  1023 set-IEEE-754  64 to BITS/FLOAT ;
: binary80  ( -- )  80  1023 set-IEEE-754  80 to BITS/FLOAT ;
: binary128 ( -- ) 113 16383 set-IEEE-754 128 to BITS/FLOAT ;
binary64

s" ttester-xf.fs"      INCLUDED
s" tester-display.fs"  INCLUDED
s" xtester-errors.fs"  INCLUDED
s" ftester-errors.fs"  INCLUDED
s" rftester.fs"        INCLUDED
s" rftester-errors.fs" INCLUDED
true VERBOSE !   6 set-precision

: ?.errors
  ?.xt-errors s"   " ?type
  ?.ft-errors s"   " ?type
  ?.rft-errors ;

: ={  rf{ t{ ;   : =>  rf-> -> ;   : }=  }rf }t ;

\ before first displayed line
PFE-HOST IFORTH-HOST or [IF] ?.cr [THEN]

PFE-HOST [IF]
  ?." Host is pfe, "
\ 0 REDEFINED-MSG !
[THEN]
GFORTH-HOST [IF]
  ?." Host is gforth, "
\ 0 WARNINGS !
[THEN]
IFORTH-HOST [IF]
  ?." Host is iForth, "
\ 0 WARNING !
[THEN]

?." floats are " BITS/FLOAT . ?." bits.

BITS/FLOAT 80 = [IF] \ extended double 20.27 significant digits
  rf" 1e-17"  \ rel err
  rf" 1e-17"  \ abs err
  GFORTH-HOST [IF] 22 
  [ELSE] IFORTH-HOST
    [IF] 18 [ELSE] 21 [THEN]
  [THEN] set-precision
[THEN]

BITS/FLOAT 64 = [IF] \ double 16.95 significant digits
  rf" 1e-15"  \ rel err
  rf" 1e-15"  \ abs err
  GFORTH-HOST [IF] 18 [ELSE] 17 [THEN] set-precision
[THEN]

\ untested
BITS/FLOAT 32 = [IF]  \ single 8.225 significant digits
  rf" 1.5e-7"  \ rel err
  rf" 1.5e-7"  \ abs err
  GFORTH-HOST [IF] 10 [ELSE] 9 [THEN] set-precision
[THEN]
rfconstant abs-err-default   rfconstant rel-err-default 

: set-rft-epsilons  ( -- )
  abs-err-default RFT-ABS-ERROR  rf!
  rel-err-default RFT-REL-ERROR  rf! ;
set-rft-epsilons

: rft-mrel=  (rf: m r -- s: flag )
(
Let eps1 and eps2 be the values of RFT-ABS-ERROR and
RFT-REL-ERROR.  Leave true if |m - r| < max [eps1, eps2 * |r|],
else false.  The "m" in the name stands for "moderated".  We use
this instead of RFT-REL0= because sometimes our reference values
are very small but nonzero, causing RFT-REL= to be used where it
is inaccurate.
)
  rfdup rft-rel-error rf@ rf* |rf|
  rft-abs-error rf@ rf-max
  -rfrot rf- |rf| rfswap rf< ;

: set-rft-mode-mrel  ( -- )  ['] rft-mrel= rft-test=-xt ! ;

set-rft-mode-mrel

rf" 1.001E0" rf" 1E0" rf- rf" 1E0" rf/
    rfconstant TEST-REL-ERR \ from ttester-xf-test.fs
TEST-REL-ERR rft-rel-error rf!

={ rf" 1e-15"         => rf" 1.001e-15" }=
={ rf" 1.0009999e-15" => rf" 1e-15" }=
={ rf" 1.0009E10" rf" 1E10" rft-mrel= => true }=
={ rf" 1.0011E10" rf" 1E10" rft-mrel= => false }=
set-rft-epsilons

rf"  0.7853981633974483096157E0" rfconstant  pi/4
rf" -0.7853981633974483096157E0" rfconstant -pi/4
rf"  0.5497787143782138167310E1" rfconstant  7pi/4
rf" -0.1570796326794896619231E1" rfconstant -pi/2
rf"  0.4712388980384689857694E1" rfconstant  3pi/2

rf"  0.2356194490192344928847E1" rfconstant  3pi/4
rf" -0.2356194490192344928847E1" rfconstant -3pi/4
rf"  0.3926990816987241548078E1" rfconstant  5pi/4

rf" -0.3141592653589793238463E1" rfconstant -pi
rf"  0.2718281828459045235360E1" rfconstant  e
rf" -0.2718281828459045235360E1" rfconstant -e
rf"  0.3678794411714423215955E0" rfconstant  1/e
rf" -0.3678794411714423215955E0" rfconstant -1/e
rf"  0.6931471805599453094172E0" rfconstant  ln2
rf" -0.6931471805599453094172E0" rfconstant -ln2
rf"  0.1414213562373095048802E1" rfconstant  rt2
rf" -0.1414213562373095048802E1" rfconstant -rt2
rf"  0.7071067811865475244008E0" rfconstant  1/rt2
rf" -0.7071067811865475244008E0" rfconstant -1/rt2

\ only two choices
PRINCIPAL-ARG [IF]
s" arg output -pi < arg <= pi" ?type
  -3pi/4 rfconstant 225arg
  -pi/2  rfconstant 270arg
  -pi/4  rfconstant 315arg
[ELSE]
s" with arg output 0 <= arg < 2pi ?type
   5pi/4 rfconstant 225arg
   3pi/2 rfconstant 270arg
   7pi/4 rfconstant 315arg
[THEN] ?.cr

defer gauge
defer func
defer inverse

\ ZVARIABLE, Z!, and Z@ have to be tested before the next
\ words are used.

zvariable zatemp   zvariable zbtemp

: ?gauge  (rf: z -- )
(
Compare the functions whose xt's are in FUNC and GAUGE.
) 
  zdup zatemp z! gauge   =>   zatemp z@ func ;

: ?2gauge  (rf: z1 z2 -- )
(
Same as above with 2 complex arguments.
) 
  zbtemp z! zatemp z!
  zatemp z@ zbtemp z@ gauge   =>   zatemp z@ zbtemp z@ func ;

: ?inverse  (rf: z -- )
(
Check that INVERSE FUNCT, i.e., func[inverse], is the identity
mapping.
)
  zdup zatemp z! inverse func   =>   zatemp z@ ;


\ *** SPECIAL RF WORDS

testing  RF2* RF2/ RF^2 1/RF -0RF=

rf"   0" rfconstant  +0.0
rf"  -0" rfconstant  -0.0
rf"   1" rfconstant   1.0
rf"   2" rfconstant   2.0
rf"   3" rfconstant   3.0
rf"   4" rfconstant   4.0
rf"   5" rfconstant   5.0
rf"   6" rfconstant   6.0
rf"   7" rfconstant   7.0
rf"   8" rfconstant   8.0
rf"  10" rfconstant  10.0
rf"  20" rfconstant  20.0
rf"  24" rfconstant  24.0
rf"  25" rfconstant  25.0
rf" 128" rfconstant 128.0
rf"  -1" rfconstant  -1.0
rf"  -2" rfconstant  -2.0
rf"  -3" rfconstant  -3.0
rf"  -4" rfconstant  -4.0
rf"  -5" rfconstant  -5.0
rf"  -6" rfconstant  -6.0
rf"  -7" rfconstant  -7.0
rf"  -8" rfconstant  -8.0
rf" -10" rfconstant -10.0
rf" -20" rfconstant -20.0
rf" -24" rfconstant -24.0
rf" -25" rfconstant -25.0
rf" -128"   rfconstant -128.0
rf"  0.5e0" rfconstant  0.5
rf" -0.5e0" rfconstant -0.5

set-rft-mode-exact 

={  2.0 1/rf =>  0.5 }=
={ -2.0 1/rf => -0.5 }=

={ +0.0 rf^2 => +0.0 }=
={  2.0 rf^2 =>  4.0 }=
={ -2.0 rf^2 =>  4.0 }=

={ +0.0   rf2* => +0.0 }=
={  2.0   rf2* =>  4.0 }=
set-rft-mode-mrel
={  1/rt2 rf2* =>  rt2 }=
={ -1/rt2 rf2* => -rt2 }=

set-rft-mode-exact
={ +0.0 rf2/ => +0.0   }=
={  4.0 rf2/ =>  2.0   }=
set-rft-mode-mrel
={  rt2 rf2/ =>  1/rt2 }=
={ -rt2 rf2/ => -1/rt2 }=

={ -0.0 -0rf= => true }=
={ +0.0 -0rf= => false }=


\ *** LOAD AND STORE

testing  ZCONSTANT ZVARIABLE Z@ Z!

set-rft-mode-exact
={ 1.0 2.0 zconstant 1+i2 =>       }=
={ 1+i2                 => 1.0 2.0 }=
={ : equ zconstant ;    =>       }=
={ 1+i2 equ z=(1+i2)    =>       }=
={ z=(1+i2)             => 1+i2  }=

+0.0 +0.0 zconstant  0+i0
 1.0 +0.0 zconstant  1+i0
-1.0 +0.0 zconstant -1+i0
+0.0  1.0 zconstant  0+i1
+0.0 -1.0 zconstant  0-i1

={ zvariable zv1 =>      }=
={ 1+i2 zv1 z!   =>      }=
={ zv1 z@        => 1+i2 }=

testing  COMPLEX COMPLEXES COMPLEX+

={    COMPLEX   => 2 cells }=
={  0 COMPLEXES => 0 }=
={  3 COMPLEXES => COMPLEX 3 * }=
={ -1 COMPLEXES => COMPLEX negate }=
={  1 COMPLEX+  => 1 COMPLEX + }=


\ *** COMPLEX STACK MANIPULATION

testing  ZDROP ZDUP ZSWAP ZOVER ZNIP ZTUCK ZROT -ZROT

set-rft-mode-exact
={ 0+i0      zdrop =>                }=
={ 1+i0      zdup  => 1+i0 1+i0      }=
={ 0+i0 1+i0 zswap => 1+i0 0+i0      }=
={ 0+i0 1+i0 zover => 0+i0 1+i0 0+i0 }=
={ 0+i0 1+i0 znip  => 1+i0           }=
={ 0+i0 1+i0 ztuck => 1+i0 0+i0 1+i0 }=
={ 0+i0 1+i0 0+i1  zrot => 1+i0 0+i1 0+i0 }=
={ 0+i0 1+i0 0+i1 -zrot => 0+i1 0+i0 1+i0 }=


\ *** COMPLEX ALGEBRA

testing  REAL IMAG CONJG Z*F  Z/F Z* Z/ Z+ Z-

set-rft-mode-exact
={ 1+i0    real  => 1.0 }=
={ 0+i1    imag  => 1.0 }=
={ 1.0 2.0 conjg => 1.0 -2.0 }=

set-rft-mode-mrel  \ true also works in pfe
={ 1.0 2.0 3.0	   z*f =>  3.0  6.0 }=
={ 3.0 6.0 3.0	   z/f =>  1.0  2.0 }=
={ 1.0 2.0 3.0 4.0 z*  => -5.0 10.0 }=
={ 1.0 2.0 1+i0    z*  =>  1.0  2.0 }=
={ 1.0 2.0 0+i1    z*  => -2.0  1.0 }=
={ 1.0 1.0 3.0 4.0 z/  =>  7.0 25.0 rf/ -1.0 25.0 rf/ }=
={ 1.0 1.0 4.0 3.0 z/  =>  7.0 25.0 rf/  1.0 25.0 rf/ }=
={ 1.0 2.0 3.0 4.0 z+  =>  4.0  6.0 }=
={ 1.0 2.0 3.0 4.0 z-  => -2.0 -2.0 }=

testing  ZNEGATE Z2* Z2/ I* -I*

: -z  znegate ;

set-rft-mode-mrel  \ ignore signed zero
={  0+i0 -z  =>  0+i0 }=
={  0+i0 z2* =>  0+i0 }=
={  0+i0 z2/ =>  0+i0 }=
={  0+i0 i*  =>  0+i0 }=
={  0+i0 -i* =>  0+i0 }=

set-rft-mode-exact
={  1.0 -2.0 -z  => -1.0  2.0  }=
={  4.0 -2.0 z2* =>  8.0 -4.0 }=
={ -4.0  2.0 z2* => -8.0  4.0 }=
={  4.0 -2.0 z2/ =>  2.0 -1.0 }=
={ -4.0  2.0 z2/ => -2.0  1.0 }=
={  4.0 -2.0 i*  =>  2.0  4.0 }=
={ -4.0  2.0 i*  => -2.0 -4.0 }=
={  4.0 -2.0 -i* => -2.0 -4.0 }=
={ -4.0  2.0 -i* =>  2.0  4.0 }=


\ *** MINIMAL (MIXED) OPERATIONS

testing  X+ X- Y+ Y-

set-rft-mode-exact
={ 1.0 2.0 3.0 x+ =>  4.0 2.0 }=
={ 1.0 2.0 3.0 x- => -2.0 2.0 }=
={ 1.0 2.0 4.0 y+ =>  1.0  6.0 }=
={ 1.0 2.0 4.0 y- =>  1.0 -2.0 }=

[DEFINED] i*rf/z [IF]
testing  Z*>REAL Z*>IMAG  Z*F Z/F RF*Z F/Z Z*I*F -I*Z/F I*FZ* I*F/Z

set-rft-mode-exact
={  1.0  2.0 3.0 4.0 z*>real => -5.0 }=
={  1.0  2.0 3.0 4.0 z*>imag => 10.0 }=
={ -1.0  2.0 3.0 z*f => -1.0  2.0  3.0 +0.0 z* }=
={ -3.0 -6.0 3.0 z/f => -3.0 -6.0  3.0 +0.0 z/ }=
={  3.0  1.0 2.0 rf*z =>  3.0  +0.0  1.0 2.0 z* }=
set-rft-mode-mrel  \ pfe works with exact
={  3.0  6.0 3.0 rf/z =>  3.0  +0.0  6.0 3.0  z/ }=
set-rft-mode-exact
={ -1.0  2.0 3.0  z*i*rf => -1.0  2.0  +0.0 3.0 z* }=
={ -3.0 -6.0 3.0 -i*z/f => -3.0 -6.0  +0.0 3.0 z/ }=
={  3.0  1.0 2.0  i*rf*z =>  +0.0  3.0  1.0 2.0 z* }=
={  3.0  6.0 3.0  i*rf/z =>  +0.0  3.0  6.0 3.0 z/ }=
[THEN]


\ *** ALGEBRAIC FUNCTIONS

testing  |Z| |Z|^2 1/Z Z^2 Z^N

set-rft-mode-exact
={  0+i0 |z|   => +0.0 }=
={  0+i0 |z|^2 => +0.0 }=
={  0+i0 z^2   => 0+i0 }=

set-rft-mode-mrel
={  3.0  4.0 |z|   =>  5.0 }=
={ -3.0  4.0 |z|   =>  5.0 }=
={  3.0 -4.0 |z|   =>  5.0 }=
={  3.0  4.0 |z|^2 => 25.0 }=
={ -3.0  4.0 |z|^2 => 25.0 }=
={  3.0 -4.0 |z|^2 => 25.0 }=
={  3.0  4.0 1/z   =>  3.0  25.0 rf/ -4.0 25.0 rf/ }=
={ -3.0  4.0 1/z   => -3.0  25.0 rf/ -4.0 25.0 rf/ }=
={  3.0 -4.0 1/z   => -3.0 -25.0 rf/  4.0 25.0 rf/ }=
={  1+i0     1/z   =>  1+i0 }=
={  3.0  4.0 z^2   => -7.0  24.0 }=
={ -3.0  4.0 z^2   => -7.0 -24.0 }=
={  3.0 -4.0 z^2   => -7.0 -24.0 }=
={ -3.0 -4.0 z^2   => -7.0  24.0 }=

set-rft-mode-exact
={  0+i0     0 z^n =>  1+i0 }=
={  1+i0     0 z^n =>  1+i0 }=
={ -1+i0     0 z^n =>  1+i0 }=
={  0+i1     0 z^n =>  1+i0 }=
={  0-i1     0 z^n =>  1+i0 }=
={  rt2  rt2 0 z^n =>  1+i0 }=
={  rt2 -rt2 0 z^n =>  1+i0 }=
={ -rt2  rt2 0 z^n =>  1+i0 }=
={ -rt2 -rt2 0 z^n =>  1+i0 }=

={  0+i0     1 z^n =>  0+i0 }=
={  1+i0     1 z^n =>  1+i0 }=
={ -1+i0     1 z^n => -1+i0 }=
={  0+i1     1 z^n =>  0+i1 }=
={  0-i1     1 z^n =>  0-i1 }=
={  rt2  rt2 1 z^n =>  rt2  rt2 }=
={  rt2 -rt2 1 z^n =>  rt2 -rt2 }=
={ -rt2  rt2 1 z^n => -rt2  rt2 }=
={ -rt2 -rt2 1 z^n => -rt2 -rt2 }=

={  0+i0     2 z^n =>  0+i0 }=
={  1+i0     2 z^n =>  1+i0 }=
={ -1+i0     2 z^n =>  1+i0 }=
={  0+i1     2 z^n => -1+i0 }=
set-rft-mode-mrel \ avoid signed zero discrepancy
={  0-i1     2 z^n => -1+i0 }=
set-rft-mode-exact
={  3.0  4.0 2 z^n => -7.0  24.0 }=
={ -3.0  4.0 2 z^n => -7.0 -24.0 }=
={  3.0 -4.0 2 z^n => -7.0 -24.0 }=
={ -3.0 -4.0 2 z^n => -7.0  24.0 }=

={  0+i0     5 z^n =>  0+i0 }=
={  1+i0     5 z^n =>  1+i0 }=
={ -1+i0     5 z^n => -1+i0 }=
={  0+i1     5 z^n =>  0+i1 }=
={  0-i1     5 z^n =>  0-i1 }=
={  2.0  2.0 5 z^n => -128.0 -128.0 }=
={  2.0 -2.0 5 z^n => -128.0  128.0 }=
={ -2.0  2.0 5 z^n =>  128.0 -128.0 }=
={ -2.0 -2.0 5 z^n =>  128.0  128.0 }=


\ *** ELEMENTARY FUNCTIONS

testing  ARG >POLAR POLAR> ZSQRT ZLN ZEXP Z^

set-rft-mode-mrel
={  0+i0     arg => +0.0   }=
={  1+i0     arg => +0.0   }=
={  0+i1     arg => pi/2   }=
={  0-i1     arg => 270arg }=
={ -1+i0     arg => pi     }=
={  2.0  2.0 arg => pi/4   }=
={  3.0 -3.0 arg => 315arg }=
={ -rt2  rt2 arg => 3pi/4  }=
={ -rt2 -rt2 arg => 225arg }=

rt2 rf2*    rfconstant 2rt2
rt2 3.0 rf* rfconstant 3rt2

set-rft-mode-mrel
={  0+i0     >polar => 0+i0     }=
={  1+i0     >polar => 1+i0     }=
={  0+i1     >polar => 1.0 pi/2   }=
={  0-i1     >polar => 1.0 270arg }=
={ -1+i0     >polar => 1.0 pi     }=
={  2.0  2.0 >polar => 2rt2 pi/4   }=
={  3.0 -3.0 >polar => 3rt2 315arg }=
={ -rt2  rt2 >polar => 2.0   3pi/4  }=
={ -rt2 -rt2 >polar => 2.0   225arg }=

set-rft-mode-exact
={ 0+i0        polar> =>  0+i0 }=
={ 1+i0        polar> =>  1+i0 }=
set-rft-mode-mrel
={ 1.0   pi/2  polar> =>  0+i1 }= \ Re(f) = -6.123234E-17
={ 1.0  -pi/2  polar> =>  0-i1 }= \ Re(f) =  6.123234E-17
={ 1.0   pi    polar> => -1+i0 }= \ Im(f) =  1.224647E-16
={ 2rt2  pi/4  polar> =>  2.0   2.0  }=
={ 3rt2 -pi/4  polar> =>  3.0  -3.0  }=
={ 2.0   3pi/4 polar> => -rt2  rt2 }=
={ 2.0  -3pi/4 polar> => -rt2 -rt2 }=

: gsqrt  (rf: z -- exp[[ln|z|+iarg[z]]/2] )
  zln z2/ zexp ; 

' zsqrt is func   ' gsqrt is gauge
set-rft-mode-mrel
={ 0+i0 zsqrt => 0+i0 }=
={  2.0 +0.0 ?gauge }=
={ -2.0 +0.0 ?gauge }= \ Re(g) = 8.659561E-17
={ +0.0  2.0 ?gauge }=
={ +0.0 -2.0 ?gauge }=
={  rt2  rt2 ?gauge }=
={  rt2 -rt2 ?gauge }=
={ -rt2  rt2 ?gauge }=
={ -rt2 -rt2 ?gauge }=

set-rft-mode-exact
={  1+i0     zln => 0+i0      }=
set-rft-mode-mrel
={  0+i1     zln => +0.0 pi/2   }=
={  0-i1     zln => +0.0 270arg }=
={ -1+i0     zln => +0.0 pi     }=
={ -2.0 +0.0 zln => ln2 pi     }=
={  rt2  rt2 zln => ln2 pi/4   }=
={  rt2 -rt2 zln => ln2 315arg }=
={ -rt2  rt2 zln => ln2 3pi/4  }=
={ -rt2 -rt2 zln => ln2 225arg }=

 1/rt2 rf2/ rfconstant  1/2rt2
 1/2rt2 -rf rfconstant -1/2rt2

set-rft-mode-exact
={  0+i0       zexp =>  1+i0 }=
={  ln2 +0.0   zexp =>  2.0   +0.0 }=
set-rft-mode-mrel
={ -ln2 +0.0   zexp =>  0.5    +0.0 }=
={ +0.0  pi    zexp => -1+i0    }= \ Im(f) = 1.224647E-16
={ +0.0  pi/2  zexp =>  0+i1    }= \ Re(f) = 6.123234E-17
={ +0.0 -pi/2  zexp =>  0+i1 conjg }= \ Re(f) = 6.123234E-17
={ +0.0  pi/4  zexp =>  1/rt2   1/rt2  }=
={ +0.0 -pi/4  zexp =>  1/rt2  -1/rt2  }=
={ +0.0  3pi/4 zexp => -1/rt2   1/rt2  }=
={ +0.0 -3pi/4 zexp => -1/rt2  -1/rt2  }=
={  ln2  pi/4  zexp =>  rt2     rt2    }=
={  ln2 -pi/4  zexp =>  rt2    -rt2    }=
={ -ln2  3pi/4 zexp => -1/2rt2  1/2rt2 }=
={ -ln2 -3pi/4 zexp => -1/2rt2 -1/2rt2 }=

set-rft-mode-exact
={  1+i0     0+i0  z^ =>  1+i0 }=
={ -1+i0     0+i0  z^ =>  1+i0 }=
={  0+i1     0+i0  z^ =>  1+i0 }=
={  0-i1     0+i0  z^ =>  1+i0 }=
={  rt2  rt2 0+i0  z^ =>  1+i0 }=
={  rt2 -rt2 0+i0  z^ =>  1+i0 }=
={ -rt2  rt2 0+i0  z^ =>  1+i0 }=
={ -rt2 -rt2 0+i0  z^ =>  1+i0 }=

: identical  (rf: z -- z )  ;

: z^(z=1)   1+i0 z^ ;

' z^(z=1) is func   ' identical is gauge
set-rft-mode-mrel
={  1+i0     ?gauge }=
={ -1+i0     ?gauge }= \ Im(f) = 1.224647E-16
={  0+i1     ?gauge }= \ Re(f) = 6.123234E-17
={  0-i1     ?gauge }= \ Re(f) = 6.123234E-17
={  rt2  rt2 ?gauge }=
={  rt2 -rt2 ?gauge }=
={ -rt2  rt2 ?gauge }=
={ -rt2 -rt2 ?gauge }=

:noname  (rf: z -- z^(1+i2)  1+i2 z^ ;          is func
:noname  (rf: z -- z^(1+i2)  zln 1+i2 z* zexp ; is gauge
={  1+i0     ?gauge }=
={ -1+i0     ?gauge }=
={  0+i1     ?gauge }=
={  0-i1     ?gauge }=
={  rt2  rt2 ?gauge }=
={  rt2 -rt2 ?gauge }= \ pfe 1
={ -rt2  rt2 ?gauge }=
={ -rt2 -rt2 ?gauge }= \ pfe 2

testing  ZCOSH ZSINH ZTANH ZCOTH ZCOS ZSIN ZTAN ZCOT

e 1/e rf+ rf2/ rfconstant ch1
e 1/e rf- rf2/ rfconstant sh1
      ch1 +0.0 zconstant  zch1
      sh1 +0.0 zconstant  zsh1
  sh1 ch1 rf/  rfconstant th1
  ch1 sh1 rf/  rfconstant cth1
      th1 +0.0 zconstant  zth1
     cth1 +0.0 zconstant  zcth1

ch1 sh1 rt2 z/f zconstant zC1
      zC1 conjg zconstant zC2
sh1 ch1 rt2 z/f zconstant zC3
      zC3 conjg zconstant zC4 

set-rft-mode-exact
={  0+i0      zcosh => 1+i0 }=
set-rft-mode-mrel
={  1+i0      zcosh => zch1 }=
={ -1+i0      zcosh => zch1 }=
={ +0.0  pi/2 zcosh => 0+i0 }= \ Re(f) = 6.123234E-17
={ +0.0 -pi/2 zcosh => 0+i0 }= \ Re(f) = 6.123234E-17
={  1.0  pi/4 zcosh => zC1  }=
={  1.0 -pi/4 zcosh => zC2  }=
={ -1.0  pi/4 zcosh => zC2  }=
={ -1.0 -pi/4 zcosh => zC1  }=

set-rft-mode-exact
={  0+i0      zsinh => 0+i0 }=
set-rft-mode-mrel
={  1+i0      zsinh => zsh1 }=
={ -1+i0      zsinh => sh1 -rf +0.0 }=
={ +0.0  pi/2 zsinh => 0+i1 }=
={ +0.0 -pi/2 zsinh => 0+i1 conjg }=
={  1.0  pi/4 zsinh => zC3  }=
={  1.0 -pi/4 zsinh => zC4  }=
={ -1.0  pi/4 zsinh => zC4 -z }=
={ -1.0 -pi/4 zsinh => zC3 -z }=

1.0  pi/4 zdup zsinh zswap zcosh z/ zconstant ztanhA
1.0 -pi/4 zdup zsinh zswap zcosh z/ zconstant ztanhB
ztanhA 1/z zconstant zcothA
ztanhB 1/z zconstant zcothB

set-rft-mode-exact
={  0+i0      ztanh => 0+i0 }=
set-rft-mode-mrel
={  1+i0      ztanh => zth1 }=
={ -1+i0      ztanh => zth1 -z }=
={ +0.0  pi/4 ztanh => 0+i1 }=
={ +0.0 -pi/4 ztanh => 0+i1 conjg }=
={  1.0  pi/4 ztanh => ztanhA }=
={  1.0 -pi/4 ztanh => ztanhB }=
={ -1.0  pi/4 ztanh => ztanhB -z  }=
={ -1.0 -pi/4 ztanh => ztanhA -z  }=

set-rft-mode-mrel
={  1+i0      zcoth => zcth1 }=
={ -1+i0      zcoth => zcth1 -z  }=
={ +0.0  pi/4 zcoth => 0+i1 conjg }=
={ +0.0 -pi/4 zcoth => 0+i1 }=
={  1.0  pi/4 zcoth => zcothA }=
={  1.0 -pi/4 zcoth => zcothB }=
={ -1.0  pi/4 zcoth => zcothB -z }=
={ -1.0 -pi/4 zcoth => zcothA -z }=

:noname  -i* zcos ; is func
' zcosh is gauge
set-rft-mode-exact
={  0+i0      ?gauge }=
={  1+i0      ?gauge }=
={ -1+i0      ?gauge }=
={ +0.0  pi/2 ?gauge }=
={ +0.0 -pi/2 ?gauge }=
={  1.0  pi/4 ?gauge }=
={  1.0 -pi/4 ?gauge }=
={ -1.0  pi/4 ?gauge }=
={ -1.0 -pi/4 ?gauge }=

:noname  i* zsin ;  is func
:noname  zsinh i* ; is gauge
set-rft-mode-mrel
={  0+i0      ?gauge }=
={  1+i0      ?gauge }=
={ -1+i0      ?gauge }=
={ +0.0  pi/2 ?gauge }=
={ +0.0 -pi/2 ?gauge }=
={  1.0  pi/4 ?gauge }=
={  1.0 -pi/4 ?gauge }=
={ -1.0  pi/4 ?gauge }=
={ -1.0 -pi/4 ?gauge }=

:noname  i* ztan ;  is func
:noname  ztanh i* ; is gauge
set-rft-mode-mrel
={  0+i0      ?gauge }=
={  1+i0      ?gauge }=
={ -1+i0      ?gauge }=
={ +0.0  pi/4 ?gauge }=
={ +0.0 -pi/4 ?gauge }=
={  1.0  pi/4 ?gauge }=
={  1.0 -pi/4 ?gauge }=
={ -1.0  pi/4 ?gauge }=
={ -1.0 -pi/4 ?gauge }=

:noname  i* zcot ;     is func
:noname  zcoth -i* ;   is gauge
set-rft-mode-mrel
={  1+i0      ?gauge }=
={ -1+i0      ?gauge }=
={ +0.0  pi/4 ?gauge }=
={ +0.0 -pi/4 ?gauge }=
={  1.0  pi/4 ?gauge }=
={  1.0 -pi/4 ?gauge }=
={ -1.0  pi/4 ?gauge }=
={ -1.0 -pi/4 ?gauge }=


\ *** INVERSE FUNCTIONS

true to PRINCIPAL-ARG

testing  ZASINH ZACOSH ZATANH ZACOTH

\ Inverse hyperbolic gauges.  Note that in the principal
\ expressions for the gauges here, and in GACOS in the next
\ section, it is important to use "1.0 x+" instead of "1+i0 z+"
\ to preserve the sign of zero on the branch cuts.  That is
\ not tested in this file (see complex-ieee-test.fs).

: gasinh   ( z -- [ln[z+sqrt[z^2+1]]] )
  zdup z^2 1.0 x+ zsqrt z+ zln ;

: gacosh   ( z -- 2ln[sqrt[[z+1]/2]+sqrt[[z-1]/2] )
  zdup 1.0 x- z2/ zsqrt   zswap 1.0 x+ z2/ zsqrt
  z+ zln z2* ;

: gatanh   ( z -- [ln[1+z]-ln[1-z]]/2 )
  zdup 1.0 x+ zln   zswap -z 1.0 x+ zln   z- z2/ ;

: gacoth  ( z = [ln[-1-z]-ln[1-z]]/2 )
(
Use -1.0 +0.0 Z+ instead of 1+i0 Z- so -0 doesn't give the wrong
value on the ZLN principal branch cut.
)
  -z zdup -1+i0 z+ zln   zswap 1+i0 z+ zln   z- z2/ ;

\ Check that the gauges are inverses.  The order func(inverse)
\ used here, e.g., ZACOSH COSH in Forth reverse polish, should
\ work for all branches of the inverse when func is meromorphic.

' gasinh is inverse   ' zsinh is func
set-rft-mode-mrel
={ 0+i0       ?inverse }=
={ zsh1       ?inverse }=
={ zsh1 -z    ?inverse }=
={ 0+i1       ?inverse }=
={ 0+i1 conjg ?inverse }=
={ zC3        ?inverse }=
={ zC4        ?inverse }=
={ zC4 -z     ?inverse }=
={ zC3 -z     ?inverse }=

' zasinh is func   ' gasinh is gauge
set-rft-mode-mrel
={ 0+i0       ?gauge }=
={ zsh1       ?gauge }=
={ zsh1 -z    ?gauge }=
={ 0+i1       ?gauge }=
={ 0+i1 conjg ?gauge }=
={ zC3        ?gauge }=
={ zC4        ?gauge }=
={ zC4 -z     ?gauge }=
={ zC3 -z     ?gauge }=

' gacosh is inverse   ' zcosh  is func
set-rft-mode-mrel
={ 1+i0 ?inverse }=
={ zch1 ?inverse }=
={ 0+i0 ?inverse }= \ Re(f^-1 f) = 6.123234E-17, Im((f^-1 f) = 4.440892E-16
={ zC1  ?inverse }=
={ zC2  ?inverse }=

' zacosh is func   ' gacosh is gauge   
set-rft-mode-mrel
={ 1+i0 ?gauge }=
={ zch1 ?gauge }=
={ 0+i0 ?gauge }= \ Re(g) = 2.220446E-16
={ zC1  ?gauge }=
={ zC2  ?gauge }=

' gatanh is inverse   ' ztanh is func
set-rft-mode-mrel
={ 0+i0       ?inverse }=
={ zth1       ?inverse }=
={ zth1 -z    ?inverse }=
={ 0+i1       ?inverse }=
={ 0+i1 conjg ?inverse }=
={ ztanhA     ?inverse }=
={ ztanhB     ?inverse }=
={ ztanhB -z  ?inverse }=
={ ztanhA -z  ?inverse }=

' zatanh is func   ' gatanh is gauge
set-rft-mode-mrel
={  0+i0       ?gauge }=
={  0+i1       ?gauge }=
={  1.0  1.0     ?gauge }=
={  1.0 -1.0     ?gauge }=
={ -1.0  1.0     ?gauge }=
={ -1.0 -1.0     ?gauge }=
={  0+i1 conjg ?gauge }=
={ zth1        ?gauge }=
={ zth1 -z     ?gauge }=
={ ztanhA      ?gauge }=
={ ztanhB      ?gauge }=
={ ztanhB -z   ?gauge }=
={ ztanhA -z   ?gauge }=

' gacoth is inverse   ' zcoth is func
set-rft-mode-mrel
={ zcth1      ?inverse }=
={ zcth1 -z   ?inverse }=
={ 0+i1 conjg  ?inverse }=
={ 0+i1        ?inverse }=
={ zcothA     ?inverse }=
={ zcothB     ?inverse }=
={ zcothB -z  ?inverse }=
={ zcothA -z  ?inverse }=

[DEFINED] zacoth [IF]
' zacoth is func   ' gacoth is gauge
set-rft-mode-mrel
={ zcth1      ?gauge }=
={ zcth1 -z   ?gauge }=
={ 0+i1 conjg ?gauge }=
={ 0+i1       ?gauge }=
={ zcothA     ?gauge }=
={ zcothB     ?gauge }=
={ zcothB -z  ?gauge }=
={ zcothA -z  ?gauge }=
[THEN]

testing  ZASIN ZACOS ZATAN ZACOT

\ Inverse trigonometric gauges.  GACOS is uncouth in the sense
\ of Corless, Davenport, Jeffrey, and Watt, i.e., not related to
\ the inverse hyperbolic counterpart by the naive identity.

: gacos   (rf: z -- -2iln[sqrt[[1+z]/2]+isqrt[[1-z]/2]] )
  zdup 1.0 x+ z2/ zsqrt   zswap -z 1.0 x+ z2/ zsqrt
  i* z+ zln z2* -i*  ;

: gasin   i*  gasinh -i* ;
: gatan   i*  gatanh -i* ;
: gacot   -i* gacoth -i* ;

\ We've checked that the inverse hyperbolic gauges are inverses,
\ so where they're couthly related, it's sufficient to check one
\ case of each inverse trigonometric gauge, where both input and
\ output are full complex numbers.  We checked by hand that zC1
\ works.

' gacos is inverse   ' zcos is func
set-rft-mode-mrel
={ zC1 ?inverse }=
={ zC2 ?inverse }=
={ zC3 ?inverse }=
={ zC4 ?inverse }=
={ zC1 gasin zsin => zC1 }=
={ zC1 gatan ztan => zC1 }=
={ zC1 gacot zcot => zC1 }=

' zacos is func
' gacos is gauge
\ stay off the cut |x| >= 1
set-rft-mode-mrel
={  +0.0   +0.0    ?gauge }= \ Im(g) = -4.440892E-16
={  pi/4  +0.0   ?gauge }= \ Im(g) = 2.220446E-16
={ -pi/4  +0.0   ?gauge }= \ Im(g) = 2.220446E-16
={  +0.0    pi/4 ?gauge }=
={  +0.0   -pi/4 ?gauge }=
={  +0.0    pi/2 ?gauge }=
={  +0.0   -pi/2 ?gauge }=
={  pi/2  pi/4 ?gauge }=
={  pi/2 -pi/4 ?gauge }=
={ -pi/2  pi/4 ?gauge }=
={ -pi/2 -pi/4 ?gauge }=

' zasin is func
' gasin is gauge
\ stay off the cut |x| >= 1
set-rft-mode-mrel
={  +0.0   +0.0    ?gauge }=
={  pi/4  +0.0   ?gauge }= \ Im(g) = -1.348379E-17
={ -pi/4  +0.0   ?gauge }= \ Im(g) = -1.348379E-17
={  +0.0    pi/4 ?gauge }=
={  +0.0   -pi/4 ?gauge }=
={  +0.0    pi/2 ?gauge }=
={  +0.0   -pi/2 ?gauge }=
={  pi/2  pi/4 ?gauge }=
={  pi/2 -pi/4 ?gauge }=
={ -pi/2  pi/4 ?gauge }=
={ -pi/2 -pi/4 ?gauge }=

' zatan is func
' gatan is gauge
\ stay off the cut |y| >= 1
set-rft-mode-mrel
={  +0.0   +0.0    ?gauge }=
={  pi/4  +0.0   ?gauge }=
={ -pi/4  +0.0   ?gauge }=
={  +0.0    pi/4 ?gauge }=
={  +0.0   -pi/4 ?gauge }=
={  pi/2  +0.0   ?gauge }=
={ -pi/2  +0.0   ?gauge }=
={  pi/2  pi/4 ?gauge }=
={  pi/2 -pi/4 ?gauge }=
={ -pi/2  pi/4 ?gauge }=
={ -pi/2 -pi/4 ?gauge }=

[DEFINED] zacot [IF]
' zacot is func
' gacot is gauge
\ stay off the cut |y| <= 1
set-rft-mode-mrel
={  pi/2  +0.0   ?gauge }=
={ -pi/2  +0.0   ?gauge }=
={  +0.0    pi/2 ?gauge }=
={  +0.0   -pi/2 ?gauge }=
={  pi    +0.0   ?gauge }=
={ -pi    +0.0   ?gauge }=
={  pi/4  pi/2 ?gauge }=
={  pi/4 -pi/2 ?gauge }=
={ -pi/4  pi/2 ?gauge }=
={ -pi/4 -pi/2 ?gauge }=
[THEN]

[THEN] \ inverse functions

\ free rf-constants free-zconstants

?.errors GFORTH-HOST [IF] ?.cr [THEN]
