(    Title:  Tests for the GNU Multiple Precision word set
      File:  gmpfr-test.fs
    Author:  David N. Williams
   Version:  0.8.6
   License:  Public Domain
  Log file:  gmpfr.log
   Started:  February 2, 2011
   Revised:  May 12, 2011

WARNING:  These tests often clear mfspace and rfspace, which
could clobber any nonconstant values stored by the user in the
mf and rf variables MFT-ABS-ERROR, MFT-REL-ERROR, RFT-ABS-ERROR,
and RFT-REL-ERROR.

Note that the binding of b@, b!, q@, q!, etc., to the gcrstack
library is tested implicitly by their use in btester, qtester,
etc.
)
decimal

[UNDEFINED] \\ [IF]
  : \\  ( -- )  -1 parse 2drop BEGIN refill 0= UNTIL ;
[THEN]
: .#  ( n -- )  cr ." #" . cr ;

: PROVIDE ( "<flag-constant>" -- ) true constant immediate ;

PROVIDE MULTI-INTEGERS
PROVIDE MULTI-RATIONALS
PROVIDE MULTI-FLOATS
PROVIDE RELIABLE-FLOATS
PROVIDE RANDOMS

s" gmpfr.fs"	       INCLUDED
s" ttester-xf.fs"      INCLUDED
s" tester-display.fs"  INCLUDED
s" xtester-errors.fs"  INCLUDED
s" ftester-errors.fs"  INCLUDED
MULTI-INTEGERS [IF]
s" btester.fs"	       INCLUDED
s" btester-errors.fs"  INCLUDED
[THEN]
MULTI-RATIONALS [IF]
s" qtester.fs"         INCLUDED
s" qtester-errors.fs"  INCLUDED
[THEN]
MULTI-FLOATS [IF]
s" mftester.fs"        INCLUDED
s" mftester-errors.fs" INCLUDED
[THEN]
RELIABLE-FLOATS [IF]
s" rftester.fs"        INCLUDED
s" rftester-errors.fs" INCLUDED
[THEN]
true VERBOSE !

[UNDEFINED] bits/cell [IF]
s" ADDRESS-UNIT-BITS" environment?
  0= [IF] cr .( ***Can't determine BITS/CELL) ABORT [THEN]
  1 cells * constant BITS/CELL [THEN]

BITS/CELL 32 = BITS/CELL 64 = or
0= [IF] cr .( *** BITS/CELL must be 32 or 64) ABORT [THEN]

s" MAX-N" environment?
0= [IF] cr .( ***Can't determine MAX-N) ABORT [THEN]
constant MAXN

s" MAX-U" environment?
0= [IF] cr .( ***Can't determine MAX-U) ABORT [THEN]
constant MAXU

-1 MAXU <>
[IF] cr .( ***System does not use two's complement) ABORT [THEN]

: ?.errors
  ?.xt-errors s"   " ?type
  ?.ft-errors s"   " ?type ?.cr
  MULTI-INTEGERS  [IF] ?.bt-errors  s"   " ?type [THEN]
  MULTI-RATIONALS [IF] ?.qt-errors  s"   " ?type [THEN]
  MULTI-FLOATS    [IF] ?.mft-errors s"   " ?type [THEN]
  RELIABLE-FLOATS [IF] ?.rft-errors [THEN]
;

: ={
  MULTI-INTEGERS  [IF] b{  [THEN]
  MULTI-RATIONALS [IF] q{  [THEN]
  MULTI-FLOATS    [IF] mf{ [THEN]
  RELIABLE-FLOATS [IF] rf{ [THEN] t{ ;

: =>
  MULTI-INTEGERS  [IF] b->  [THEN]
  MULTI-RATIONALS [IF] q->  [THEN]
  MULTI-FLOATS    [IF] mf-> [THEN]
  RELIABLE-FLOATS [IF] rf-> [THEN] -> ;

: }=
  MULTI-INTEGERS  [IF] }b  [THEN]
  MULTI-RATIONALS [IF] }q  [THEN]
  MULTI-FLOATS    [IF] }mf [THEN]
  RELIABLE-FLOATS [IF] }rf [THEN] }t ;

\ before the first displayed line
PFE-HOST IFORTH-HOST or [IF] ?.cr [THEN]

\ libgmp bindings always loaded
?." bits/limb: " mp_bits_per_limb ?. ?.cr


\ *** GENERAL USE

[UNDEFINED] S= [IF] : S=  ( s -- flag)  compare 0= ; [THEN] 

: MZSTR-s= ( s -- flag ) MZBUF mcount s= ;
: MZSTR.   ( -- )        MZBUF mcount type ;

variable texp   0 texp !  \ used with all four types


MULTI-INTEGERS [IF]
\ *** INTEGERS

\ SETUP

bspace free throw
/MPZ 2 cells + 7 * CONSTANT /BTBUF
/BTBUF make-bspace

\ TROUBLE-SHOOTING WORDS

: .bstat  ( -- )
  VERBOSE @ IF
    ." bbuffer size: " bspace >/gbuf @ . ." bytes" cr
    ." bunused: " bspace gunused . ." bytes   " cr
    ." bdepth:  " bspace gdepth . cr
    base @ hex
    ." bbuffer: 0x" bspace >gbuf . cr
    ." bbreak:  0x" bspace >gbrkp @ . cr
    ." bsp:     0x" bspace >gsp @ . cr
    ." bsp0:    0x" bspace >gsp0 @ . cr
    bgarbage? IF ." There is some b garbage. "
    ELSE ." There is no b garbage. " THEN cr
    ." Garbage collection lock: "
    bgc-lock@ IF ." ON" ELSE ." OFF" THEN cr
    base ! THEN ;

: dump-bspace  ( -- )
  VERBOSE @ IF
    bspace bspace >/gbuf @ /gspace-header + cell+ ( #bytes) dump
    [ PFE-HOST ] [IF] cr [THEN]
    .bstat THEN ;

: dump-b  ( 'b -- )
  ." b: " bspace >/grec @ cell- bounds ?DO i @ . 1 cells +LOOP cr ;

: bsp@    ( --  bsp ) bspace >gsp @ ;
: 0bspace ( -- )      bspace 0gspace ;  \ leaves bvars in limbo

\ TESTS

TESTING b" bs@ (bdrop) bdrop bdepth b. BCONSTANT BVARIABLE

: ?b.    (b: b -- ) VERBOSE @ IF b. ELSE bdrop THEN ;
: >mbstr ( 'b -- )  MZBUF cell+ base @ rot mpz_get_str zstrlen MZBUF ! ;

b"  0" bconstant  0.b
b"  1" bconstant  1.b   b"  -1" bconstant  -1.b
b"  2" bconstant  2.b   b"  -2" bconstant  -2.b
b"  3" bconstant  3.b   b"  -3" bconstant  -3.b
b"  4" bconstant  4.b   b"  -4" bconstant  -4.b 
b"  5" bconstant  5.b   b"  -5" bconstant  -5.b
b"  6" bconstant  6.b   b"  -6" bconstant  -6.b
b"  7" bconstant  7.b   b"  -7" bconstant  -7.b
b"  8" bconstant  8.b   b"  -8" bconstant  -8.b
b"  9" bconstant  9.b   b"  -9" bconstant  -9.b
b" 10" bconstant 10.b   b" -10" bconstant -10.b

0bspace
t{ b" 104" bdepth bs@ cell- @ ( blink) -> 1 bsp@ }t
t{ s" 104" MZSTR-s= -> true }t
t{ bs@ >mbstr s" 104" MZSTR-s= -> true }t
t{ bgarbage? bdepth BCONSTANT 104.b bgarbage? bdepth 104.b bdepth
   -> false 1 true 0 1 }t  \ copy grec to ext, garbage grec
?." You should see 104: " t{ ?b. bdepth -> 0 }t ?.cr

t{ : compiled-104.b  104.b ; -> }t
t{ compiled-104.b bs@ >mbstr s" 104" MZSTR-s= -> true }t

t{ 0.b bs@ >mbstr s" 0" MZSTR-s= -> true }t
t{ (bdrop) bdepth bdrop bdepth -> 1 0 }t

t{ BVARIABLE bvar bvar @ -> 0 }t

BITS/CELL 32 = hex
[IF]
  b"   7fffffff" bconstant MAXN.b
  b"   ffffffff" bconstant MAXU.b
  b" 1 fffffffe" bconstant 2*MAXU.b
[ELSE]
  b"   7fffffff ffffffff" bconstant MAXN.b
  b"   ffffffff ffffffff" bconstant MAXU.b
  b" 1 ffffffff fffffffe" bconstant 2*MAXU.b
[THEN]
decimal

BITS/CELL 32 = [IF]
t{   MAXN.b bs@ >mbstr s" 2147483647" MZSTR-s= bdrop -> true }t
t{ 2*MAXU.b bs@ >mbstr s" 8589934590" MZSTR-s= bdrop -> true }t
[ELSE]
t{   MAXN.b bs@ >mbstr  s" 9223372036854775807" MZSTR-s= bdrop -> true }t
t{ 2*MAXU.b bs@ >mbstr s" 36893488147419103230" MZSTR-s= bdrop -> true }t
[THEN]

TESTING >bs >bs-copy b,'b b, bs2@ bs3@ b2drop b3drop

: bdynam? ( 'b -- flag ) bspace is-grec ; 

={ 1.b bs@ dup >bs bdynam?      => 1.b 1.b false }=
={ 1.b bs@ >bs-copy bs@ bdynam? => 1.b 1.b true }=

={ 2.b align here b,'b dup >bs = => true 2.b }=

={ 1.b 2.b     bs2@ >bs >bs     => 1.b 2.b 2.b 1.b }=
={ 1.b 2.b 3.b bs3@ >bs >bs >bs => 1.b 2.b 3.b 3.b 2.b 1.b }=

={ 1.b 2.b         b2drop => }=
={ 1.b 2.b 3.b     b2drop => 1.b }=
={ 1.b 2.b 3.b     b3drop =>  }=
={ 1.b 2.b 3.b 2.b b3drop => 1.b }=

={ 1.b here dup b, b@ cell+ => 1.b here }=

TESTING bsgn b> b= b< b<= b>= b0> b0= b0< b0>= b0<=

={  2.b bsgn =>  1 }=
={  0.b bsgn =>  0 }=
={ -1.b bsgn => -1 }=

t{ 0.b bdepth -> 1 }t      \ bstack convervation check...
={ 1.b 2.b b> => false }=
t{ bdrop bdepth -> 0 }t   \ ...for ={ => }=
={ 2.b 1.b b> => true }=
={ 2.b 2.b b> => false }=

={ 1.b 2.b b= => false }=
={ 2.b 1.b b= => false }=
={ 2.b 2.b b= => true }=

={ 1.b 2.b b< => true }=
={ 2.b 1.b b< => false }=
={ 2.b 2.b b< => false }=

={ 1.b 2.b b<= => true }=
={ 2.b 1.b b<= => false }=
={ 2.b 2.b b<= => true }=

={ 1.b 2.b b>= => false }=
={ 2.b 1.b b>= => true }=
={ 2.b 2.b b>= => true }=

={  1.b b0> => true }=
={  0.b b0> => false }=
={ -1.b b0> => false }=

={  1.b b0= => false }=
={  0.b b0= => true }=
={ -1.b b0= => false }=

={  1.b b0< => false }=
={  0.b b0< => false }=
={ -1.b b0< => true }=

={  1.b b0>= => true }=
={  0.b b0>= => true }=
={ -1.b b0>= => false }=

={  1.b b0<= => false }=
={  0.b b0<= => true }=
={ -1.b b0<= => true }=

TESTING bn> bn= bn< bn>= bn<=

={  1.b 2 bn> => false }=
={  2.b 1 bn> => true }=
={  2.b 2 bn> => false }=

={  1.b 2 bn= => false }=
={  2.b 1 bn= => false }=
={  2.b 2 bn= => true }=

={  1.b 2 bn< => true }=
={  2.b 1 bn< => false }=
={  2.b 2 bn< => false }=

={  1.b 2 bn>= => false }=
={  2.b 1 bn>= => true }=
={  2.b 2 bn>= => true }=

={  1.b 2 bn<= => true }=
={  2.b 1 bn<= => false }=
={  2.b 2 bn<= => true }=

TESTING bu> bu= bu< bu>= bu<=

={  2.b -1 bu> => false }=
={  2.b  1 bu> => true }=
={  2.b  2 bu> => false }=

={  1.b 2 bu= => false }=
={  2.b 1 bu= => false }=
={  2.b 2 bu= => true }=

={  1.b  2 bu< => true }=
={  2.b  1 bu< => false }=
={  2.b -1 bu< => true }=

={  1.b 2 bu>= => false }=
={  2.b 1 bu>= => true }=
={  2.b 2 bu>= => true }=

={  1.b -1 bu<= => true }=
={  2.b  1 bu<= => false }=
={  2.b  2 bu<= => true }=

TESTING bdf> bdf= bdf< bdf>= bdf<=

={  1.b 2e bdf> => false }=
={  2.b 1e bdf> => true }=
={  2.b 2e bdf> => false }=

={  1.b 2e bdf= => false }=
={  2.b 1e bdf= => false }=
={  2.b 2e bdf= => true }=

={  1.b 2e bdf< => true }=
={  2.b 1e bdf< => false }=
={  2.b 2e bdf< => false }=

={  1.b 2e bdf>= => false }=
={  2.b 1e bdf>= => true }=
={  2.b 2e bdf>= => true }=

={  1.b 2e bdf<= => true }=
={  2.b 1e bdf<= => false }=
={  2.b 2e bdf<= => true }=

TESTING b|>| b|=| b|<| b|<=| b|>=|

={  1.b -2.b b|>| => false }=
={  2.b -1.b b|>| => true }=
={ -2.b  2.b b|>| => false }=

={  1.b  2.b b|=| => false }=
={ -2.b  2.b b|=| => true }=
={  2.b -2.b b|=| => true }=

={  1.b -2.b b|<| => true }=
={  2.b  1.b b|<| => false }=
={  2.b -2.b b|<| => false }=

={  1.b  2.b b|<=| => true }=
={ -2.b  1.b b|<=| => false }=
={  2.b -2.b b|<=| => true }=

={  1.b  2.b b|>=| => false }=
={  2.b  1.b b|>=| => true }=
={ -2.b  2.b b|>=| => true }=

TESTING bdf|>| bdf|=| bdf|<| bdf|<=| bdf|>=|

={  1.b -2e bdf|>| => false }=
={  2.b -1e bdf|>| => true }=
={ -2.b  2e bdf|>| => false }=

={  1.b  2e bdf|=| => false }=
={ -2.b  2e bdf|=| => true }=
={  2.b -2e bdf|=| => true }=

={  1.b -2e bdf|<| => true }=
={  2.b  1e bdf|<| => false }=
={  2.b -2e bdf|<| => false }=

={  1.b  2e bdf|<=| => true }=
={ -2.b  1e bdf|<=| => false }=
={  2.b -2e bdf|<=| => true }=

={  1.b  2e bdf|>=| => false }=
={  2.b  1e bdf|>=| => true }=
={ -2.b  2e bdf|>=| => true }=

TESTING bu|>| bu|=| bu|<| bu|<=| bu|>=|

\ assume two's complement
={  2.b -1 bu|>| => false }=
={  2.b  1 bu|>| => true }=
={  2.b  2 bu|>| => false }=

={  1.b  2 bu|=| => false }=
={  2.b  1 bu|=| => false }=
={ -2.b  2 bu|=| => true }=

={  1.b  2 bu|<| => true }=
={ -2.b  1 bu|<| => false }=
={ -2.b -1 bu|<| => true }=

={  1.b 2 bu|>=| => false }=
={ -2.b 1 bu|>=| => true }=
={  2.b 2 bu|>=| => true }=

={  1.b -1 bu|<=| => true }=
={  2.b  1 bu|<=| => false }=
={  2.b  2 bu|<=| => true }=

TESTING 0>ba 0>by 0>bz u>ba n>ba df>ba b>ba ba>b by>b bz>b ba.

={ 0>ba ba>b => 0.b }=
={ 0>by by>b => 0.b }=
={ 0>bz bz>b => 0.b }=
hex
={ FFFFFFFF u>ba ba>b => b" ffffffff" }=
={ -1       n>ba ba>b => b" -1" }=
={ -2  n>ba ba>b => b" -2" }=
decimal
={ 13e    df>ba ba>b => b" 13" }=
={  2.b   b>ba ba>b =>  2.b }=

: ?ba. ( -- ) VERBOSE @ IF ba. THEN ;

?." You should see 105: " ={ b" 105" b>ba ?ba. => }= ?.cr

TESTING ba>n  ba>u  ba>df  ba>df-exp

={ MAXN.b b>ba ba>n  => MAXN }=
={ MAXU.b b>ba ba>u  => MAXU }=
={ 10.b   b>ba ba>df => 10e }=
\ 10.0 = 0.625 x 2^4
={ 10.b b>ba texp ba>df-exp texp @ => 0.625e 4 }=

TESTING -ba |ba| ba+ by+ ba- by- bau+ bau-

={  3 n>ba -ba  ba>b => -3.b }=
={  3 n>ba |ba| ba>b =>  3.b }=
={ -2 n>ba |ba| ba>b =>  2.b }=

={ 0>ba 1.b ba+ ba>b =>  1.b }=
={ -2.b	    ba+ ba>b => -1.b }=
={ -3.b	    ba- ba>b =>  2.b }=
hex
={ FFFFFFFF bau+ ba>b => b" 1 0000 0001" }=
={ FFFFFFFF bau- ba>b => b" 2" }=
decimal

TESTING ba* ban* bau* b*a+ bu*a+ b*a- bu*a- ba2^u*

={  1 n>ba -2.b      ba*    ba>b => -2.b }=
={ -1                ban*   ba>b => 2.b }=
={ -1                bau*   ba>b => 2*MAXU.b }=
={ -1 n>ba  2.b  2.b b*a+   ba>b =>  3.b }=
={  1 n>ba -2.b  2   bu*a+  ba>b => -3.b }=
={  1 n>ba  2.b  2.b b*a-   ba>b => -3.b }=
={ -1 n>ba -2.b  2   bu*a-  ba>b =>  3.b }=
={  1 n>ba 2         ba2^u* ba>b =>  4.b }=

TESTING bac/ bacrem bac/rem bacu/ bacurem bacu/rem bac/urem ba2^uc/ ba2^ucrem
---
The floored and symmetric division examples in the table below
are taken from DPANS94 3.2.2.1:

  Ceiled             Floored             Symmetric         
  num  den quo rem    num  den quo rem    num  den quo rem  
   10   7   2  -4      10   7   1   3      10   7   1   3   
  -10   7  -1  -3     -10   7  -2   4     -10   7  -1  -3   
   10  -7  -1   3      10  -7  -2  -4      10  -7  -1   3   
  -10  -7   2   4     -10  -7   1  -3     -10  -7   1  -3   

Note that we only need the first two rows to distinguish calls
among the c [ceiled], f [floored], and s [symmetric] functions.
---

={ 10.b b>ba 7.b bac/          ba>b =>  2.b }=
={ 10.b b>ba 7.b bacrem        ba>b => -4.b }=
={ 10.b b>ba 7.b bac/rem ba>b  by>b =>  2.b -4.b }=
={ 10.b b>ba 7   bauc/         ba>b =>  2.b  4 }=
={ 10.b b>ba 7   baucrem       ba>b => -4.b  4 }=
={ 10.b b>ba 7   bauc/rem ba>b by>b =>  2.b -4.b 4 }=
={ 10.b b>ba 7   bacurem       ba>b => 10.b  4 }=  \ no change in x
\ 2^3 = 8
={ 10.b b>ba 3 ba2^uc/	 ba>b =>  2.b }=
={ 10.b b>ba 3 ba2^ucrem ba>b => -6.b }=

={ -10.b b>ba 7.b bac/          ba>b => -1.b }=
={ -10.b b>ba 7.b bacrem        ba>b => -3.b }=
={ -10.b b>ba 7.b bac/rem ba>b  by>b => -1.b -3.b }=
={ -10.b b>ba 7   bauc/         ba>b => -1.b  3 }=
={ -10.b b>ba 7   baucrem       ba>b => -3.b  3 }=
={ -10.b b>ba 7   bauc/rem ba>b by>b => -1.b -3.b 3 }=
={ -10.b b>ba 7   bacurem       ba>b => -10.b 3 }=  \ no change in x
\ 2^3 = 8
={ -10.b b>ba 3 ba2^uc/	  ba>b => -1.b }=
={ -10.b b>ba 3 ba2^ucrem ba>b => -2.b }=

TESTING baf/ bafrem baf/rem bafu/ bafurem bafu/rem baf/urem ba2^uf/ ba2^ufrem

={ 10.b b>ba 7.b baf/          ba>b => 1.b }=
={ 10.b b>ba 7.b bafrem        ba>b => 3.b }=
={ 10.b b>ba 7.b baf/rem ba>b  by>b => 1.b 3.b }=
={ 10.b b>ba 7   bauf/         ba>b => 1.b 3 }=
={ 10.b b>ba 7   baufrem       ba>b => 3.b 3 }=
={ 10.b b>ba 7   bauf/rem ba>b by>b => 1.b 3.b 3 }=
={ 10.b b>ba 7   bafurem       ba>b => 10.b 3 }=  \ no change in x
\ 2^3 = 8
={ 10.b b>ba 3 ba2^uf/	 ba>b => 1.b }=
={ 10.b b>ba 3 ba2^ufrem ba>b => 2.b }=

={ -10.b b>ba 7.b baf/          ba>b =>  -2.b }=
={ -10.b b>ba 7.b bafrem        ba>b =>   4.b }=
={ -10.b b>ba 7.b baf/rem ba>b  by>b =>  -2.b 4.b }=
={ -10.b b>ba 7   bauf/         ba>b =>  -2.b 4 }=
={ -10.b b>ba 7   baufrem       ba>b =>   4.b 4 }=
={ -10.b b>ba 7   bauf/rem ba>b by>b =>  -2.b 4.b 4 }=
={ -10.b b>ba 7   bafurem       ba>b => -10.b 4 }=  \ no change in x
\ 2^3 = 8
={ -10.b b>ba 3 ba2^uf/	  ba>b => -2.b }=
={ -10.b b>ba 3 ba2^ufrem ba>b =>  6.b }=

TESTING bas/ basrem bas/rem basu/ basurem basu/rem bas/urem ba2^us/ ba2^usrem

={ 10.b b>ba 7.b bas/          ba>b => 1.b }=
={ 10.b b>ba 7.b basrem        ba>b => 3.b }=
={ 10.b b>ba 7.b bas/rem ba>b  by>b => 1.b 3.b }=
={ 10.b b>ba 7   baus/         ba>b => 1.b 3 }=
={ 10.b b>ba 7   bausrem       ba>b => 3.b 3 }=
={ 10.b b>ba 7   baus/rem ba>b by>b => 1.b 3.b 3 }=
={ 10.b b>ba 7   basurem       ba>b => 10.b 3 }=  \ no change in x
\ 2^3 = 8
={ 10.b b>ba 3 ba2^us/	 ba>b => 1.b }=
={ 10.b b>ba 3 ba2^usrem ba>b => 2.b }=

={ -10.b b>ba 7.b bas/          ba>b =>  -1.b }=
={ -10.b b>ba 7.b basrem        ba>b =>  -3.b }=
={ -10.b b>ba 7.b bas/rem ba>b by>b  =>  -1.b -3.b }=
={ -10.b b>ba 7   baus/         ba>b =>  -1.b  3 }=
={ -10.b b>ba 7   bausrem       ba>b =>  -3.b  3 }=
={ -10.b b>ba 7   baus/rem ba>b by>b =>  -1.b -3.b 3 }=
={ -10.b b>ba 7   basurem       ba>b => -10.b  3 }=  \ no change in x
\ 2^3 = 8
={ -10.b b>ba 3 ba2^us/	  ba>b => -1.b }=
={ -10.b b>ba 3 ba2^usrem ba>b => -2.b }=

TESTING bamod baumod ba/exact bau/exact

={  10.b b>ba 7.b bamod ba>b  => 3.b }=
={ -10.b b>ba 7.b bamod ba>b  => 4.b }=
={  10.b b>ba 7   baumod ba>b => 3.b 3 }=
={ -10.b b>ba 7   baumod ba>b => 4.b 4 }=

={  10.b b>ba  2.b ba/exact ba>b  =>  5.b }=
={ -10.b b>ba  2.b ba/exact ba>b  => -5.b }=
={  10.b b>ba -2.b ba/exact ba>b  => -5.b }=
={  10.b b>ba  2   bau/exact ba>b =>  5.b }=
={ -10.b b>ba  2   bau/exact ba>b => -5.b }=

TESTING ba-pow-mod ba-pow-mod-sec ba^u-mod ba^u u^u>ba

={  2.b b>ba 3.b 7.b ba-pow-mod ba>b     => 1.b }=
={  2.b b>ba 5.b 7.b ba-pow-mod ba>b     => 4.b }=
={  2.b b>ba 3.b 7.b ba-pow-mod-sec ba>b => 1.b }=
={  2.b b>ba 5.b 7.b ba-pow-mod-sec ba>b => 4.b }=
={  2.b b>ba 3   7.b ba^u-mod ba>b       => 1.b }=
={  2.b b>ba 5   7.b ba^u-mod ba>b       => 4.b }=

={ 2.b b>ba 3 ba^u   ba>b => 8.b }=
={ 2        3 u^u>ba ba>b => 8.b }=

TESTING ba-nroot ba-nroot-rem ba-sqrt ba-sqrt-rem

={  8.b b>ba 3 ba-nroot     ba>b      => 2.b true }=
={ 10.b b>ba 3 ba-nroot     ba>b      => 2.b false }=
={  8.b b>ba 3 ba-nroot-rem ba>b by>b => 2.b 0.b }=
={ 10.b b>ba 3 ba-nroot-rem ba>b by>b => 2.b 2.b }=
={  9.b b>ba   ba-sqrt      ba>b      => 3.b }=
={ 10.b b>ba   ba-sqrt-rem  ba>b by>b => 3.b 1.b }=

TESTING ba-next-prime ba-gcd bau-gcd ba-gcd-st ba-gcd-s ba-lcm  bau-lcm 

={ b" 258" b>ba        ba-next-prime ba>b           => b" 263" }=
={  b" 20" b>ba b" 12" ba-gcd        ba>b           => 4.b }=
={  b" 20" b>ba    12  bau-gcd       ba>b           => 4 4.b }=
={  b" 20" b>ba b" 12" ba-gcd-st     by>b bz>b ba>b => -1.b 2.b 4.b }=
={  b" 20" b>ba b" 12" ba-gcd-s      by>b ba>b      => -1.b 4.b }=
={  b" 20" b>ba b" 12" ba-lcm        ba>b           => b" 60" }=
={  b" 20" b>ba    12  bau-lcm       ba>b           => b" 60" }=

TESTING ba-mod-invert ba-remove fac>ba ba-bin bin>ba

\ [(7 mod 4) x 3] mod 4 = 1
={     7.b b>ba 4.b ba-mod-invert ba>b => true 3.b }=
={ b" 20"  b>ba 2.b ba-remove ba>b => 2 5.b }=
={     4            fac>ba    ba>b => b" 24" }=
={   10.b  b>ba 3   ba-bin    ba>b => b" 120" }=
={   10         3   bin>ba    ba>b => b" 120" }=

TESTING fib>ba fib2>ba luc>ba luc2>ba 

={ 6 fib>ba  ba>b      => 8.b }=
={ 6 fib2>ba by>b ba>b => 5.b 8.b }=
={ 6 luc>ba  ba>b      => b" 18" }=
={ 6 luc2>ba by>b ba>b => b" 11" b" 18" }=

TESTING ba-and ba-or ba-xor ba-invert

hex
={ b" 0f0f" b>ba  b" f00" ba-and    ba>b => b" f00" }=
={ b" 0f0f" b>ba b" f000" ba-or     ba>b => b" ff0f" }=
={ b" 0f0f" b>ba  b" f00" ba-xor    ba>b => b" f" }=
={      1.b b>ba          ba-invert ba>b => -2.b }=
={   MAXU.b b>ba          ba-invert ba>b => MAXU.b 1 bu+ -b }=
decimal

TESTING ba-bit-on ba-bit-off ba-bit-flip ba-bit-test

hex
={ b" 0f0f" b>ba 4 ba-bit-on   ba>b => b" f1f" }=
={ b" 0f0f" b>ba 3 ba-bit-off  ba>b => b" f07" }=
={ b" 0f0f" b>ba 3 ba-bit-flip ba>b => b" f07" }=
={ b" 0f07" b>ba 3 ba-bit-flip ba>b => b" f0f" }=
={ b" 0f0f" b>ba 3 ba-bit-test      => true }=
={ b" 0f0f" b>ba 4 ba-bit-test      => false }=
decimal

TESTING 0>b n>b u>b df>b b>n b>u b>df b>df-exp

={ 	0>b => 0.b }=
={ MAXN n>b => MAXN.b }=
={ MAXU u>b => MAXU.b }=
={ 10e df>b => 10.b }=

={ MAXN.b b>n => MAXN }=
={ MAXU.b b>u => MAXU }=
={ 10.b   b>df => 10e }=
\ 10.0 = 0.625 x 2^4
={ 10.b texp b>df-exp texp @ => 0.625e 4 }=

TESTING -b |b| b+ b- bu+ bu- ub- b* bn* bu*

={  2.b     -b  => -2.b }=
={ -2.b     |b| =>  2.b }=
={ 1.b 2.b  b+  =>  3.b }=
={ 1.b 2.b  b-  => -1.b }=
={ 1.b 2    bu+ =>  3.b }=
={ 1.b 2    bu- => -1.b }=
={ 1   2.b  ub- => -1.b }=
={ 2.b 2.b  b*  =>  4.b }=
={ 2.b -2   bn* => -4.b }=
={ 2.b MAXU bu* => 2*MAXU.b }=

TESTING b*+ b*- bu*+ bu*- b2^u*

={ -1.b  2.b  2.b b*+   =>  3.b }=
={  1.b -2.b  2   bu*+  => -3.b }=
={  1.b  2.b  2.b b*-   => -3.b }=
={ -1.b -2.b  2   bu*-  =>  3.b }=
={ -1.b  2        b2^u* => -4.b }=

TESTING bc/ bcrem bc/rem bcu/ bcurem bcu/rem bc/urem b2^uc/ b2^ucrem

={ 10.b 7.b bc/     =>  2.b }=
={ 10.b 7.b bcrem   => -4.b }=
={ 10.b 7.b bc/rem  =>  2.b -4.b }=
={ 10.b 7   buc/    =>  2.b  4 }=
={ 10.b 7   bucrem  => -4.b  4 }=
={ 10.b 7   buc/rem =>  2.b -4.b 4 }=
={ 10.b 7   bcurem  =>  4 }=
\ 2^3 = 8
={ 10.b 3 b2^uc/   =>  2.b }=
={ 10.b 3 b2^ucrem => -6.b }=

={ -10.b 7.b bc/     => -1.b }=
={ -10.b 7.b bcrem   => -3.b }=
={ -10.b 7.b bc/rem  => -1.b -3.b }=
={ -10.b 7   buc/    => -1.b  3 }=
={ -10.b 7   bucrem  => -3.b  3 }=
={ -10.b 7   buc/rem => -1.b -3.b 3 }=
={ -10.b 7   bcurem  =>  3 }=
\ 2^3 = 8
={ -10.b 3 b2^uc/    => -1.b }=
={ -10.b 3 b2^ucrem  => -2.b }=

TESTING bf/ bfrem bf/rem bfu/ bfurem bfu/rem bf/urem b2^uf/ b2^ufrem

={ 10.b 7.b bf/     => 1.b }=
={ 10.b 7.b bfrem   => 3.b }=
={ 10.b 7.b bf/rem  => 1.b 3.b }=
={ 10.b 7   buf/    => 1.b 3 }=
={ 10.b 7   bufrem  => 3.b 3 }=
={ 10.b 7   buf/rem => 1.b 3.b 3 }=
={ 10.b 7   bfurem  => 3 }=
\ 2^3 = 8
={ 10.b 3 b2^uf/    => 1.b }=
={ 10.b 3 b2^ufrem  => 2.b }=

={ -10.b 7.b bf/     => -2.b }=
={ -10.b 7.b bfrem   =>  4.b }=
={ -10.b 7.b bf/rem  => -2.b 4.b }=
={ -10.b 7   buf/    => -2.b 4 }=
={ -10.b 7   bufrem  =>  4.b 4 }=
={ -10.b 7   buf/rem => -2.b 4.b 4 }=
={ -10.b 7   bfurem  =>  4 }=
\ 2^3 = 8
={ -10.b 3 b2^uf/    => -2.b }=
={ -10.b 3 b2^ufrem  =>  6.b }=

TESTING bs/ bsrem bs/rem bsu/ bsurem bsu/rem bs/urem b2^us/ b2^usrem

={ 10.b 7.b bs/     => 1.b }=
={ 10.b 7.b bsrem   => 3.b }=
={ 10.b 7.b bs/rem  => 1.b 3.b }=
={ 10.b 7   bus/    => 1.b 3 }=
={ 10.b 7   busrem  => 3.b 3 }=
={ 10.b 7   bus/rem => 1.b 3.b 3 }=
={ 10.b 7   bsurem  => 3 }=
\ 2^3 = 8
={ 10.b 3 b2^us/   => 1.b }=
={ 10.b 3 b2^usrem => 2.b }=

={ -10.b 7.b bs/     => -1.b }=
={ -10.b 7.b bsrem   => -3.b }=
={ -10.b 7.b bs/rem  => -1.b -3.b }=
={ -10.b 7   bus/    => -1.b  3 }=
={ -10.b 7   busrem  => -3.b  3 }=
={ -10.b 7   bus/rem => -1.b -3.b 3 }=
={ -10.b 7   bsurem  =>  3 }=
\ 2^3 = 8
={ -10.b 3   b2^us/   => -1.b }=
={ -10.b 3   b2^usrem => -2.b }=

TESTING bmod bumod b/exact bu/exact

={  10.b 7.b bmod  => 3.b }=
={ -10.b 7.b bmod  => 4.b }=
={  10.b 7   bumod => 3.b 3 }=
={ -10.b 7   bumod => 4.b 4 }=

TESTING b-divisible bu-divisible b2^u-divisible

={ 10.b 2.b b-divisible     => true }=
={  7.b 2.b b-divisible     => false }=
={ 10.b 2   bu-divisible   => true }=
={  7.b 2   bu-divisible   => false }=
={  4.b 2   b2^u-divisible => true }=
={  7.b 2   b2^u-divisible => false }=

TESTING b-congruent bu-congruent b2^u-congruent

={  7.b 3.b 4.b b-congruent     => true }=
={ 10.b 3.b 4.b b-congruent     => false }=
={  7.b 3   4   bu-congruent   => true }=
={ 10.b 3   4   bu-congruent   => false }=
={  7.b 3.b 2   b2^u-congruent => true }=
={ 10.b 3.b 2   b2^u-congruent => false }=

TESTING b-pow-mod b-pow-mod-sec b^u-mod b^u u^u>b

={  2.b 3.b 7.b b-pow-mod     => 1.b }=
={  2.b 5.b 7.b b-pow-mod     => 4.b }=
={  2.b 3.b 7.b b-pow-mod-sec => 1.b }=
={  2.b 5.b 7.b b-pow-mod-sec => 4.b }=
={  2.b 3   7.b b^u-mod       => 1.b }=
={  2.b 5   7.b b^u-mod       => 4.b }=

={ 2.b 3 b^u   => 8.b }=
={ 2   3 u^u>b => 8.b }=

TESTING b-nroot b-nroot-rem b-sqrt b-sqrt-rem

={  8.b 3 b-nroot     => 2.b true }=
={ 10.b 3 b-nroot     => 2.b false }=
={  8.b 3 b-nroot-rem => 2.b 0.b }=
={ 10.b 3 b-nroot-rem => 2.b 2.b }=
={  9.b   b-sqrt      => 3.b }=
={ 10.b   b-sqrt-rem  => 3.b 1.b }=

TESTING b-perfect-pow b-perfect-squ

={  8.b b-perfect-pow => true }=
={ 10.b b-perfect-pow => false }=
={  8.b b-perfect-squ => false }=
={  9.b b-perfect-squ => true }=

TESTING b-prob-prime b-next-prime b-gcd bu-gcd>u bu-gcd b-gcd-st b-gcd-s

={ b" 258"     3  b-prob-prime => 0 }=
={ b" 263"     3  b-prob-prime => 2 }=
={ b" 258"        b-next-prime => b" 263" }=
={  b" 20" b" 12" b-gcd        =>  4.b }=
={  b" 20"    12  bu-gcd>u    => 4  }=
={  b" 20"    12  bu-gcd      => 4 4.b }=
={  b" 20" b" 12" b-gcd-st     => -1.b 2.b 4.b }=
={  b" 20" b" 12" b-gcd-s      => -1.b 4.b }=

TESTING b-lcm bu-lcm b-mod-invert b-jacobi b-legendre b-kronecker  
---
It's important to test returns of -1 for the jacobi, legendre,
and kronecker symbols, to be sure that's properly extended from
and int to a cell with 64-bit systems.  It's sufficient to do
that for one of b-jacobi, b-legendre, or b-kronecker, because
mpz_jacobi, mpz_legendre, and mpz_kronecker are defined to be
identical in ligbmp 5.0.1, and for the n and u variants of
kronecker, because there no such variants for jacobi or
legendre.
---

={  b" 20" b" 12" b-lcm        => b" 60" }=
={  b" 20"    12  bu-lcm      => b" 60" }=
={     7.b    4.b b-mod-invert => true 3.b }=

\ 20 mod 5 = 0
={ b" 20" b" 15" b-jacobi => 0 }=

\ 100 = (2x5)^2 <> 0 mod 3
={ b" 100" 3.b b-legendre => 1 }=

\ 101 <> x^2 <> 0 mod 3
={ b" 101" 3.b b-legendre => -1 }=

\ 101 <> x^2 <> 0 mod 7
={ b" 101" 7.b b-legendre => -1 }=

\ 21 = 3 x 7
={ b" 101" b" 21" b-jacobi => -1 -1 * }=

\ gmp aliases kro(a,b) to jac(a,b), at least when b <> 2 
={  b" 20" b" 15" b-kronecker => 0 }=
={ b" 101" b" 21" b-kronecker => -1 -1 * }=

TESTING bn-kronecker bu-kronecker nb-kronecker ub-kronecker

\ distinction between u and n not really tested

={ b"  -20" 15 bn-kronecker => 0 }=
={ b" -101" 21 bn-kronecker => -1 -1 * }=
={ b"  101"  3 bn-kronecker => -1 }=
={ b"   20" 15 bu-kronecker => 0 }=
={ b"  101" 21 bu-kronecker => -1 -1 * }=
={ b"  101"  3 bu-kronecker => -1 }=

={  -20 b" 15" nb-kronecker => 0 }=
={ -101 b" 21" nb-kronecker => -1 -1 * }=
={  101 b"  3" nb-kronecker => -1 }=
={   20 b" 15" ub-kronecker => 0 }=
={  101 b" 21" ub-kronecker => -1 -1 * }=
={  101 b"  3" ub-kronecker => -1 }=

TESTING b-remove fac>b bu-bin bin>b fib>b fib2>b luc>b luc2>b

\ [(7 mod 4) x 3] mod 4 = 1
={ b" 20"  2.b b-remove => 2 5.b }=
={     4       fac>b   => b" 24" }=
={    10.b 3   bu-bin  => b" 120" }=
={    10   3   bin>b   => b" 120" }=

={ 6 fib>b  => 8.b }=
={ 6 fib2>b => 5.b 8.b }=
={ 6 luc>b  => b" 18" }=
={ 6 luc2>b => b" 11" b" 18" }=

TESTING b-and b-or b-xor b-invert

hex
={ b" 0f0f"  b" f00" b-and    =>  b" f00" }=
={ b" 0f0f" b" f000" b-or     => b" ff0f" }=
={ b" 0f0f"  b" f00" b-xor    =>    b" f" }=
={     1.b           b-invert => -2.b }=
={  MAXU.b           b-invert => MAXU.b 1 bu+ -b }=
decimal

TESTING b-pop-count b-ham-dist b-scan0 b-scan1

hex
={ b" 0f0f"          b-pop-count =>  8 }=
={ b" 0f0f" b" 0e0e" b-ham-dist  =>  2 }=
={ b" 0f0f"       3  b-scan0     =>  4 }=
={ b" 0f0f"       4  b-scan1     =>  8 }=
decimal

TESTING b-bit-on b-bit-off b-bit-flip b-bit-test

hex
={ b" 0f0f" 4 b-bit-on   => b" f1f" }=
={ b" 0f0f" 3 b-bit-off  => b" f07" }=
={ b" 0f0f" 3 b-bit-flip => b" f07" }=
={ b" 0f07" 3 b-bit-flip => b" f0f" }=
={ b" 0f0f" 3 b-bit-test => true }=
={ b" 0f0f" 4 b-bit-test => false }=
decimal

\ TESTING  bgarbage? collect-bgarbage

\ ={ bgarbage? collect-bgarbage bgarbage? => true true false }=
[THEN]  \ MULTI-INTEGERS


MULTI-RATIONALS [IF]
\ *** RATIONALS

\ SETUP

qspace free throw  \ release the default space
/MPQ 2 cells + 7 * CONSTANT /QTBUF
/QTBUF make-qspace

\ TROUBLE-SHOOTING WORDS

: .qstat  ( -- )
  VERBOSE @ IF
    ." qbuffer size: " qspace >/gbuf @ . ." bytes" cr
    ." qunused: " qspace gunused . ." bytes   " cr
    ." qdepth:  " qspace gdepth . cr
    base @ hex
    ." qbuffer: 0x" qspace >gbuf . cr
    ." qbreak:  0x" qspace >gbrkp @ . cr
    ." qsp:     0x" qspace >gsp @ . cr
    ." qsp0:    0x" qspace >gsp0 @ . cr
    decimal
    qgarbage? IF ." There is some q garbage. "
    ELSE ." There is no q garbage. " THEN cr
    ." Garbage collection lock: "
    qgc-lock@ IF ." ON" ELSE ." OFF" THEN cr
    base ! THEN ;

: dump-qspace  ( -- )
  VERBOSE @ IF
    qspace dup >/gbuf @ /gspace-header + cell+ ( #bytes) dump
    [ PFE-HOST ] [IF] cr [THEN]
    .qstat THEN ;

: dump-q  ( 'f -- )
  ." q: " qspace >/grec @ cell-  bounds ?DO i @ . 1 cells +LOOP cr ;

: qsp@    ( -- qsp ) qspace >gsp @ ;
: 0qspace ( -- )     qspace 0gspace ;

\ TESTS

TESTING q" qs@ (qdrop) qdrop qdepth q. QCONSTANT QVARIABLE

: ?q.    (q: q -- ) VERBOSE @ IF q. ELSE qdrop THEN ;
: >mqstr ( 'b -- )  MZBUF cell+ base @ rot mpq_get_str zstrlen MZBUF ! ;

q" 0"   qconstant 0/1
q" 1/2" qconstant 1/2   q" -1/2" qconstant -1/2
q" 2/3" qconstant 2/3   q" -2/3" qconstant -2/3
q" 3/4" qconstant 3/4   q" -3/4" qconstant -3/4

0qspace
t{ q" 104/3" qdepth qs@ cell- @ ( blink) -> 1 qsp@ }t
t{ s" 104/3" MZSTR-s= -> true }t
t{ qs@ >mqstr s" 104/3" MZSTR-s= -> true }t
t{ qgarbage? qdepth QCONSTANT 104/3 qgarbage? qdepth 104/3 qdepth
   -> false 1 true 0 1 }t  \ copy grec to ext, garbage grec
?." You should see 104/3: " t{ ?q. qdepth -> 0 }t ?.cr

t{ : compiled-104/3  104/3 ; -> }t
t{ compiled-104/3 qs@ >mqstr s" 104/3" MZSTR-s= -> true }t

t{ 0/1 qs@ >mqstr s" 0" MZSTR-s= -> true }t
t{ (qdrop) qdepth qdrop qdepth -> 1 0 }t

t{ QVARIABLE qvar qvar @ -> 0 }t

t{ 1/2 qs@ >mqstr s" 1/2" MZSTR-s= qdrop -> true }t

TESTING >qs >qs-copy q,'q qs2@ qs3@ q2drop q3drop

: qdynam? ( 'q -- flag ) qspace is-grec ; 

={ 1/2 qs@ dup >qs qdynam?      => 1/2 1/2 false }=
={ 1/2 qs@ >qs-copy qs@ qdynam? => 1/2 1/2 true }=

={ 2/3 align here q,'q dup >qs = => true 2/3 }=

={ 1/2 2/3     qs2@ >qs >qs     => 1/2 2/3 2/3 1/2 }=
={ 1/2 2/3 3/4 qs3@ >qs >qs >qs => 1/2 2/3 3/4 3/4 2/3 1/2 }=

={ 1/2 2/3         q2drop => }=
={ 1/2 2/3 3/4     q2drop => 1/2 }=
={ 1/2 2/3 3/4     q3drop =>  }=
={ 1/2 2/3 3/4 2/3 q3drop => 1/2 }=

={ 3/4 here dup q, q@ cell+ => 3/4 here }=

TESTING qsgn q> q= q< q<= q>= q0> q0= q0< q0>= q0<=

={  2/3 qsgn =>  1 }=
={  0/1 qsgn =>  0 }=
={ -1/2 qsgn => -1 }=

t{ 0/1 qdepth -> 1 }t     \ qstack convervation check...
={ 1/2 2/3 q> => false }=
t{ qdrop qdepth -> 0 }t   \ ...for ={ => }=
={ 2/3 1/2 q> => true }=
={ 2/3 2/3 q> => false }=

={ 1/2 2/3 q= => false }=
={ 2/3 1/2 q= => false }=
={ 2/3 2/3 q= => true }=

={ 1/2 2/3 q< => true }=
={ 2/3 1/2 q< => false }=
={ 2/3 2/3 q< => false }=

={ 1/2 2/3 q<= => true }=
={ 2/3 1/2 q<= => false }=
={ 2/3 2/3 q<= => true }=

={ 1/2 2/3 q>= => false }=
={ 2/3 1/2 q>= => true }=
={ 2/3 2/3 q>= => true }=

={  1/2 q0> => true }=
={  0/1 q0> => false }=
={ -1/2 q0> => false }=

={  1/2 q0= => false }=
={  0/1 q0= => true }=
={ -1/2 q0= => false }=

={  1/2 q0< => false }=
={  0/1 q0< => false }=
={ -1/2 q0< => true }=

={  1/2 q0>= => true }=
={  0/1 q0>= => true }=
={ -1/2 q0>= => false }=

={  1/2 q0<= => false }=
={  0/1 q0<= => true }=
={ -1/2 q0<= => true }=

TESTING qnu> qnu= qnu< qnu>= qnu<=

={  1/2 2 3 qnu> => false }=
={  2/3 1 2 qnu> => true }=
={  2/3 2 3 qnu> => false }=

={  1/2 2 3 qnu= => false }=
={  2/3 1 3 qnu= => false }=
={  2/3 2 3 qnu= => true }=

={  1/2 2 3 qnu< => true }=
={  2/3 1 2 qnu< => false }=
={  2/3 2 3 qnu< => false }=

={  1/2 2 3 qnu>= => false }=
={  2/3 1 2 qnu>= => true }=
={  2/3 2 3 qnu>= => true }=

={  1/2 2 3 qnu<= => true }=
={  2/3 1 2 qnu<= => false }=
={  2/3 2 3 qnu<= => true }=

TESTING quu> quu= quu< quu>= quu<=

={  2/3 MAXU 2 quu> => false }=
={  2/3    1 2 quu> => true }=
={  2/3    2 3 quu> => false }=

={  1/2 2 3 quu= => false }=
={  2/3 1 2 quu= => false }=
={  2/3 2 3 quu= => true }=

={  1/2    2 3 quu< => true }=
={  2/3    1 2 quu< => false }=
={  2/3 MAXU 2 quu< => true }=

={  1/2 2 3 quu>= => false }=
={  2/3 1 2 quu>= => true }=
={  2/3 2 3 quu>= => true }=

={  1/2 MAXU 2 quu<= => true }=
={  2/3    1 2 quu<= => false }=
={  2/3    2 3 quu<= => true }=

TESTING 0>qa nu>qa uu>qa df>qa q>qa qa>q qa>df qacanon qa.

: ?qa. ( -- ) VERBOSE @ IF qa. THEN ; 

={ 0>qa qa>q => 0/1 }=
hex
={ -1       2 nu>qa qa>q => q" -1/2" }=
={ FFFFFFFF 2 uu>qa qa>q => q" ffffffff/2" }=
decimal
={ 2 3 nu>qa qa>q =>     2/3 }=
={ 13e df>qa qa>q => q" 13/1" }=
={ 2/3 q>qa qa>q  =>     2/3 }=
={ 1/2 q>qa qa>df =>    0.5e }=

={  12 16 nu>qa qacanon qa>q => 3/4 }=

?." You should see 105/3: " ={ 105 3 uu>qa ?qa. => }= ?.cr

TESTING -qa |qa| 1/qa qa+ qa- qa* qa/ qa2^u* qa2^u/

q" 4/3" qconstant 4/3

={  3 4 nu>qa -qa  qa>q => -3/4 }=
={  3 4 nu>qa |qa| qa>q =>  3/4 }=
={ -2 3 nu>qa |qa| qa>q =>  2/3 }=
={  3 4 nu>qa 1/qa qa>q =>  4/3 }=


={ 0>qa 1/2 qa+  qa>q =>  1/2 }=
={ -2/3	    qa+  qa>q => q" -1/6" }=
={ -3/4     qa-  qa>q => q"  7/12" }=

={  2 3 nu>qa 3/4 qa* qa>q => 1/2 }=
={  4 9 nu>qa 2/3 qa/ qa>q => 2/3 }=

={  2 3 nu>qa 3 qa2^u* qa>q => q" 16/3" }=
={  2 3 nu>qa 3 qa2^u/ qa>q => q" 1/12" }=

TESTING 0>q nu>q uu>q df>q q>df qcanon

={	      0>q  =>  q" 0" }=
={ -1       2 nu>q => q" -1/2" }=
hex
={ FFFFFFFF 2 uu>q => q"  ffffffff/2" }=
decimal
={ 2 3        nu>q =>     2/3 }=
={ 13e        df>q => q" 13/1" }=
={ q" 13/1"   q>df =>    13e }=

={  12 16 nu>q qcanon => 3/4 }=

TESTING -q |q| 1/q q+ q- q* q/ q2^u* q2^u/

={  3/4 -q  => -3/4 }=
={  3/4 |q| =>  3/4 }=
={ -2/3 |q| =>  2/3 }=
={  3/4 1/q =>  4/3 }=

={    1/2    1/2  q+ =>   q" 1" }=
={   -3/4 q" 1/3" q- => q" -13/12" }=
={    2/3    3/4  q* =>      1/2 }=
={ q" 4/9"   2/3  q/ =>      2/3 }=

={  2/3 3 q2^u* => q" 16/3" }=
={  2/3 3 q2^u/ => q" 1/12" }=

\ for mixed stack tests
q" 3/1" qconstant 3/1   q" 4/1" qconstant 4/1   q" 5/1" qconstant 5/1
q" 3/5" qconstant 3/5   q" 4/5" qconstant 4/5
q" 9/4" qconstant 9/4
[THEN]  \ MULTI-RATIONALS


MULTI-FLOATS [IF]
\ *** FLOATING POINT

\ SETUP

mfspace free throw  \ release the default space
/MPF 2 cells + 7 * CONSTANT /MFTBUF
/MFTBUF make-mfspace

\ TROUBLE-SHOOTING WORDS

: .mfstat  ( -- )
  VERBOSE @ IF
    ." mfbuffer size: " mfspace >/gbuf @ . ." bytes" cr
    ." mfunused: " mfspace gunused . ." bytes   " cr
    ." mfdepth:  " mfspace gdepth . cr
    base @ hex
    ." mfbuffer: 0x" mfspace >gbuf . cr
    ." mfbreak:  0x" mfspace >gbrkp @ . cr
    ." mfsp:     0x" mfspace >gsp @ . cr
    ." mfsp0:    0x" mfspace >gsp0 @ . cr
    decimal
    ." mfaprec:   " mfa-start-precision @ . cr
    mfgarbage? IF ." There is some mf garbage. "
    ELSE ." There is no mf garbage. " THEN cr
    ." Garbage collection lock: "
    mfgc-lock@ IF ." ON" ELSE ." OFF" THEN cr
    base ! THEN ;

: dump-mfspace  ( -- )
  VERBOSE @ IF
    mfspace dup >/gbuf @ /gspace-header + cell+ ( #bytes) dump
    [ PFE-HOST ] [IF] cr [THEN]
    mfspace .mfstat THEN ;

: dump-mf  ( 'f -- )
  ." mf: " mfspace >/grec @ cell-  bounds ?DO i @ . 1 cells +LOOP cr ;

: mfsp@    ( -- mfsp ) mfspace >gsp @ ;
: 0mfspace ( -- )      mfspace 0gspace ;

TESTING set-default-mfprecision get-default-mfprecision

t{ get-default-mfprecision 256 set-default-mfprecision -> 128 }t
t{ get-default-mfprecision 128 set-default-mfprecision -> 256  }t
t{ get-default-mfprecision -> 128  }t

t{ mfa-start-precision @ -> get-default-mfprecision }t

TESTING mf" mfs@ (mfdrop) mfdrop mfdepth mf. MFCONSTANT MFVARIABLE

: ?mf.	 (mf: f -- )	  VERBOSE @ IF mf.   ELSE mfdrop THEN ;
: ?(mf.) (mf: f s: u -- ) VERBOSE @ IF (mf.) ELSE mfdrop drop THEN ;

mf"  0e0" mfconstant  0.mf
mf"  1e0" mfconstant  1.mf   mf"  -1e0" mfconstant  -1.mf
mf"  2e0" mfconstant  2.mf   mf"  -2e0" mfconstant  -2.mf
mf"  3e0" mfconstant  3.mf   mf"  -3e0" mfconstant  -3.mf
mf"  4e0" mfconstant  4.mf   mf"  -4e0" mfconstant  -4.mf 
mf"  5e0" mfconstant  5.mf   mf"  -5e0" mfconstant  -5.mf
mf"  6e0" mfconstant  6.mf   mf"  -6e0" mfconstant  -6.mf
mf"  7e0" mfconstant  7.mf   mf"  -7e0" mfconstant  -7.mf
mf"  8e0" mfconstant  8.mf   mf"  -8e0" mfconstant  -8.mf
mf"  9e0" mfconstant  9.mf   mf"  -9e0" mfconstant  -9.mf
mf" 10e0" mfconstant 10.mf   mf" -10e0" mfconstant -10.mf

0mfspace
t{ mf" 104" mfdepth mfs@ cell- @ ( blink) -> 1 mfsp@ }t
t{ s" 104" MZSTR-s= -> true }t
t{ 3 mfs@ >MFSTR s" 104" MZSTR-s= MFEXP @ -> true 3 }t
t{ mfgarbage? mfdepth MFCONSTANT 104.mf mfgarbage? mfdepth 104.mf mfdepth
   -> false 1 true 0 1 }t  \ copy grec to ext, garbage grec
?." You should see 0.104E3: "
t{ get-output-mfprecision 3 set-output-mfprecision
     get-output-mfprecision -> 6 3 }t
t{ ?mf. mfdepth -> 0 }t ?.cr
6 set-output-mfprecision

t{ : compiled-104.mf  104.mf ; -> }t
t{ MFEXP off compiled-104.mf 3 mfs@ >MFSTR s" 104" MZSTR-s= MFEXP @ -> true 3 }t

t{ 1 0.mf mfs@ >MFSTR s" " MZSTR-s= -> true }t  \ no trailing zero digits
t{ (mfdrop) mfdepth mfdrop mfdepth -> 1 0 }t

t{ FVARIABLE fvar fvar @ -> 0 }t

TESTING >mfs >mfs-copy mf,'f mf, mfs2@ mfs3@ mf2drop mf3drop

: mfdynam? ( 'f -- flag ) mfspace is-grec ; 

={ 1.mf mfs@ dup >mfs mfdynam?       => 1.mf 1.mf false }=
={ 1.mf mfs@ >mfs-copy mfs@ mfdynam? => 1.mf 1.mf true }=

={ 2.mf align here mf,'f dup >mfs = => true 2.mf }=

={ 1.mf 2.mf     mfs2@ >mfs >mfs      => 1.mf 2.mf 2.mf 1.mf }=
={ 1.mf 2.mf 3.mf mfs3@ >mfs >mfs >mfs => 1.mf 2.mf 3.mf 3.mf 2.mf 1.mf }=

={ 1.mf 2.mf         mf2drop => }=
={ 1.mf 2.mf 3.mf     mf2drop => 1.mf }=
={ 1.mf 2.mf 3.mf     mf3drop =>  }=
={ 1.mf 2.mf 3.mf 2.mf mf3drop => 1.mf }=

={ 2.mf here dup mf, mf@ cell+ => 2.mf here }=

TESTING mfsgn mf> mf= mf< mf<= mf>= mf0> mf0= mf0< mf0>= mf0<=

={  2.mf mfsgn =>  1 }=
={  0.mf mfsgn =>  0 }=
={ -1.mf mfsgn => -1 }=

t{ 0.mf mfdepth -> 1 }t      \ mfstack convervation check...
={ 1.mf 2.mf mf> => false }=
t{ mfdrop mfdepth -> 0 }t   \ ...for ={ => }=
={ 2.mf 1.mf mf> => true }=
={ 2.mf 2.mf mf> => false }=

={ 1.mf 2.mf mf= => false }=
={ 2.mf 1.mf mf= => false }=
={ 2.mf 2.mf mf= => true }=

={ 1.mf 2.mf mf< => true }=
={ 2.mf 1.mf mf< => false }=
={ 2.mf 2.mf mf< => false }=

={ 1.mf 2.mf mf<= => true }=
={ 2.mf 1.mf mf<= => false }=
={ 2.mf 2.mf mf<= => true }=

={ 1.mf 2.mf mf>= => false }=
={ 2.mf 1.mf mf>= => true }=
={ 2.mf 2.mf mf>= => true }=

={  1.mf mf0> => true }=
={  0.mf mf0> => false }=
={ -1.mf mf0> => false }=

={  1.mf mf0= => false }=
={  0.mf mf0= => true }=
={ -1.mf mf0= => false }=

={  1.mf mf0< => false }=
={  0.mf mf0< => false }=
={ -1.mf mf0< => true }=

={  1.mf mf0>= => true }=
={  0.mf mf0>= => true }=
={ -1.mf mf0>= => false }=

={  1.mf mf0<= => false }=
={  0.mf mf0<= => true }=
={ -1.mf mf0<= => true }=

TESTING mfn> mfn= mfn< mfn>= mfn<=

={  1.mf 2 mfn> => false }=
={  2.mf 1 mfn> => true }=
={  2.mf 2 mfn> => false }=

={  1.mf 2 mfn= => false }=
={  2.mf 1 mfn= => false }=
={  2.mf 2 mfn= => true }=

={  1.mf 2 mfn< => true }=
={  2.mf 1 mfn< => false }=
={  2.mf 2 mfn< => false }=

={  1.mf 2 mfn>= => false }=
={  2.mf 1 mfn>= => true }=
={  2.mf 2 mfn>= => true }=

={  1.mf 2 mfn<= => true }=
={  2.mf 1 mfn<= => false }=
={  2.mf 2 mfn<= => true }=

TESTING mfu> mfu= mfu< mfu>= mfu<=

={  2.mf -1 mfu> => false }=
={  2.mf  1 mfu> => true }=
={  2.mf  2 mfu> => false }=

={  1.mf 2 mfu= => false }=
={  2.mf 1 mfu= => false }=
={  2.mf 2 mfu= => true }=

={  1.mf  2 mfu< => true }=
={  2.mf  1 mfu< => false }=
={  2.mf -1 mfu< => true }=

={  1.mf 2 mfu>= => false }=
={  2.mf 1 mfu>= => true }=
={  2.mf 2 mfu>= => true }=

={  1.mf -1 mfu<= => true }=
={  2.mf  1 mfu<= => false }=
={  2.mf  2 mfu<= => true }=

TESTING mfdf> mfdf= mfdf< mfdf>= mfdf<=

={  1.mf 2e mfdf> => false }=
={  2.mf 1e mfdf> => true }=
={  2.mf 2e mfdf> => false }=

={  1.mf 2e mfdf= => false }=
={  2.mf 1e mfdf= => false }=
={  2.mf 2e mfdf= => true }=

={  1.mf 2e mfdf< => true }=
={  2.mf 1e mfdf< => false }=
={  2.mf 2e mfdf< => false }=

={  1.mf 2e mfdf>= => false }=
={  2.mf 1e mfdf>= => true }=
={  2.mf 2e mfdf>= => true }=

={  1.mf 2e mfdf<= => true }=
={  2.mf 1e mfdf<= => false }=
={  2.mf 2e mfdf<= => true }=

TESTING 0>mfa n>mfa u>mfa df>mfa mf>mfa mfa>mf mfa>n mfa>u mfa>df mfa>df-exp
TESTING mfa.

: ?mfa.	  ( -- )   VERBOSE @ IF mfa.   ELSE drop THEN ;
: ?(mfa.) ( u -- ) VERBOSE @ IF (mfa.) ELSE drop THEN ;

BITS/CELL 32 = hex
[IF]
  mf" 7fff ffff"   mf" ffff ffff"
[ELSE]
  mf" 7fff ffff ffff ffff"   mf" ffff ffff ffff ffff"
[THEN] decimal
mfconstant MAXU.mf   mfconstant MAXN.mf

={ 0>mfa	mfa>mf => 0.mf }=
={ MAXN n>mfa	mfa>mf => MAXN.mf }=
={ MAXU u>mfa	mfa>mf => MAXU.mf }=
={  2e  df>mfa	mfa>mf => 2.mf }=
={  2.mf mf>mfa	mfa>mf => 2.mf }=

={ -10.mf mf>mfa mfa>n  => -10 }=
={ -10.mf mf>mfa mfa>u  =>  10 }=
={  10.mf mf>mfa mfa>df =>  10e }=
\ 10.0 = 0.625 x 2^4
={ 10.mf mf>mfa texp mfa>df-exp texp @ => 0.625e 4 }=

?." You should see 0.151@3: " 105 u>mfa 8 base !
get-output-mfprecision 3 set-output-mfprecision ?mfa. decimal ?.cr
set-output-mfprecision

TESTING -mfa |mfa| mfa+ mfa- mfau+ mfau- umfa- mfa* mfau* mfa/ mfau/ umfa/

={  3 n>mfa -mfa   mfa>mf => -3.mf }=
={  3 n>mfa |mfa|  mfa>mf =>  3.mf }=
={ -2 n>mfa |mfa|  mfa>mf =>  2.mf }=
={ 0>mfa 1.mf mfa+ mfa>mf =>  1.mf }=
={ -2.mf    mfa+   mfa>mf => -1.mf }=
={ -3.mf    mfa-   mfa>mf =>  2.mf }=
hex
={ FFFFFFFF mfau+ mfa>mf => mf" 1 0000 0001" }=
={ FFFFFFFF mfau- mfa>mf => mf" 2" }=
decimal
={  2 3 n>mfa   umfa-  mfa>mf =>  -1.mf }=

={  2 n>mfa 4.mf mfa*  mfa>mf => 8.mf }=
={  3 n>mfa 2	 mfau* mfa>mf => 6.mf }=
={  8 n>mfa 2.mf mfa/  mfa>mf => 4.mf }=
={ 10 n>mfa 2	 mfau/ mfa>mf => 5.mf }=
={ 12 2 n>mfa	 umfa/ mfa>mf => 6.mf }=

TESTING mfa-sqrt usqrt>mfa mfa^u mfa2^u* mfa2^u/ mfarel-

={ 16 n>mfa	 mfa-sqrt  mfa>mf => 4.mf }=
={ 25		 usqrt>mfa mfa>mf => 5.mf }=
={  4 n>mfa 2	 mfa^u	   mfa>mf => mf" 16" }=
={  4 n>mfa 3	 mfa2^u*   mfa>mf => mf" 32" }=
={ 16 n>mfa 2	 mfa2^u/   mfa>mf => 4.mf }=
={ 16 n>mfa 2.mf mfarel-   mfa>mf => 7.mf }=

TESTING mfa-ceil mfa-floor mfa-trunc mfa-integer

={  1.5e df>mfa mfa-ceil  mfa>mf =>  2.mf }=
={ -1.5e df>mfa mfa-floor mfa>mf => -2.mf }=
={ -1.5e df>mfa mfa-trunc mfa>mf => -1.mf }=

={  1.5e df>mfa mfa-integer => false }=
={  2.0e df>mfa mfa-integer => true }=

TESTING 0>mf n>mf u>mf df>mf mf>n mf>u mf>df mf>df-exp

={	0>mf =>  0.mf }=
={ MAXN n>mf => MAXN.mf }=
={ MAXU u>mf => MAXU.mf }=
={ 10e df>mf => 10.mf }=

={ MAXN.mf mf>n  => MAXN }=
={ MAXU.mf mf>u  => MAXU }=
={ 10.mf   mf>df => 10e }=
\ 10.0 = 0.625 x 2^4
={ 10.mf texp mf>df-exp texp @ => 0.625e  4 }=

TESTING -mf |mf| mf+ mf- mfu+ mfu- umf- mf* mfu* mf/ mfu/ umf/

={  3 n>mf -mf     => -3.mf }=
={  3 n>mf |mf|    =>  3.mf }=
={ -2 n>mf |mf|	   =>  2.mf }=
={ -2.mf  1.mf mf+ => -1.mf }=
={  5.mf -3.mf mf- =>  8.mf }=
={ 1.mf MAXU  mfu+ => MAXU.mf 1.mf mf+ }=
={ MAXU.mf 1  mfu- => MAXU.mf 1.mf mf- }=
={  2    3.mf umf- =>  -1.mf }=

={  2 n>mf 4.mf mf*  => 8.mf }=
={  3 n>mf 2	mfu* => 6.mf }=
={  8 n>mf 2.mf mf/  => 4.mf }=
={ 10 n>mf 2	mfu/ => 5.mf }=
={ 12 2 n>mf	umf/ => 6.mf }=

TESTING mf-sqrt usqrt>mf mf^u mf2^u* mf2^u/ mfrel-

={ 16 n>mf	mf-sqrt	 => 4.mf }=
={ 25		usqrt>mf => 5.mf }=
={  4 n>mf 2	mf^u	 => mf" 16" }=
={  4 n>mf 3	mf2^u*	 => mf" 32" }=
={ 16 n>mf 2	mf2^u/	 => 4.mf }=
={ 16 n>mf 2.mf mfrel-	 => 7.mf }=

TESTING mf-ceil mf-floor mf-trunc mf-integer

={ mf"  1.5" mf-ceil  =>  2.mf }=
={ mf" -1.5" mf-floor => -2.mf }=
={ mf" -1.5" mf-trunc => -1.mf }=

={ mf" 1.5" mf-integer => false }=
={     2.mf  mf-integer => true }=

\ for mixed stack tests
mf" 0.5" mfconstant 0.5.mf
[THEN]  \ MULTI-FLOATS


RELIABLE-FLOATS [IF]
\ *** RELIABLE FLOATING POINT

\ SETUP

rfspace free throw  \ release the default space
/MPFR 2 cells + 7 * CONSTANT /FRTBUF
/FRTBUF make-rfspace

0 [IF]  \ DEBUGGING
variable #frees  #frees off
: noname 1 #frees +! mpfr_clear ;
rfspace >gfree-rec-xt !
[THEN]

\ TROUBLE-SHOOTING WORDS

: .rfstat  ( -- )
  VERBOSE @ IF
    ." rfbuffer size: " rfspace >/gbuf @ . ." bytes" cr
    ." rfunused: " rfspace gunused . ." bytes   " cr
    ." rfdepth:  " rfspace gdepth . cr
    base @ hex
    ." rfbuffer: 0x" rfspace >gbuf . cr
    ." rfbreak:  0x" rfspace >gbrkp @ . cr
    ." rfsp:     0x" rfspace >gsp @ . cr
    ." rfsp0:    0x" rfspace >gsp0 @ . cr
    decimal
    rfgarbage? IF ." There is some rf garbage. "
    ELSE ." There is no rf garbage. " THEN cr
    ." Garbage collection lock: "
    rfgc-lock@ IF ." ON" ELSE ." OFF" THEN cr
    base ! THEN ;

: dump-rfspace  ( -- )
  VERBOSE @ IF
    rfspace dup >/gbuf @ /gspace-header + cell+ ( #bytes) dump
    [ PFE-HOST ] [IF] cr [THEN]
    rfspace .rfstat THEN ;

: dump-rf  ( 'f -- )
  ." rf: " rfspace >/grec @ cell-  bounds ?DO i @ . 1 cells +LOOP cr ;

: rfsp@    ( -- rfsp ) rfspace >gsp @ ;
: 0rfspace ( -- )      rfspace 0gspace ;

TESTING set-default-rfprecision get-default-rfprecision

\ binary64		\ binary128
53 constant p64		113 constant p128

t{ get-default-rfprecision p64  set-default-rfprecision -> p128 }t
t{ get-default-rfprecision p128 set-default-rfprecision -> p64  }t
t{ get-default-rfprecision -> p128  }t

TESTING get-rfemin get-rfemax set-rfemin set-rfemax set-IEEE-754

\ binary64			\ binary128
 1024 constant rfemax64		 16384 constant rfemax128	
-1073 constant rfemin64		-16493 constant rfemin128

: binary64  ( -- )  53  1023 set-IEEE-754 ;
: binary128 ( -- ) 113 16383 set-IEEE-754 ;

t{ get-rfemin rfemin64  set-rfemin -> rfemin128 true }t
t{ get-rfemax rfemax64  set-rfemax -> rfemax128 true }t
t{ get-rfemin rfemin128 set-rfemin -> rfemin64  true }t
t{ get-rfemax rfemax128 set-rfemax -> rfemax64  true }t

t{ binary64  get-default-rfprecision get-rfemax get-rfemin
     -> p64 rfemax64 rfemin64 }t
t{ binary128 get-default-rfprecision get-rfemax get-rfemin
     -> p128 rfemax128 rfemin128 }t

TESTING RFEMIN-MIN RFEMIN-MAX RFEMAX-MIN RFEMAX-MAX
?." MPFR implementation limits:
?." RFEMIN-MIN: " RFEMIN-MIN ?. ?."   RFEMIN-MAX: " RFEMIN-MAX ?. ?.cr
?." RFEMAX-MIN: " RFEMAX-MIN ?. ?."   RFEMAX-MAX: " RFEMAX-MAX ?. ?.cr

TESTING get-rfround set-rfround RF-NEAREST RF-UP RF-DOWN RF-TOWARD RF-AWAY

t{ get-rfround -> RF-NEAREST }t
t{ MPFR_RNDZ set-rfround get-rfround -> MPFR_RNDZ }t
t{ MPFR_RNDN set-rfround get-rfround -> MPFR_RNDN }t

t{ RF-NEAREST mpfr_print_rnd_mode zcount s" MPFR_RNDN" s= -> true }t
t{ RF-UP      mpfr_print_rnd_mode zcount s" MPFR_RNDU" s= -> true }t
t{ RF-DOWN    mpfr_print_rnd_mode zcount s" MPFR_RNDD" s= -> true }t
t{ RF-TOWARD  mpfr_print_rnd_mode zcount s" MPFR_RNDZ" s= -> true }t
t{ RF-AWAY    mpfr_print_rnd_mode zcount s" MPFR_RNDA" s= -> true }t

TESTING exception flags

t{ clr-rfflags -> }t
t{ rfunderflow rfoverflow rfinvalid rfinexact rferange -> 0 0 0 0 0 }t

t{ set-rfunderflow rfunderflow ->  1 }t
t{ set-rfoverflow  rfoverflow  ->  2 }t
t{ set-rfinvalid   rfinvalid   ->  4 }t
t{ set-rfinexact   rfinexact   ->  8 }t
t{ set-rferange    rferange    -> 16 }t

t{ clr-rfunderflow rfunderflow -> 0 }t
t{ clr-rfoverflow  rfoverflow  -> 0 }t
t{ clr-rfinvalid   rfinvalid   -> 0 }t
t{ clr-rfinexact   rfinexact   -> 0 }t
t{ clr-rferange    rferange    -> 0 }t

TESTING rf" rfs@ (rfdrop) rfdrop rfdepth rf. RFCONSTANT RFVARIABLE

: ?rf.   (rf: f -- )	  VERBOSE @ IF rf.   ELSE rfdrop THEN ;
: ?(rf.) (rf: f s: u -- ) VERBOSE @ IF (rf.) ELSE rfdrop drop THEN ;

rf"  0e0" rfconstant  0.rf
rf"  1e0" rfconstant  1.rf   rf"  -1e0" rfconstant  -1.rf
rf"  2e0" rfconstant  2.rf   rf"  -2e0" rfconstant  -2.rf
rf"  3e0" rfconstant  3.rf   rf"  -3e0" rfconstant  -3.rf
rf"  4e0" rfconstant  4.rf   rf"  -4e0" rfconstant  -4.rf 
rf"  5e0" rfconstant  5.rf   rf"  -5e0" rfconstant  -5.rf
rf"  6e0" rfconstant  6.rf   rf"  -6e0" rfconstant  -6.rf
rf"  7e0" rfconstant  7.rf   rf"  -7e0" rfconstant  -7.rf
rf"  8e0" rfconstant  8.rf   rf"  -8e0" rfconstant  -8.rf
rf"  9e0" rfconstant  9.rf   rf"  -9e0" rfconstant  -9.rf
rf" 10e0" rfconstant 10.rf   rf" -10e0" rfconstant -10.rf

0rfspace
t{ rf" 104" rfdepth rfs@ cell- @ ( blink) -> 1 rfsp@ }t
t{ s" 104" MZSTR-s= -> true }t
t{ 3 rfs@ >RFSTR s" 104" MZSTR-s= RFEXP @ -> true 3 }t
t{ rfgarbage? rfdepth RFCONSTANT 104.rf rfgarbage? rfdepth 104.rf rfdepth
   -> false 1 true 0 1 }t  \ copy grec to ext, garbage grec
?." You should see 0.104E3: "
t{ get-output-rfprecision 3 set-output-rfprecision
     get-output-rfprecision -> 6 3 }t
t{ ?rf. rfdepth -> 0 }t ?.cr
6 set-output-rfprecision

t{ : compiled-104.rf  104.rf ; -> }t
t{ RFEXP off compiled-104.rf 3 rfs@ >RFSTR s" 104" MZSTR-s= RFEXP @ -> true 3 }t

t{ 1 0.rf rfs@ >RFSTR s" 0" MZSTR-s= -> true }t  \ no trailing zero digits
t{ (rfdrop) rfdepth rfdrop rfdepth -> 1 0 }t

TESTING >rfs >rfs-copy rf,'f rf, rfs2@ rfs3@ rf2drop rf3drop

: rfdynam? ( 'f -- flag ) rfspace is-grec ; 

free-rfrecs
rfgc-off
={ 1.rf rfs@ dup >rfs rfdynam?       => 1.rf 1.rf false }=
={ 1.rf rfs@ >rfs-copy rfs@ rfdynam? => 1.rf 1.rf true }=

={ 2.rf align here rf,'f dup >rfs = => true 2.rf }=

={ 1.rf 2.rf      rfs2@ >rfs >rfs      => 1.rf 2.rf 2.rf 1.rf }=
={ 1.rf 2.rf 3.rf rfs3@ >rfs >rfs >rfs => 1.rf 2.rf 3.rf 3.rf 2.rf 1.rf }=

={ 1.rf 2.rf	       rf2drop => }=
={ 1.rf 2.rf 3.rf      rf2drop => 1.rf }=
={ 1.rf 2.rf 3.rf      rf3drop =>  }=
={ 1.rf 2.rf 3.rf 2.rf rf3drop => 1.rf }=
rfgc-on

={ 2.rf here dup rf, rf@ cell+ => 2.rf here }=

TESTING rfsgn rf> rf= rf< rf<= rf>= rf0> rf0= rf0< rf0>= rf0<=

={  2.rf rfsgn =>  1 }=
={  0.rf rfsgn =>  0 }=
={ -1.rf rfsgn => -1 }=

t{ 0.rf rfdepth -> 1 }t      \ rfstack convervation check...
={ 1.rf 2.rf rf> => false }=
t{ rfdrop rfdepth -> 0 }t   \ ...for ={ => }=
={ 2.rf 1.rf rf> => true }=
={ 2.rf 2.rf rf> => false }=

={ 1.rf 2.rf rf= => false }=
={ 2.rf 1.rf rf= => false }=
={ 2.rf 2.rf rf= => true }=

={ 1.rf 2.rf rf< => true }=
={ 2.rf 1.rf rf< => false }=
={ 2.rf 2.rf rf< => false }=

={ 1.rf 2.rf rf<= => true }=
={ 2.rf 1.rf rf<= => false }=
={ 2.rf 2.rf rf<= => true }=

={ 1.rf 2.rf rf>= => false }=
={ 2.rf 1.rf rf>= => true }=
={ 2.rf 2.rf rf>= => true }=

={  1.rf rf0> => true }=
={  0.rf rf0> => false }=
={ -1.rf rf0> => false }=

={  1.rf rf0= => false }=
={  0.rf rf0= => true }=
={ -1.rf rf0= => false }=

={  1.rf rf0< => false }=
={  0.rf rf0< => false }=
={ -1.rf rf0< => true }=

={  1.rf rf0>= => true }=
={  0.rf rf0>= => true }=
={ -1.rf rf0>= => false }=

={  1.rf rf0<= => false }=
={  0.rf rf0<= => true }=
={ -1.rf rf0<= => true }=

TESTING rfn> rfn= rfn< rfn>= rfn<=

={  1.rf 2 rfn> => false }=
={  2.rf 1 rfn> => true }=
={  2.rf 2 rfn> => false }=

={  1.rf 2 rfn= => false }=
={  2.rf 1 rfn= => false }=
={  2.rf 2 rfn= => true }=

={  1.rf 2 rfn< => true }=
={  2.rf 1 rfn< => false }=
={  2.rf 2 rfn< => false }=

={  1.rf 2 rfn>= => false }=
={  2.rf 1 rfn>= => true }=
={  2.rf 2 rfn>= => true }=

={  1.rf 2 rfn<= => true }=
={  2.rf 1 rfn<= => false }=
={  2.rf 2 rfn<= => true }=

TESTING rfu> rfu= rfu< rfu>= rfu<=

={  2.rf -1 rfu> => false }=
={  2.rf  1 rfu> => true }=
={  2.rf  2 rfu> => false }=

={  1.rf 2 rfu= => false }=
={  2.rf 1 rfu= => false }=
={  2.rf 2 rfu= => true }=

={  1.rf  2 rfu< => true }=
={  2.rf  1 rfu< => false }=
={  2.rf -1 rfu< => true }=

={  1.rf 2 rfu>= => false }=
={  2.rf 1 rfu>= => true }=
={  2.rf 2 rfu>= => true }=

={  1.rf -1 rfu<= => true }=
={  2.rf  1 rfu<= => false }=
={  2.rf  2 rfu<= => true }=

TESTING rfdf> rfdf= rfdf< rfdf>= rfdf<=

={  1.rf 2e rfdf> => false }=
={  2.rf 1e rfdf> => true }=
={  2.rf 2e rfdf> => false }=

={  1.rf 2e rfdf= => false }=
={  2.rf 1e rfdf= => false }=
={  2.rf 2e rfdf= => true }=

={  1.rf 2e rfdf< => true }=
={  2.rf 1e rfdf< => false }=
={  2.rf 2e rfdf< => false }=

={  1.rf 2e rfdf>= => false }=
={  2.rf 1e rfdf>= => true }=
={  2.rf 2e rfdf>= => true }=

={  1.rf 2e rfdf<= => true }=
={  2.rf 1e rfdf<= => false }=
={  2.rf 2e rfdf<= => true }=

TESTING n>rfa u>rfa df>rfa rf>rfa rfa>rf rfa>n rfa>u rfa>df rfa>df-exp rfa.

: ?rfa.   ( -- )   VERBOSE @ IF rfa.   ELSE drop THEN ;
: ?(rfa.) ( u -- ) VERBOSE @ IF (rfa.) ELSE drop THEN ;

BITS/CELL 32 = hex
[IF]
  rf" 7fffffff"   rf" ffffffff"
[ELSE]
  rf" 7fffffffffffffff"   rf" ffffffffffffffff"
[THEN] decimal
rfconstant MAXU.rf   rfconstant MAXN.rf

={ MAXN n>rfa	rfa>rf => MAXN.rf }=
={ MAXU u>rfa	rfa>rf => MAXU.rf }=
={  2e  df>rfa	rfa>rf => 2.rf }=
={  2.rf rf>rfa rfa>rf => 2.rf }=

={ -10.rf  rf>rfa rfa>n  => -10 }=
={ MAXU.rf rf>rfa rfa>u  => MAXU }=
={  10.rf  rf>rfa rfa>df =>  10e }=
\ 10.0 = 0.625 x 2^4
={ 10.rf rf>rfa texp rfa>df-exp texp @ => 0.625e 4 }=

?." You should see 0.151@3: " 105 u>rfa 8 base !
get-output-rfprecision 3 set-output-rfprecision ?rfa. decimal ?.cr
set-output-rfprecision

TESTING pi>rfa ln2>rfa euler>rfa catalan>rfa

binary64
={ pi>rfa      rfa>rf => -1e facos df>rf }=
={ ln2>rfa     rfa>rf => 2e fln df>rf }=
={ euler>rfa   rfa>rf => rf" 0.5772156649015328606065" }=
={ catalan>rfa rfa>rf =>
     rf" 0.915965594177219015054603514932384111" }=
binary128

TESTING -rfa |rfa| rfa+ rfa- rfau+ rfau- urfa- rfan+ rfan- nrfa-

:  0.rfa  0 n>rfa ;
:  2.rfa  2 n>rfa ;   : -2.rfa  -2 n>rfa ;
:  3.rfa  3 n>rfa ;
:  4.rfa  4 n>rfa ;
:  8.rfa  8 n>rfa ;
: 10.rfa 10 n>rfa ;

={  3.rfa -rfa  rfa>rf => -3.rf }=
={  3.rfa |rfa| rfa>rf =>  3.rf }=
={ -2.rfa |rfa| rfa>rf =>  2.rf }=

={ 0.rfa 1.rf rfa+ rfa>rf =>  1.rf }=
={ -2.rf     rfa+ rfa>rf => -1.rf }=
={ -3.rf     rfa- rfa>rf =>  2.rf }=
hex
={ FFFFFFFF rfau+ rfa>rf => rf" 100000001" }=
={ FFFFFFFF rfau- rfa>rf => rf" 2" }=
decimal
={  2 3.rfa urfa- rfa>rf =>  -1.rf }=

={  2.rfa -3 rfan+ rfa>rf => -1.rf }=
={  2.rfa -3 rfan- rfa>rf =>  5.rf }=
={ -2.rfa -3 nrfa- rfa>rf => -1.rf }=

TESTING rfadf+ rfadf- dfrfa-

={ 4.rfa -3e rfadf+ rfa>rf =>  1.rf }=
={ 4.rfa -3e rfadf- rfa>rf =>  7.rf }=
={ 4.rfa -3e dfrfa- rfa>rf => -7.rf }=

TESTING rfa* rfau* rfan* rfadf* rfa*+ rfa*-

={  2.rfa  4.rf rfa*   rfa>rf =>  8.rf }=
={  3.rfa  2	rfau*  rfa>rf =>  6.rf }=
={  3.rfa -2	rfan*  rfa>rf => -6.rf }=
={  2.rfa -2e	rfadf* rfa>rf => -4.rf }=

={ 2.rfa 1.rf 3.rf rfa*+ rfa>rf => 7.rf }=
={ 2.rfa 1.rf 3.rf rfa*- rfa>rf => 5.rf }=

TESTING rfa/ rfau/ urfa/ rfan/ nrfa/ rfadf/ dfrfa/

={   8.rfa  2.rf  rfa/   rfa>rf =>  4.rf }=
={  10.rfa  2	  rfau/  rfa>rf =>  5.rf }=
={  12	    2.rfa urfa/  rfa>rf =>  6.rf }=
={  10.rfa -2	  rfan/  rfa>rf => -5.rf }=
={ -12	    2.rfa nrfa/  rfa>rf => -6.rf }=
={  10.rfa -2e	  rfadf/ rfa>rf => -5.rf }=
={ -12e	    2.rfa dfrfa/ rfa>rf => -6.rf }=

TESTING rfa-sqrt usqrt>rfa 1/rfa-sqrt rfa-cbrt rfa-uroot

: 16.rfa 16 n>rfa ;   16 n>rf rfconstant  16.rf

={  16.rfa	rfa-sqrt	  rfa>rf =>  4.rf }=
={  25		usqrt>rfa rfa>rf =>  5.rf }=
={   4.rfa	1/rfa-sqrt rfa>rf => rf" 0.5e0" }=
={   8.rfa	rfa-cbrt	  rfa>rf =>  2.rf }=
={ -32 n>rfa 5	rfa-uroot  rfa>rf => -2.rf }=

TESTING rfa-square rfa-pow rfa^u rfa^n u^rfa u^u>rfa

={  2.rfa       rfa-square rfa>rf =>   4.rf }=
={  4.rfa  2.rf rfa-pow	  rfa>rf =>  16.rf }=
={  4.rfa  2    rfa^u	  rfa>rf =>  16.rf }=
={  4.rfa -2    rfa^n	  rfa>rf => hex rf" 0.1p0" decimal }=
={  4  2.rfa    u^rfa	  rfa>rf =>  16.rf }=
={  2  3        u^u>rfa	  rfa>rf =>   8.rf }=

TESTING rfa2^n* rfa2^u* rfa2^n/ rfa2^u/

={  4.rfa -1 rfa2^n* rfa>rf =>  2.rf }=
={  4.rfa  3 rfa2^u* rfa>rf => rf" 32" }=
={  4.rfa -2 rfa2^n/ rfa>rf => 16.rf }=
={ 16.rfa  2 rfa2^u/ rfa>rf =>  4.rf }=

TESTING rfa-max rfa-min rfa-reldiff rfa-dim

={  4.rfa 16.rf rfa-max rfa>rf => 16.rf }=
={ 16.rfa  4.rf rfa-max rfa>rf => 16.rf }=
={  4.rfa 16.rf rfa-min rfa>rf =>  4.rf }=
={ 16.rfa  4.rf rfa-min rfa>rf =>  4.rf }=

={ 16.rfa 2.rf rfa-reldiff rfa>rf => 7.rf }=
={  4.rfa 2.rf rfa-dim  rfa>rf => 2.rf }=
={  2.rfa 4.rf rfa-dim  rfa>rf => 0.rf }=

TESTING rfa-ceil rfa-floor rfa-trunc rfa-integer

={  1.5e df>rfa rfa-ceil  rfa>rf =>  2.rf }=
={ -1.5e df>rfa rfa-floor rfa>rf => -2.rf }=
={ -1.5e df>rfa rfa-trunc rfa>rf => -1.rf }=

={  1.5e df>rfa rfa-integer => false }=
={  2.0e df>rfa rfa-integer => true }=

TESTING rfa-ln rfa-log2 rfa-log10 rfa-exp rfa-exp2 rfa-exp10

binary64
={  2.rfa rfa-ln   rfa>rf => 2e fln df>rf }=
={ 10.rfa rfa-log2  rfa>rf => 1e 2e flog f/ df>rf }=
={  2.rfa rfa-log10 rfa>rf => 2e flog df>rf }=
={  2.rfa rfa-exp   rfa>rf => 2e fexp df>rf }=
={  2.rfa rfa-exp2  rfa>rf => 4.rf }=
={  2.rfa rfa-exp10 rfa>rf => 100 u>rf }=
binary128

TESTING rfa-cos rfa-sin rfa-tan rfa-sincos rfa-sec rfa-csc rfa-cot

:  pi/6.rfa ( -- ) pi>rf rf>rfa 6 rfau/ ;

pi/6.rfa rfa>rf rfconstant pi/6.rf
1e 2e f/ df>rf  rfconstant 1/2.rf
1e 3e fsqrt f/
  fdup  df>rf   rfconstant 1/rt3.rf
  2e f* df>rf   rfconstant 2/rt3.rf
3e fsqrt
  fdup  df>rf   rfconstant rt3.rf
  2e f/ df>rf   rfconstant rt3/2.rf   

\ rounding not always correct
SET-RFT-MODE-REL

={ pi/6.rfa rfa-cos rfa>rf => rt3/2.rf }=
={ pi/6.rfa rfa-sin rfa>rf => 1/2.rf }=
={ pi/6.rfa rfa-tan rfa>rf => 1/rt3.rf }=
={ pi/6.rfa rfa-sec rfa>rf => 2/rt3.rf }=
={ pi/6.rfa rfa-csc rfa>rf => 2.rf }=
={ pi/6.rfa rfa-cot rfa>rf => rt3.rf }=
={ pi/6.rfa rfa-sincos rfa>rf rfx>rf => 1/2.rf rt3/2.rf }=

TESTING rfa-acos rfa-asin rfa-atan rfa-atan2

: 1.rfa   ( -- ) 1 n>rfa ;
: rt3.rfa ( -- ) 3 usqrt>rfa ;

={ rt3/2.rf rf>rfa rfa-acos  rfa>rf => pi/6.rf }=
={ 1/2.rf   rf>rfa rfa-asin  rfa>rf => pi/6.rf }=
={ 1/rt3.rf rf>rfa rfa-atan  rfa>rf => pi/6.rf }=
={ 1.rf rt3.rfa    rfa-atan2 rfa>rf => pi/6.rf }=

TESTING rfa-cosh rfa-sinh rfa-tanh rfa-sinhcosh

3e fsqrt fconstant rt3.f

={ rt3.rfa rfa-cosh rfa>rf => rt3.f fcosh df>rf }=
={ rt3.rfa rfa-sinh rfa>rf => rt3.f fsinh df>rf }=
={ rt3.rfa rfa-tanh rfa>rf => rt3.f ftanh df>rf }=
={ rt3.rfa rfa-sinhcosh rfa>rf rfx>rf =>
     rt3.f fdup fsinh df>rf fcosh df>rf }=

TESTING rfa-sech rfa-csch rfa-coth rfa-acosh rfa-asinh rfa-atanh

[UNDEFINED] 1/F [IF] : 1/F( f: r -- 1/r ) 1e fswap f/ ; [THEN]

={ rt3.rfa rfa-sech rfa>rf => rt3.f fcosh 1/f df>rf }=
={ rt3.rfa rfa-csch rfa>rf => rt3.f fsinh 1/f df>rf }=
={ rt3.rfa rfa-coth rfa>rf => rt3.f ftanh 1/f df>rf }=

={ rt3.rfa rfa-acosh rfa>rf => rt3.f facosh df>rf }=
={ rt3.rfa rfa-asinh rfa>rf => rt3.f fasinh df>rf }=
={ 1/rt3.rf rf>rfa rfa-atanh rfa>rf => rt3.f 1/f fatanh df>rf }=

TESTING ufac>rfa rfa-ln1p rfa-expm1 rfa-eint rfa-li2 rfa-gamma rfa-lngamma

:  1/2.rfa  0.5e df>rfa ;
: -1/2.rfa -0.5e df>rfa ;   -1/2.rfa rfa>rf rfconstant -1/2.rf

={ 3       ufac>rfa  rfa>rf => 6.rf }=
={ rt3.rfa rfa-ln1p rfa>rf => rt3.f flnp1 df>rf }=
={ rt3.rfa rfa-expm1 rfa>rf => rt3.f fexpm1 df>rf }=

\ http://functions.wolfram.com/
={  1.rfa rfa-eint    rfa>rf => rf"  1.8951178163559368" }=
={  1.rfa rfa-li2     rfa>rf => rf"  1.6449340668482264" }=
={ -1/2.rfa rfa-gamma rfa>rf => rf" -3.5449077018110321" }=
={  3.rfa rfa-lngamma rfa>rf => rf"  0.69314718055994531" }=

TESTING rfa-lgamma rfa-digamma rfa-zeta uzeta>rfa rfa-erf rfa-erfc

={ -1/2.rfa rfa-lgamma	rfa>rf =>
      rf" 3.5449077018110321" rf>rfa rfa-ln rfa>rf -1 }=
={  1/2.rfa rfa-digamma rfa>rf => rf" -1.9635100260214235" }=
={    3.rfa rfa-zeta	rfa>rf => rf"  1.2020569031595943" }=
={    3     uzeta>rfa	rfa>rf => rf"  1.2020569031595943" }=
={    1.rfa rfa-erf	rfa>rf => rf"  0.84270079294971487" }=
={    1.rfa rfa-erfc	rfa>rf => rf"  0.15729920705028513" }=

TESTING rfa-j0 rfa-j1 rfa-jn rfa-y0 rfa-y1 rfa-yn rfa-ai rfa-agm rfa-hypot

={ 1.rfa   rfa-j0 rfa>rf => rf"  0.76519768655796655" }=
={ 1.rfa   rfa-j1 rfa>rf => rf"  0.44005058574493352" }=
={ 1.rfa 2 rfa-jn rfa>rf => rf"  0.11490348493190048" }=
={ 1.rfa   rfa-y0 rfa>rf => rf"  0.088256964215676958" }=
={ 1.rfa   rfa-y1 rfa>rf => rf" -0.78121282130028872" }=
={ 1.rfa 2 rfa-yn rfa>rf => rf" -1.6506826068162544" }=
={ 0.rfa   rfa-ai rfa>rf => rf"  0.35502805388781724" }=
={ 1.rfa 2.rf   rfa-agm   rfa>rf => rf" 1.4567910310469069" }=
={ 1.rfa rt3.rf rfa-hypot rfa>rf => 2.rf }=

TESTING rfa-signbit rfa-setsign rfa-copysign

={  2.rfa rfa-signbit => false }=
={ -2.rfa rfa-signbit => true }=
={  2.rfa true  rfa-setsign  rfa>rf => -2.rf }=
={  3.rfa -1.rf rfa-copysign rfa>rf => -3.rf }=

TESTING 0>rfa -0>rfa nan>rfa -nan>rfa inf>rfa -inf>rfa
TESTING rfa-finite rfa-infinite rfa-nan rfa-regular

={  2.rfa   rfa-finite	 => true }=
={  2.rfa   rfa-infinite => false }=
={  inf>rfa rfa-finite	 => false }=
={  inf>rfa rfa-infinite => true }=
={  inf>rfa rfa-signbit  => false }=
={ -inf>rfa rfa-finite	 => false }=
={ -inf>rfa rfa-infinite => true }=
={ -inf>rfa rfa-signbit	 => true }=
={  0>rfa   rfa-finite	 => true }=
={  0>rfa   rfa-regular  => false }=
={  0>rfa   rfa>rf rf0=  => true }=
={ -0>rfa   rfa-finite	 => true }=
={ -0>rfa   rfa-regular	 => false }=
={ -0>rfa   rfa>rf rf0=  => true }=
={ -0>rfa   rfa>rf	 => 0.rf }=
={ -0>rfa   rfa-signbit	 => true }=
={  nan>rfa rfa-finite	 => false }=
={  nan>rfa rfa-nan	 => true }=
={  nan>rfa rfa-signbit	 => false }=
={  nan>rfa rfa-regular	 => false }=
={ -nan>rfa rfa-finite	 => false }=
={ -nan>rfa rfa-nan	 => true }=
={ -nan>rfa rfa-signbit	 => true }=
={  2.rfa   rfa-regular	 => true }=
={  inf>rfa rfa-regular	 => false }=

TESTING rfa-nextafter rfa-nextup rfa-nextdown rfa-checkrange rfa-subnormalize

hex
rf" 1p1024" rfconstant maxbin64+1.rf
decimal

binary64
hex
rf" 1.0000000000001p0"  rfconstant 1-up.rf
rf" 1.FFFFFFFFFFFFFp-1" rfconstant 1-down.rf
decimal

SET-RFT-MODE-EXACT

={ 1.rfa  2.rf rfa-nextafter rfa>rf => 1-up.rf }=
={ 1.rfa -2.rf rfa-nextafter rfa>rf => 1-down.rf }=

={ 1.rfa rfa-nextup   rfa>rf => 1-up.rf }=
={ 1.rfa rfa-nextdown rfa>rf => 1-down.rf }=

={ 0 maxbin64+1.rf rf>rfa rfa-checkrange rfa-infinite => true }=

\ Improved example from MPFR 3.0.1 manual, pp. 34, 35.
hex
rf" 1.fffffffffffffp-1022"   rfconstant a.rf 
rf" 1p5" 		     rfconstant b.rf
rf" 0.0fffffffffffff8p-1022" rfconstant a/b.rf		  \ 53 bits
rf" 0.0ffffffffffffp-1022"   rfconstant a/b-subnormal.rf  \ 48 bits
decimal

RF-TOWARD set-rfround
\ RFA/ uses full precision
={ a.rf rf>rfa b.rf rfa/ rfa>rf => a/b.rf }=
={ a.rf rf>rfa b.rf rfa/ rfret @ rfa-subnormalize rfa>rf => a/b-subnormal.rf }=
RF-NEAREST set-rfround
binary128

TESTING pi>rf ln2>rf euler>rf catalan>rf free-rfcache

binary64
={ pi>rf      => -1e facos df>rf }=
={ ln2>rf     => 2e fln df>rf }=
={ euler>rf   => rf" 0.5772156649015328606065" }=
={ catalan>rf =>
     rf" 0.915965594177219015054603514932384111" }=

\ stack effect only
={ free-rfcache => }=
binary128

TESTING n>rf u>rf df>rf rf>n rf>u rf>df rf>df-exp

SET-RFT-MODE-EXACT

={ MAXN n>rf => MAXN.rf }=
={ MAXU u>rf => MAXU.rf }=
={ 10e df>rf => 10.rf }=

={ MAXN.rf rf>n  => MAXN }=
={ MAXU.rf rf>u  => MAXU }=
={ 10.rf   rf>df => 10e }=
\ 10.0 = 0.625 x 2^4
={ 10.rf texp rf>df-exp texp @ => 0.625e  4 }=

TESTING -rf |rf| rf+ rfn+ rfu+ rfdf+ rf- rfu- urf- rfdf- dfrf-

={  3.rf -rf  => -3.rf }=
={  3.rf |rf| =>  3.rf }=
={ -2.rf |rf| =>  2.rf }=

={ -2.rf  1.rf rf+  => -1.rf }=
={  2.rf -1    rfn+ =>  1.rf }=
={  1.rf MAXU  rfu+ => MAXU.rf 1.rf rf+ }=
={  1.rf  3e   rfdf+ => 4.rf }=

={  5.rf  -3.rf rf-   =>  8.rf }=
={  2.rf   1    rfn-  =>  1.rf }=
={  2.rf   1    nrf-  => -1.rf }=
={ MAXU.rf 1    rfu-  => MAXU.rf 1.rf rf- }=
={  2      3.rf urf-  => -1.rf }=
={  1.rf   3e   rfdf- => -2.rf }=
={  3e     1.rf dfrf- =>  2.rf }=

TESTING rf* rfn* rfu* rf*+ rf*- rf/ rfn/ nrf/ rfu/ urf/ rfdf/ dfrf/

={ 2.rf  4.rf rf*  =>  8.rf }=
={ 3.rf -2    rfn* => -6.rf }=
={ 3.rf  2    rfu* =>  6.rf }=

={ 1.rf 3.rf 2.rf rf*+ => 7.rf  }=
={ 1.rf 3.rf 2.rf rf*- => 5.rf  }=

={   8.rf  2.rf rf/   =>  4.rf }=
={  10.rf -2    rfn/  => -5.rf }=
={  10.rf  2    rfu/  =>  5.rf }=
={ -12     2.rf nrf/  => -6.rf }=
={  12     2.rf urf/  =>  6.rf }=
={   8.rf  2e   rfdf/ =>  4.rf }=
={   8e    2.rf dfrf/ =>  4.rf }=

TESTING rf-sqrt usqrt>rf 1/rf-sqrt rf-cbrt rfuroot

={ 16.rf rf-sqrt   => 4.rf }=
={ 25	 usqrt>rf => 5.rf }=
={  4.rf 1/rf-sqrt => rf" 0.5e0" }=

={   8.rf    rf-cbrt   =>  2.rf }=
={ -32 n>rf 5 rfuroot => -2.rf }=

TESTING rf-square rf-pow rf^n rf^u u^rf u^u>rf rf2^n* rf2^u* rf2^n/ rf2^u/

={  2.rf	rf-square =>  4.rf }=
={  4.rf  2.rf	rf-pow	  => 16.rf }=
={  4.rf -2	rf^n	  => hex rf" 0.1p0" decimal }=
={  4.rf  2	rf^u	  => 16.rf }=
={  4.rf -1	rf2^n*	  =>  2.rf }=
={  4.rf  3	rf2^u*	  => rf" 32" }=
={  4.rf -2	rf2^n/	  => 16.rf }=
={ 16.rf  2	rf2^u/	  =>  4.rf }=

TESTING rf-max rf-min rf-reldiff rf-dim

={  4.rf 16.rf rf-max => 16.rf }=
={ 16.rf  4.rf rf-max => 16.rf }=
={  4.rf 16.rf rf-min =>  4.rf }=
={ 16.rf  4.rf rf-min =>  4.rf }=

={ 16.rf  2.rf	rf-reldiff => 7.rf }=

={  4.rf 2.rf rf-dim => 2.rf }=
SET-RFT-MODE-EXACT
={  2.rf 4.rf rf-dim => 0.rf }=
SET-RFT-MODE-REL

TESTING rf-ceil rf-floor rf-trunc rf-integer

={ rf"  1.5" rf-ceil  =>  2.rf }=
={ rf" -1.5" rf-floor => -2.rf }=
={ rf" -1.5" rf-trunc => -1.rf }=

={ rf" 1.5"  rf-integer => false }=
={     2.rf  rf-integer => true }=

TESTING rf-ln rf-log2 rf-log10 rf-exp rf-exp2 rf-exp10

53 set-default-rfprecision
={  2.rf rf-ln	  => 2e fln df>rf }=
={ 10.rf rf-log2  => 1e 2e flog f/ df>rf }=
={  2.rf rf-log10 => 2e flog df>rf }=
={  2.rf rf-exp   => 2e fexp df>rf }=
={  2.rf rf-exp2  => 4.rf }=
={  2.rf rf-exp10 => 100 u>rf }=

TESTING rf-cos rf-sin rf-tan rf-sincos rf-sec rf-csc rf-cot

SET-RFT-MODE-REL
={ pi/6.rf rf-cos => rt3/2.rf }=
={ pi/6.rf rf-sin => 1/2.rf }=
={ pi/6.rf rf-tan => 1/rt3.rf }=
={ pi/6.rf rf-sec => 2/rt3.rf }=
={ pi/6.rf rf-csc => 2.rf }=
={ pi/6.rf rf-cot => rt3.rf }=
={ pi/6.rf rf-sincos => 1/2.rf rt3/2.rf }=

TESTING rf-acos rf-asin rf-atan rf-atan2

={ rt3/2.rf rf-acos => pi/6.rf }=
={ 1/2.rf   rf-asin => pi/6.rf }=
={ 1/rt3.rf rf-atan => pi/6.rf }=
={ 1.rf rt3.rf rf-atan2 => pi/6.rf }=

TESTING rf-cosh rf-sinh rf-tanh rf-sinhcosh

={ rt3.rf rf-cosh => rt3.f fcosh df>rf }=
={ rt3.rf rf-sinh => rt3.f fsinh df>rf }=
={ rt3.rf rf-tanh => rt3.f ftanh df>rf }=
={ rt3.rf rf-sinhcosh => rt3.f fdup fsinh df>rf fcosh df>rf }=

TESTING rf-sech rf-csch rf-coth rf-acosh rf-asinh rf-atanh

={ rt3.rf rf-sech => rt3.f fcosh 1/f df>rf }=
={ rt3.rf rf-csch => rt3.f fsinh 1/f df>rf }=
={ rt3.rf rf-coth => rt3.f ftanh 1/f df>rf }=

={ rt3.rf   rf-acosh => rt3.f facosh df>rf }=
={ rt3.rf   rf-asinh => rt3.f fasinh df>rf }=
={ 1/rt3.rf rf-atanh => rt3.f 1/f fatanh df>rf }=

TESTING ufac>rf rf-ln1p rf-expm1 rf-eint rf-li2 rf-gamma rf-lngamma

={ 3      ufac>rf  => 6.rf }=
={ rt3.rf rf-ln1p  => rt3.f flnp1 df>rf }=
={ rt3.rf rf-expm1 => rt3.f fexpm1 df>rf }=

\ http://functions.wolfram.com/
={ 1.rf rf-eint    => rf" 1.8951178163559368" }=
={ 1.rf rf-li2     => rf" 1.6449340668482264" }=
={ 3.rf rf-gamma   => 2.rf }=
={ 3.rf rf-lngamma => rf" 0.69314718055994531" }=

TESTING rf-lgamma rf-digamma rf-zeta uzeta>rf rf-erf rf-erfc

={ -1/2.rf rf-lgamma  => rf"  3.5449077018110321" rf-ln -1 }=
={  1/2.rf rf-digamma => rf" -1.9635100260214235" }=
={    3.rf rf-zeta    => rf"  1.2020569031595943" }=
={    3    uzeta>rf   => rf"  1.2020569031595943" }=
={    1.rf rf-erf     => rf"  0.84270079294971487" }=
={    1.rf rf-erfc    => rf"  0.15729920705028513" }=

TESTING rf-j0 rf-j1 rf-jn rf-y0 rf-y1 rf-yn rf-ai rf-agm rf-hypot

={ 1.rf   rf-j0 => rf"  0.76519768655796655" }=
={ 1.rf   rf-j1 => rf"  0.44005058574493352" }=
={ 1.rf 2 rf-jn => rf"  0.11490348493190048" }=
={ 1.rf   rf-y0 => rf"  0.088256964215676958" }=
={ 1.rf   rf-y1 => rf" -0.78121282130028872" }=
={ 1.rf 2 rf-yn => rf" -1.6506826068162544" }=
={ 0.rf   rf-ai => rf"  0.35502805388781724" }=
={ 1.rf 2.rf   rf-agm   => rf" 1.4567910310469069" }=
={ 1.rf rt3.rf rf-hypot => 2.rf }=

TESTING rf-signbit is-rf-signbit rf-setsign rf-copysign

={  2.rf rf-signbit => false }=
={ -2.rf rf-signbit => true }=
={  2.rf is-rf-signbit => false 2.rf }=
={ -2.rf is-rf-signbit => true -2.rf }=
={  2.rf true  rf-setsign  => -2.rf }=
={  3.rf -1.rf rf-copysign => -3.rf }=

TESTING 0>rf -0>rf nan>rf -nan>rf inf>rf -inf>rf
TESTING rf-finite rf-infinite rf-nan rf-regular

rf" +0"   rfconstant +0.rf
rf" -0"   rfconstant -0.rf
rf"  inf" rfconstant  inf.rf
rf" -inf" rfconstant -inf.rf
rf"  nan" rfconstant  nan.rf
\ rf" -nan" rfconstant -nan.rf  \ THIS DOESN'T WORK
-nan>rfa rfa>rf rfconstant -nan.rf

SET-RFT-MODE-EXACT

={  0>rf   is-rf-signbit => false 0.rf }=
={ -0>rf   is-rf-signbit => true 0.rf }=
={  nan>rf is-rf-signbit rf-nan      => false true }=
={ -nan>rf is-rf-signbit rf-nan	     => true  true }=
={  inf>rf is-rf-signbit rf-infinite => false true }=
={ -inf>rf is-rf-signbit rf-infinite => true  true }=

={  2.rf   rf-finite   => true }=
={  2.rf   rf-infinite => false }=
={  inf.rf rf-finite   => false }=
={  inf.rf rf-infinite => true }=
={  inf.rf rf-signbit  => false }=
={ -inf.rf rf-finite   => false }=
={ -inf.rf rf-infinite => true }=
={ -inf.rf rf-signbit  => true }=
={ +0.rf   rf-finite   => true }=
={ +0.rf   rf-regular  => false }=
={ +0.rf   rf0=	       => true }=
={ -0.rf   rf-finite   => true }=
={ -0.rf   rf-regular  => false }=
={ -0.rf   rf0=	       => true }=
={ -0.rf 	       => 0.rf }=
={ -0.rf   rf-signbit  => true }=
={  nan.rf rf-finite   => false }=
={  nan.rf rf-nan      => true }=
={  nan.rf rf-signbit  => false }=
={  nan.rf rf-regular  => false }=
={ -nan.rf rf-finite   => false }=
={ -nan.rf rf-nan      => true }=
={ -nan.rf rf-signbit  => true }=
={  2.rf   rf-regular  => true }=
={  inf.rf rf-regular  => false }=

TESTING rf-nextafter rf-nextup rf-nextdown rf-checkrange rf-subnormalize

SET-RFT-MODE-EXACT
binary64
---
1.rf doesn't work here, because it's defined in binary128,
whereas 1-up.rf and 1-down.rf are defined in binary64, so
results get rounded.
---
={ 1 u>rf  2.rf rf-nextafter => 1-up.rf }=
={ 1 u>rf rf-nextup   => 1-up.rf }=
={ 1 u>rf rf-nextdown => 1-down.rf }=

={ 0 maxbin64+1.rf rf-checkrange rf-infinite => true }=

RF-TOWARD set-rfround
={ a.rf b.rf rf/ => a/b.rf }=
={ a.rf b.rf rf/ rfret @ rf-subnormalize => a/b-subnormal.rf }=
RF-NEAREST set-rfround
binary128

\ for mixed stack tests
rf" 0.5" rfconstant 0.5.rf
[THEN]  \ RELIABLE-FLOATS


\ *** MIXED STACKS

MULTI-INTEGERS MULTI-RATIONALS AND [IF]
TESTING b>q b>qa ba>q ba>qa q>b q>ba qa>b qa>ba

={ 4.b	     b>q	=> 4/1 }=
={ 4.b 	     b>qa qa>q	=> 4/1 }=
={ 3   u>ba  ba>q	=> 3/1 }=
={ 4   u>ba  ba>qa qa>q => 4/1 }=
={ 9/4	     q>b	=> 2.b }=
={ 9/4	     q>ba ba>b	=> 2.b }=
={ 9 4 uu>qa qa>b	=> 2.b }=
={ 9 4 uu>qa qa>ba ba>b => 2.b }=

TESTING b>qnum b>qden b>qanum b>qaden ba>qnum ba>qden ba>qanum ba>qaden
---
These tests also confirm the expected behavior for noncanonical
q's, which proves that the tests of QCANON and QACANON above are
not empty.
---
={ 3.b 4/5	    b>qnum        => 3/5 }=
={ 3.b 4/5	    b>qden        => 4/3 }=
={ 3.b 4 5 nu>qa    b>qanum qa>q  => 3/5 }=
={ 3.b 4 5 nu>qa    b>qaden qa>q  => 4/3 }=
={ 3 n>ba 4/5	    ba>qnum	  => 3/5 }=
={ 3 n>ba 4/5	    ba>qden	  => 4/3 }=
={ 3 u>ba 4 5 uu>qa ba>qanum qa>q => 3/5 }=
={ 3 u>ba 4 5 uu>qa ba>qaden qa>q => 4/3 }=

TESTING qnum>b qden>b qnum>ba qden>ba qanum>b qaden>b qanum>ba qaden>ba

={ 12 16 nu>q  qnum>b	     => b" 12" }=
={ 12 16 uu>q  qden>b	     => b" 16" }=
={  3  4 nu>q  qnum>ba ba>b  => 3.b }=
={  3  4 nu>q  qden>ba ba>b  => 4.b }=
={ 12 16 nu>qa qanum>b	     => b" 12" }=
={ 12 16 uu>qa qaden>b	     => b" 16" }=
={  3  4 nu>qa qanum>ba ba>b => 3.b }=
={  3  4 nu>qa qaden>ba ba>b => 4.b }=
[THEN]

MULTI-INTEGERS MULTI-FLOATS AND [IF]
TESTING b>mf b>mfa ba>mf ba>mfa mf>b mf>ba mfa>b mfa>ba

={ 2.b	       b>mf	     => 2.mf }=
={ 2.b	       b>mfa  mfa>mf => 2.mf }=
={ 2    u>ba   ba>mf	     => 2.mf }=
={ 2	n>ba   ba>mfa mfa>mf => 2.mf }=
={ 3.mf	       mf>b	     => 3.b }=
={ 3.mf mf>ba  ba>b	     => 3.b }=
={ 3    u>mfa  mfa>b	     => 3.b }=
={ 3e   df>mfa mfa>ba ba>b   => 3.b }=
[THEN]

MULTI-RATIONALS MULTI-FLOATS AND [IF]
TESTING q>mf q>mfa qa>mf qa>mfa mf>q mf>qa mfa>q mfa>qa

={ 1/2	      q>mf	    => 0.5.mf }=
={ 1/2	      q>mfa mfa>mf  => 0.5.mf }=
={ 1 2 uu>qa  qa>mf	    => 0.5.mf }=
={ 1 2 nu>qa  qa>mfa mfa>mf => 0.5.mf }=
={ 5.mf	      mf>q	    => 5/1 }=
={ 5.mf	      mf>qa qa>q    => 5/1 }=
={ 5   u>mfa  mfa>q	    => 5/1 }=
={ 5e  df>mfa mfa>qa qa>q   => 5/1 }=
[THEN]

MULTI-INTEGERS RELIABLE-FLOATS AND [IF]
TESTING b>rf b>rfa ba>rf ba>rfa rf>b rf>ba rfa>b rfa>ba

={ 2.b	     b>rf	 => 2.rf }=
={ 2.b	     b>rfa	 => 2.rfa }=
={ 2   u>ba  ba>rf	 => 2.rf }=
={ 2   n>ba  ba>rfa	 => 2.rfa }=
={ 3.rf	     rf>b	 => 3.b }=
={ 3.rf	     rf>ba  ba>b => 3.b }=
={ 3  u>rfa  rfa>b	 => 3.b }=
={ 3e df>rfa rfa>ba ba>b => 3.b }=

TESTING rf>b-exp rf>ba-exp rfa>b-exp rfa>ba-exp

hex
rf" 0.ffp-1" rfconstant 0.ffp-1.rf
b" ff" get-default-rfprecision 8 - b2^u* bconstant tsigand.b
decimal
get-default-rfprecision 1+ negate constant texp

={ 0.ffp-1.rf	     rf>b-exp	     => tsigand.b texp }=
={ 0.ffp-1.rf	     rf>ba-exp  ba>b => tsigand.b texp }=
={ 0.ffp-1.rf rf>rfa rfa>b-exp	     => tsigand.b texp }=
={ 0.ffp-1.rf rf>rfa rfa>ba-exp ba>b => tsigand.b texp }=
[THEN]

MULTI-RATIONALS RELIABLE-FLOATS AND [IF]
TESTING q>rf  q>rfa  qa>rf  qa>rfa

={ 1/2	     q>rf	   => 0.5.rf }=
={ 1/2	     q>rfa  rfa>rf => 0.5.rf }=
={ 1 2 uu>qa qa>rf	   => 0.5.rf }=
={ 1 2 nu>qa qa>rfa rfa>rf => 0.5.rf }=
[THEN]

MULTI-FLOATS RELIABLE-FLOATS AND [IF]
TESTING mf>rf mf>rfa mfa>rf mfa>rfa rf>mf rf>mfa rfa>mf rfa>mfa

={ 0.5.mf	 mf>rf		=> 0.5.rf }=
={ 0.5.mf	 mf>rfa  rfa>rf => 0.5.rf }=
={ 0.5.mf mf>mfa mfa>rf		=> 0.5.rf }=
={ 0.5.mf mf>mfa mfa>rfa rfa>rf => 0.5.rf }=
={ 0.5.rf	 rf>mf		=> 0.5.mf }=
={ 0.5.rf	 rf>mfa  mfa>mf => 0.5.mf }=
={ 0.5.rf rf>rfa rfa>mf		=> 0.5.mf }=
={ 0.5.rf rf>rfa rfa>mfa mfa>mf => 0.5.mf }=
[THEN]


RANDOMS [IF]
TESTING RANDSTATE: free-randstate free-randstates
---
Stack effect only.
---

\ Should do nothing.
={ free-randstates => }=

={ RANDSTATE: 'trand align 'trand /GMP-RANDSTATE + cell+ aligned => here }=
={ 'trand /GMP-RANDSTATE aligned + @ => 0 }=  \ first in list

MULTI-INTEGERS [IF]
TESTING randinit-default randinit-mt randinit-lc-2^u randinit-lc-bits
[ELSE]
TESTING randinit-default randinit-mt randinit-lc-bits
[THEN]
---
Stack effect only.  Although the fact that CLEAR-RANDSTATE in
the following doesn't give a malloc double-free error indicates
that initialization happend.
---

={          'trand randinit-default => 'trand free-randstate }=
MULTI-INTEGERS [IF]
={ 3.b 5 64 'trand randinit-lc-2^u  => 'trand free-randstate }=
[THEN]
={ 128      'trand randinit-lc-bits => true 'trand free-randstate }=
={ 129      'trand randinit-lc-bits => false }= \ no clear, wasn't initialized

\ Mersenne twister for remaining tests.
={          'trand randinit-mt      => 'trand free-randstate }=

32 constant #tbits  \ bstack, mfstack tests

\ RESEED has no effect on the limbs versions, which do not use
\ 'trand.  Used for tests with several stacks.
: reseed  ( -- )  93 'trand u>seed ;
: ?.rands  ( s -- )
  VERBOSE @ IF reseed 4 0 DO 2dup evaluate LOOP THEN 2drop ;

MULTI-INTEGERS [IF]
TESTING b>seed u>seed rand-#bits>u rand[0,n-1]>u
[ELSE]
TESTING u>seed rand-#bits>u rand[0,n-1]>u
[THEN]

MULTI-INTEGERS [IF]
={ 'trand randinit-mt b" 93" 'trand b>seed => 'trand free-randstate }=
[THEN]
={ 'trand randinit-mt    93  'trand u>seed => }=

: ?.rand-32bits ( -- )
  VERBOSE @ IF 4 0 DO 32 'trand rand-#bits>u u. LOOP THEN ;

?." 32-bit randoms: " ={ ?.rand-32bits => }= ?.cr

: ?.rand<100 ( -- )
  VERBOSE @ IF 4 0 DO 100 'trand rand[0,n-1]>u u. LOOP THEN ;

MULTI-INTEGERS [IF]
={ b" 93" 'trand b>seed => }=
?." randoms < 100 reseeded with b = 93: " ={ ?.rand<100 => }= ?.cr
[THEN]

={ reseed => }=
?." randoms < 100 reseeded with u = 93: " ={ ?.rand<100 => }= ?.cr

MULTI-INTEGERS [IF]
TESTING rand-#bits>ba rand2-#bits>ba rand-#bits>b rand2-#bits>b

: "rand-32bits>ba."   s" #tbits 'trand rand-#bits>ba ?ba. space" ;
: "rand-32bits>b."    s" #tbits 'trand rand-#bits>b ?b. space" ;
: "rand2-32bits>ba."  s" #tbits 'trand rand2-#bits>ba ?ba. space" ;
: "rand2-32bits>b."   s" #tbits 'trand rand2-#bits>b ?b. space" ;

?." You should see 2 identical + 2 identical lines of random integers:

hex
={ "rand-32bits>ba."  ?.rands => }= ?.cr
={ "rand-32bits>b."   ?.rands => }= ?.cr
={ "rand2-32bits>ba." ?.rands => }= ?.cr
={ "rand2-32bits>b."  ?.rands => }= ?.cr
decimal
[THEN]

MULTI-FLOATS [IF]
TESTING rand-sbits>mfa rand2-limbs>mfa rand-sbits>mf rand2-limbs>mf

: "rand-32bits>mfa."  s" #tbits 'trand rand-sbits>mfa 8 ?(mfa.)" ;
: "rand-32bits>mf."   s" #tbits 'trand rand-sbits>mf 8 ?(mf.)" ;
: "rand2-limbs>mfa."  s" 1 1 rand2-limbs>mfa 8 ?(mfa.)" ;
: "rand2-limbs>mf."   s" 1 1 rand2-limbs>mf 8 ?(mf.)" ;

?." You should see 2 identical and 2 different lines of random hex floats:
hex
={ "rand-32bits>mfa." ?.rands => }= ?.cr
={ "rand-32bits>mf."  ?.rands => }= ?.cr
={ "rand2-limbs>mfa." ?.rands => }= ?.cr
={ "rand2-limbs>mf."  ?.rands => }= ?.cr
decimal
[THEN]

RELIABLE-FLOATS [IF]
TESTING rand>rfa rand>rf normalized-rand>rfa normalized-rand>rf

: "rand>rfa."  s" 'trand rand>rfa 8 ?(rfa.)" ;
: "rand>rf."   s" 'trand rand>rf  8 ?(rf.)" ;
: "normalized-rand>rfa."  s" 'trand normalized-rand>rfa 8 ?(rfa.)" ;
: "normalized-rand>rf."   s" 'trand normalized-rand>rf  8 ?(rf.)" ;

?." You should see 2 pairs of identical lines of random floats:
={ "rand>rfa." ?.rands => }= ?.cr
={ "rand>rf."  ?.rands => }= ?.cr
={ "normalized-rand>rfa." ?.rands => }= ?.cr
={ "normalized-rand>rf."  ?.rands => }= ?.cr
[THEN]

TESTING copy-rand free-randstates

t{ create 'trand2 /GMP-RANDSTATE allot -> }t
={ 'trand 'trand2 copy-rand => }=

: ?.rand2-32bits ( -- )
  VERBOSE @ IF 4 0 DO 32 'trand2 rand-#bits>u u. LOOP THEN ;

: "rand2-32bits>b."    s" #tbits 'trand2 rand-#bits>b ?b. space" ;

MULTI-INTEGERS [IF]
?." You should see 2 identical lines of random integers:
={ "rand-32bits>b."  ?.rands => }= ?.cr
={ "rand2-32bits>b." ?.rands => }= ?.cr
[THEN]

\ stack effect only
={ free-randstates 'trand2 free-randstate => }=
[THEN]  \ RANDOMS


IFORTH-HOST 0= [IF]
TESTING exceptions

\ enough for 4 input characters or 3 output characters with
\ possible sign (b, q, or \ mf signicand)
1 cells 5 + to /MZBUF
={ /MZBUF 1+ ' ?MZBUF CATCH nip => -2 }=

MULTI-INTEGERS [IF]
={ b" 1234" => b" 1234" }=
?." You should see an integer: "
={  123 n>b ' ?b. CATCH => 0 ?.cr }=
t{ 1234 n>b ' b.  CATCH -> -2 }t
\ Uncomment to see exception.
\ b.
\ b" 1q"
bdrop
[THEN]

MULTI-RATIONALS [IF]
?." You should see a rational: "
={  1 5 nu>q ' ?q. CATCH => 0 ?.cr }=
t{ 12 5 nu>q '  q. CATCH -> -2 }t
\ Uncomment to see exception.
\ q.
\ q" 1:2"
qdrop
[THEN]

MULTI-FLOATS [IF]
={ 123 u>mfa 3 mfa ' >MFSTR CATCH         => 0 }=
={ 123 u>mfa 4 mfa ' >MFSTR CATCH nip nip => -2 }=
\ Uncomment to see exception.
\ mf" 38p3"
[THEN]

RELIABLE-FLOATS [IF]
={ 123 u>rfa 3 rfa ' >RFSTR CATCH         => 0 }=
={ 123 u>rfa 4 rfa ' >RFSTR CATCH nip nip => -2 }=
\ Uncomment to see exception.
\ rf" 38p3"

\ Exceptions in NORMALIZED-RAND>RFA and NORMALIZED-RAND>RF
\ should rarely occur, according to the mpfr manual for
\ mpfr_urandomb().

[THEN]

DEFAULT-/MZBUF to /MZBUF
[THEN]  \ IFORTH-HOST

\ cleanup
MULTI-INTEGERS  [IF] free-brecs  BSPACE free throw  free-bconstants  [THEN]
MULTI-RATIONALS [IF] free-qrecs  QSPACE free throw  free-qconstants  [THEN]
MULTI-FLOATS    [IF] free-mfrecs MFSPACE free throw free-mfconstants [THEN]
RELIABLE-FLOATS [IF] free-rfrecs RFSPACE free throw free-rfconstants [THEN]

?.errors GFORTH-HOST [IF] ?.cr [THEN]
