/**
 *       Title:  GMP MP 5.0.1 macros for iForth
 *        File:  libgmp_macros.c
 *    Log file:  gmpfr.log
 *  Test files:  gmpfr-test.fs
 *      Author:  David N. Williams
 *     License:  LGPL
 *     Version:  0.8.4
 *     Started:  February 28, 2011 
 *     Revised:  May 3, 2011
 *
 * For the sake of the LGPL, any part of this library not
 * derived from the GNU Multiple Precision library is
 *
 * Copyright (C) 2011 by David N. Williams
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later
 * version.
 *
 * This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU Library General Public License for moref
 * details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA.
 *
 * If you take advantage of the option in the LGPL to put a
 * particular version of this library under the GPL, the author
 * would regard it as polite if you would put any direct
 * modifications under the LGPL as well, and include a copy of
 * this request near the beginning of the modified library
 * source.  A "direct modification" is one that enhances or
 * extends the library in line with its original concept, as
 * opposed to developing a distinct application or library which
 * might use it.
 *
 * This library wraps gmp macros into callable functions, for
 * the iForth dynlibs interface.
 */

#include <stdio.h>
#ifdef ABI_64
#include <gmp64.h>
#else
#include <gmp32.h>
#endif

#define slong    signed long int
#define ulong    unsigned long int

/*** Wrappers for gmp macros ***/ 

slong mpz_sgn_macro (mpz_ptr op) { return mpz_sgn (op); }

slong mpf_sgn_macro (mpf_ptr op) { return mpf_sgn (op); }

slong mpq_sgn_macro (mpq_ptr op) { return mpq_sgn (op); }

slong mpz_cmp_si_macro (mpz_ptr op1, slong op2)
{ return mpz_cmp_si (op1, op2); }

slong mpz_cmp_ui_macro (mpz_ptr op1, ulong op2)
{ return mpz_cmp_ui (op1, op2); }

slong mpz_odd_p_macro (mpz_ptr op) { return mpz_odd_p (op); }

slong mpz_even_p_macro (mpz_ptr op) { return mpz_odd_p (op); }

slong mpq_cmp_ui_macro (mpq_ptr op1, ulong num2, ulong den2)
{ return mpq_cmp_ui (op1, num2, den2); }

slong mpq_cmp_si_macro (mpq_ptr op1, slong num2, ulong den2)
{ return mpq_cmp_si (op1, num2, den2); }

mpz_srcptr mpq_numref_macro (mpq_ptr op) { return mpq_numref (op); }

mpz_srcptr mpq_denref_macro (mpq_ptr op) { return mpq_denref (op); }

slong sizeof_mpz (void) { return sizeof (mpz_t); }
slong sizeof_mpq (void) { return sizeof (mpq_t); }
slong sizeof_mpf (void) { return sizeof (mpf_t); }
slong bits_per_mp_limb (void) { return mp_bits_per_limb; }
slong sizeof_gmp_randstate (void) { return sizeof (gmp_randstate_t); }
const char * gmp_version_const (void) { return gmp_version; }

/*** Wrappers for gmp functions that need to return signed 64 bits ***/

slong mpz64_cmp (mpz_ptr op1, mpz_ptr op2)
{ return (slong) mpz_cmp (op1, op2); }

slong mpz64_cmp_d (mpz_ptr op1, double op2)
{ return (slong) mpz_cmp_d (op1, op2); }

slong mpz64_cmpabs (mpz_ptr op1, mpz_ptr op2)
{ return (slong) mpz_cmpabs (op1, op2); }

slong mpz64_cmpabs_d (mpz_ptr op1, double op2)
{ return (slong) mpz_cmpabs_d (op1, op2); }

slong mpz64_cmpabs_ui (mpz_ptr op1, ulong op2)
{ return (slong) mpz_cmpabs_ui (op1, op2); }

slong mpz64_jacobi (mpz_ptr a, mpz_ptr b)
{ return (slong) mpz_jacobi (a, b); }

slong mpz64_legendre (mpz_ptr a, mpz_ptr b)
{ return (slong) mpz_jacobi (a, b); }

slong mpz64_kronecker (mpz_ptr a, mpz_ptr b)
{ return (slong) mpz_jacobi (a, b); }

slong mpz64_kronecker_si (mpz_ptr a, slong b)
{ return (slong) mpz_kronecker_si (a, b); }

slong mpz64_kronecker_ui (mpz_ptr a, ulong b)
{ return (slong) mpz_kronecker_ui (a, b); }

slong mpz64_si_kronecker (slong a, mpz_ptr b)
{ return (slong) mpz_si_kronecker (a, b); }

slong mpz64_ui_kronecker (ulong a, mpz_ptr b)
{ return (slong) mpz_ui_kronecker (a, b); }

slong mpq64_cmp (mpq_ptr op1, mpq_ptr op2)
{ return (slong) mpq_cmp (op1, op2); }

slong mpf64_cmp (mpf_ptr op1, mpf_ptr op2)
{ return (slong) mpf_cmp (op1, op2); }

slong mpf64_cmp_d (mpf_ptr op1, double op2)
{ return (slong) mpf_cmp_d (op1, op2); }

slong mpf64_cmp_ui (mpf_ptr op1, ulong op2)
{ return (slong) mpf_cmp_ui (op1, op2); }

slong mpf64_cmp_si (mpf_ptr op1, slong op2)
{ return (slong) mpf_cmp_si (op1, op2); }
