(*
 * LANGUAGE    : Forth
 * PROJECT     : iForth mpfr binding
 * DESCRIPTION : Interface for libmpfr
 * CATEGORY    : Multiple precision
 * AUTHOR      : David N. Williams
 * STARTED     : 2011-04-21
 * REVISED     : 2011-04-23
 * LICENSE     : LGPL
 *)

	NEEDS -dynlibs

	REVISION -mpfrlib "--- MPFR 3.0.1 interface      Version 0.8.4 ---"

\ Library: /usr/local/lib/libgmp.dylib
\ Library: /usr/local/lib/libgmp_macros.dylib
Library: libmpfr.dylib
Library: libmpfr_macros.dylib

(*
Lines that are commented out containing only C prototypes are
currently not implemented.  Some "AliasedExtern:" lines are
commented out because they require special handling, either for
libgmp macros, or for int returns that need to be 64 bits. 
Special handling is done near at end of this file.
*)

[UNDEFINED] BITS/CELL [IF]
s" ADDRESS-UNIT-BITS" environment?
  0= [IF] cr .( ***Can't determine BITS/CELL) ABORT [THEN]
  1 cells * constant BITS/CELL [THEN]

BITS/CELL 32 <> BITS/CELL 64 <> and
[IF] cr .( ***BITS/CELL must be 32 or 64) ABORT [THEN]

\ libmpfr 3.0.1 functions

\ 5.1 Initialization

AliasedExtern: mpfr_init	void mpfr_init	(a) ;
AliasedExtern: mpfr_init2	void mpfr_init2 (a, n);
AliasedExtern: mpfr_clear	void mpfr_clear (a);
AliasedExtern: mpfr_set_default_prec void mpfr_set_default_prec (n);
AliasedExtern: mpfr_get_default_prec int  mpfr_get_default_prec (void);
AliasedExtern: mpfr_set_prec	 void mpfr_set_prec (a, n);
\ see LIBMPFR MACROS
\ AliasedExtern: mpfr_get_prec	int mpfr_get_prec (a);

\ 5.2 Assignment
\ see LIBMPFR MACROS
\ c-function mpfr_set		int mpfr_set	   (a, a, n)
AliasedExtern: mpfr_set_ui	int mpfr_set_ui	   (a, n, n);
AliasedExtern: mpfr_set_si	int mpfr_set_si	   (a, n, n);
AliasedExtern: mpfr_set_uj	int __gmpfr_set_uj (a, n, n);
AliasedExtern: mpfr_set_sj	int __gmpfr_set_sj (a, n, n);
AliasedExtern: mpfr_set_flt 	int mpfr_set_flt   (a, double op, n);
AliasedExtern: mpfr_set_d	int mpfr_set_d	   (a, double op, n);
\ c-function mpfr_set_ld	int mpfr_set_ld	   (a, ld, n);

AliasedExtern: mpfr_set_z	int mpfr_set_z (a, a, n);
AliasedExtern: mpfr_set_q	int mpfr_set_q (a, a, n);
AliasedExtern: mpfr_set_f	int mpfr_set_f (a, a, n);

AliasedExtern: mpfr_set_ui_2exp	int mpfr_set_ui_2exp	(a, n, n, n);
AliasedExtern: mpfr_set_si_2exp	int mpfr_set_si_2exp	(a, n, n, n);
AliasedExtern: mpfr_set_uj_2exp	int __gmpfr_set_uj_2exp	(a, n, n, n);
AliasedExtern: mpfr_set_sj_2exp	int __gmpfr_set_sj_2exp	(a, n, n, n);
AliasedExtern: mpfr_set_z_2exp	int  mpfr_set_z_2exp	(a, a, n, n);

AliasedExtern: mpfr_set_str	int mpfr_set_str (a, a, n, n);
AliasedExtern: mpfr_strtofr	int mpfr_strtofr (a, a, a, n, n);

AliasedExtern: mpfr_set_nan	void mpfr_set_nan  (a);
AliasedExtern: mpfr_set_inf	void mpfr_set_inf  (a, n);
AliasedExtern: mpfr_set_zero	void mpfr_set_zero (a, n);

AliasedExtern: mpfr_swap	void mpfr_swap (a, a);

\ 5.3 Combined initialization and assignment

0 [IF]  \ see LIBMPFR MACROS
AliasedExtern: mpfr_init_set	int mpfr_init_set    (a, a, n);
AliasedExtern: mpfr_init_set_ui	int mpfr_init_set_ui (a, n, n);
AliasedExtern: mpfr_init_set_si	int mpfr_init_set_si (a, n, n);
AliasedExtern: mpfr_init_set_d	int  mpfr_init_set_d (a, double op, n);
\ AliasedExtern: mpfr_init_set_ld int mpfr_init_set_ld (a, long double, n);
AliasedExtern: mpfr_init_set_z	int  mpfr_init_set_z (a, a, n);
AliasedExtern: mpfr_init_set_q	int  mpfr_init_set_q (a, a, n);
AliasedExtern: mpfr_init_set_f	int  mpfr_init_set_f (a, a, n);
[THEN]

AliasedExtern: mpfr_init_set_str int mpfr_init_set_str (a, a, n, n);

\ 5.4 Conversion

AliasedExtern: mpfr_get_flt	double mpfr_get_flt	(a, n);
AliasedExtern: mpfr_get_d	double mpfr_get_d	(a, n);
\ AliasedExtern: mpfr_get_ld	long double mpfr_get_ld	(a, n);

AliasedExtern: mpfr_get_si	int mpfr_get_si	   (a, n);
AliasedExtern: mpfr_get_ui	int mpfr_get_ui	   (a, n);
AliasedExtern: mpfr_get_sj	int __gmpfr_mpfr_get_sj (a, n);
AliasedExtern: mpfr_get_uj	int __gmpfr_mpfr_get_uj (a, n);

AliasedExtern: mpfr_get_d_2exp	  double mpfr_get_d_2exp (a, a, n);
\ AliasedExtern: mpfr_get_ld_2exp long double mpfr_get_ld_2exp (a, a, n);

AliasedExtern: mpfr_get_z_2exp	int mpfr_get_z_2exp (a, a);

AliasedExtern: mpfr_get_z	int mpfr_get_z (a, a, n);
AliasedExtern: mpfr_get_f	int mpfr_get_f (a, a, n);

AliasedExtern: mpfr_get_str 	void* mpfr_get_str (a, a, n, n, a, n);

AliasedExtern: mpfr_free_str	void mpfr_free_str (a);

AliasedExtern: mpfr_fits_ulong_p   int mpfr_fits_ulong_p   (a, n);
AliasedExtern: mpfr_fits_slong_p   int mpfr_fits_slong_p   (a, n);
AliasedExtern: mpfr_fits_uint_p	   int mpfr_fits_uint_p	   (a, n);
AliasedExtern: mpfr_fits_sint_p	   int mpfr_fits_sint_p	   (a, n);
AliasedExtern: mpfr_fits_ushort_p  int mpfr_fits_ushort_p  (a, n);
AliasedExtern: mpfr_fits_sshort_p  int mpfr_fits_sshort_p  (a, n);
AliasedExtern: mpfr_fits_uintmax_p int mpfr_fits_uintmax_p (a, n);
AliasedExtern: mpfr_fits_intmax_p  int mpfr_fits_intmax_p  (a, n);

\ 5.5 Basic Arithmetic

AliasedExtern: mpfr_add		int mpfr_add	(a, a, a, n);
AliasedExtern: mpfr_add_ui	int mpfr_add_ui	(a, a, n, n);
AliasedExtern: mpfr_add_si	int mpfr_add_si	(a, a, n, n);
AliasedExtern: mpfr_add_d	int mpfr_add_d	(a, a, double op, n);
AliasedExtern: mpfr_add_z	int mpfr_add_z	(a, a, a, n);
AliasedExtern: mpfr_add_q	int mpfr_add_q	(a, a, a, n);

AliasedExtern: mpfr_sub		int mpfr_sub	(a, a, a, n);
AliasedExtern: mpfr_ui_sub	int mpfr_ui_sub	(a, n, a, n);
AliasedExtern: mpfr_sub_ui	int mpfr_sub_ui	(a, a, n, n);
AliasedExtern: mpfr_si_sub	int mpfr_si_sub (a, n, a, n);
AliasedExtern: mpfr_sub_si	int mpfr_sub_si	(a, a, n, n);
AliasedExtern: mpfr_d_sub	int mpfr_d_sub	(a, double op, a, n);
AliasedExtern: mpfr_sub_d	int mpfr_sub_d	(a, a, double op, n);
AliasedExtern: mpfr_sub_z	int mpfr_sub_z	(a, a, a, n);
AliasedExtern: mpfr_sub_q	int mpfr_sub_q	(a, a, a, n);

AliasedExtern: mpfr_mul		int mpfr_mul	(a, a, a, n);
AliasedExtern: mpfr_mul_ui	int mpfr_mul_ui	(a, a, n, n);
AliasedExtern: mpfr_mul_si	int mpfr_mul_si	(a, a, n, n);
AliasedExtern: mpfr_mul_d	int mpfr_mul_d	(a, a, double op, n);
AliasedExtern: mpfr_mul_z	int mpfr_mul_z	(a, a, a, n);
AliasedExtern: mpfr_mul_q	int mpfr_mul_q	(a, a, a, n);

AliasedExtern: mpfr_sqr		int mpfr_sqr (a, a, n);

AliasedExtern: mpfr_div		int mpfr_div	(a, a, a, n);
AliasedExtern: mpfr_ui_div	int mpfr_ui_div	(a, n, a, n);
AliasedExtern: mpfr_div_ui	int mpfr_div_ui	(a, a, n, n);
AliasedExtern: mpfr_si_div	int mpfr_si_div	(a, n, a, n);
AliasedExtern: mpfr_div_si	int mpfr_div_si	(a, a, n, n);
AliasedExtern: mpfr_d_div	int mpfr_d_div	(a, double op, a, n);
AliasedExtern: mpfr_div_d	int mpfr_div_d	(a, a, double op, n);
AliasedExtern: mpfr_div_z	int mpfr_div_z	(a, a, a, n);
AliasedExtern: mpfr_div_q	int mpfr_div_q	(a, a, a, n);

AliasedExtern: mpfr_sqrt	int mpfr_sqrt	  (a, a, n);
AliasedExtern: mpfr_sqrt_ui	int mpfr_sqrt_ui  (a, n, n);
AliasedExtern: mpfr_rec_sqrt	int mpfr_rec_sqrt (a, a, n);
AliasedExtern: mpfr_cbrt	int mpfr_cbrt	  (a, a, n);
AliasedExtern: mpfr_root	int mpfr_root	  (a, a, n, n);

AliasedExtern: mpfr_pow		int mpfr_pow	   (a, a, a, n);
AliasedExtern: mpfr_pow_ui	int mpfr_pow_ui	   (a, a, n, n);
AliasedExtern: mpfr_pow_si	int mpfr_pow_si    (a, a, n, n);
AliasedExtern: mpfr_pow_z	int mpfr_pow_z	   (a, a, a, n);
AliasedExtern: mpfr_ui_pow_ui	int mpfr_ui_pow_ui (a, n, n, n);
AliasedExtern: mpfr_ui_pow	int mpfr_ui_pow	   (a, n, a, n);

AliasedExtern: mpfr_neg		int mpfr_neg (a, a, n);
AliasedExtern: mpfr_abs		int mpfr_abs (a, a, n);

AliasedExtern: mpfr_dim		int mpfr_dim (a, a, a, n);

AliasedExtern: mpfr_mul_2ui	int mpfr_mul_2ui (a, a, n, n);
AliasedExtern: mpfr_mul_2si	int mpfr_mul_2si (a, a, n, n);
AliasedExtern: mpfr_div_2ui	int mpfr_div_2ui (a, a, n, n);
AliasedExtern: mpfr_div_2si	int mpfr_div_2si (a, a, n, n);

\ 5.6 Comparison

AliasedExtern: mpfr_cmp		int mpfr_cmp	(a, a);
AliasedExtern: mpfr_cmp_ui	int mpfr_cmp_ui	(a, n);
AliasedExtern: mpfr_cmp_si	int mpfr_cmp_si	(a, n);
AliasedExtern: mpfr_cmp_d	int mpfr_cmp_d	(a, double op);
\ AliasedExtern: mpfr_cmp_ld	int mpfr_cmp_ld	(a, long double);
AliasedExtern: mpfr_eq		int mpfr_eq	(a, a, n);
AliasedExtern: mpfr_cmp_z	int mpfr_cmp_z	(a, a);
AliasedExtern: mpfr_cmp_q	int mpfr_cmp_q	(a, a);
AliasedExtern: mpfr_cmp_f	int mpfr_cmp_f	(a, a);

AliasedExtern: mpfr_cmp_ui_2exp	int mpfr_cmp_ui_2exp (a, n, n);
AliasedExtern: mpfr_cmp_si_2exp	int mpfr_cmp_si_2exp (a, n, n);

AliasedExtern: mpfr_cmpabs	int mpfr_cmpabs	(a, a);

0 [IF]  \ see LIBMPFR MACROS
AliasedExtern: mpfr_nan_p	int mpfr_nan_p	   (a);
AliasedExtern: mpfr_inf_p	int mpfr_inf_p	   (a);
AliasedExtern: mpfr_number_p	int mpfr_number_p  (a);
AliasedExtern: mpfr_zero_p	int mpfr_zero_p	   (a);
AliasedExtern: mpfr_regular_p	int mpfr_regular_p (a);
AliasedExtern: mpfr_sgn		int mpfr_sgn	   (a);
[THEN]

AliasedExtern: mpfr_greater_p	   int mpfr_greater_p	   (a, a);
AliasedExtern: mpfr_greaterequal_p int mpfr_greaterequal_p (a, a);
AliasedExtern: mpfr_less_p	   int mpfr_less_p	   (a, a);
AliasedExtern: mpfr_lessequal_p	   int mpfr_lessequal_p	   (a, a);
AliasedExtern: mpfr_equal_p	   int mpfr_equal_p	   (a, a);
AliasedExtern: mpfr_lessgreater_p  int mpfr_lessgreater_p  (a, a);
AliasedExtern: mpfr_unordered_p	   int mpfr_unordered_p	   (a, a);


\ 5.7 Special Functions

AliasedExtern: mpfr_log		int mpfr_log	   (a, a, n);
AliasedExtern: mpfr_log2	int mpfr_log2	   (a, a, n);
AliasedExtern: mpfr_log10	int mpfr_log10	   (a, a, n);
AliasedExtern: mpfr_exp		int mpfr_exp	   (a, a, n);
AliasedExtern: mpfr_exp2	int mpfr_exp2	   (a, a, n);
AliasedExtern: mpfr_exp10	int mpfr_exp10	   (a, a, n);
AliasedExtern: mpfr_cos		int mpfr_cos	   (a, a, n);
AliasedExtern: mpfr_sin		int mpfr_sin	   (a, a, n);
AliasedExtern: mpfr_tan		int mpfr_tan	   (a, a, n);
AliasedExtern: mpfr_sin_cos	int mpfr_sin_cos   (a, a, a, n);
AliasedExtern: mpfr_sec		int mpfr_sec	   (a, a, n);
AliasedExtern: mpfr_csc		int mpfr_csc	   (a, a, n);
AliasedExtern: mpfr_cot		int mpfr_cot	   (a, a, n);
AliasedExtern: mpfr_acos	int mpfr_acos	   (a, a, n);
AliasedExtern: mpfr_asin	int mpfr_asin	   (a, a, n);
AliasedExtern: mpfr_atan	int mpfr_atan	   (a, a, n);
AliasedExtern: mpfr_atan2	int mpfr_atan2	   (a, a, a, n);
AliasedExtern: mpfr_cosh	int mpfr_cosh	   (a, a, n);
AliasedExtern: mpfr_sinh	int mpfr_sinh	   (a, a, n);
AliasedExtern: mpfr_tanh	int mpfr_tanh	   (a, a, n);
AliasedExtern: mpfr_sinh_cosh	int mpfr_sinh_cosh (a, a, a, n);
AliasedExtern: mpfr_sech	int mpfr_sech	   (a, a, n);
AliasedExtern: mpfr_csch	int mpfr_csch	   (a, a, n);
AliasedExtern: mpfr_coth	int mpfr_coth	   (a, a, n);
AliasedExtern: mpfr_acosh	int mpfr_acosh	   (a, a, n);
AliasedExtern: mpfr_asinh	int mpfr_asinh	   (a, a, n);
AliasedExtern: mpfr_atanh	int mpfr_atanh	   (a, a, n);
AliasedExtern: mpfr_fac_ui	int mpfr_fac_ui	   (a, n, n);
AliasedExtern: mpfr_log1p	int mpfr_log1p	   (a, a, n);
AliasedExtern: mpfr_expm1	int mpfr_expm1	   (a, a, n);
AliasedExtern: mpfr_eint	int mpfr_eint	   (a, a, n);
AliasedExtern: mpfr_li2		int mpfr_li2	   (a, a, n);
AliasedExtern: mpfr_gamma	int mpfr_gamma	   (a, a, n);
AliasedExtern: mpfr_lngamma	int mpfr_lngamma   (a, a, n);
AliasedExtern: mpfr_lgamma	int mpfr_lgamma	   (a, a, a, n);
AliasedExtern: mpfr_digamma	int mpfr_digamma   (a, a, n);
AliasedExtern: mpfr_zeta	int mpfr_zeta	   (a, a, n);
AliasedExtern: mpfr_zeta_ui	int mpfr_zeta_ui   (a, n, n);
AliasedExtern: mpfr_erf		int mpfr_erf	   (a, a, n);
AliasedExtern: mpfr_erfc	int mpfr_erfc	   (a, a, n);
AliasedExtern: mpfr_j0		int mpfr_j0	   (a, a, n);
AliasedExtern: mpfr_j1		int mpfr_j1	   (a, a, n);
AliasedExtern: mpfr_jn		int mpfr_jn	   (a, n, a, n);
AliasedExtern: mpfr_y0		int mpfr_y0	   (a, a, n);
AliasedExtern: mpfr_y1		int mpfr_y1	   (a, a, n);
AliasedExtern: mpfr_yn		int mpfr_yn	   (a, n, a, n);
AliasedExtern: mpfr_fma		int mpfr_fma	   (a, a, a, a, n);
AliasedExtern: mpfr_fms		int mpfr_fms	   (a, a, a, a, n);
AliasedExtern: mpfr_agm		int mpfr_agm	   (a, a, a, n);
AliasedExtern: mpfr_hypot	int mpfr_hypot	   (a, a, a, n);
AliasedExtern: mpfr_ai		int mpfr_ai	   (a, a, n);

AliasedExtern: mpfr_const_log2	  int mpfr_const_log2    (a, n);
AliasedExtern: mpfr_const_pi	  int mpfr_const_pi      (a, n);
AliasedExtern: mpfr_const_euler	  int mpfr_const_euler   (a, n);
AliasedExtern: mpfr_const_catalan int mpfr_const_catalan (a, n);

AliasedExtern: mpfr_free_cache	void mpfr_free_cache (void);

AliasedExtern: mpfr_sum		int mpfr_sum (a, a, n, n);

\ 5.8 Input and output

AliasedExtern: mpfr_out_str	int __gmpfr_out_str (a, n, n, a, n);
AliasedExtern: mpfr_inp_str	int __gmpfr_inp_str (a, a, n, n);

\ 5.9 Formatted Output

\ 5.10 Integer and Remainder Related Functions

AliasedExtern: mpfr_rint	int mpfr_rint  (a, a, n);
AliasedExtern: mpfr_ceil	int mpfr_ceil  (a, a);
AliasedExtern: mpfr_floor	int mpfr_floor (a, a);
AliasedExtern: mpfr_round	int mpfr_round (a, a);
AliasedExtern: mpfr_trunc	int mpfr_trunc (a, a);

AliasedExtern: mpfr_rint_ceil	int mpfr_rint_ceil  (a, a, n);
AliasedExtern: mpfr_rint_floor	int mpfr_rint_floor (a, a, n);
AliasedExtern: mpfr_rint_round	int mpfr_rint_round (a, a, n);
AliasedExtern: mpfr_rint_trunc	int mpfr_rint_trunc (a, a, n);

AliasedExtern: mpfr_frac	int mpfr_frac (a, a, n);

AliasedExtern: mpfr_modf	int mpfr_modf (a, a, a, n);

AliasedExtern: mpfr_fmod	int mpfr_fmod	   (a, a, a, n);
AliasedExtern: mpfr_remainder	int mpfr_remainder (a, a, a, n);
AliasedExtern: mpfr_remquo	int mpfr_remquo	   (a, a, a, a, n);

AliasedExtern: mpfr_integer_p	int mpfr_integer_p (a);

\ 5.11 Rounding Related Functions

AliasedExtern: mpfr_set_default_rounding_mode void mpfr_set_default_rounding_mode (n);
AliasedExtern: mpfr_get_default_rounding_mode int  mpfr_get_default_rounding_mode (void);

AliasedExtern: mpfr_prec_round	int mpfr_prec_round (a, n, n);
AliasedExtern: mpfr_can_round	int mpfr_can_round  (a, n, n, n, n);

AliasedExtern: mpfr_min_prec	int mpfr_min_prec (a);

\ AliasedExtern: mpfr_print_rnd_mode void * mpfr_print_rnd_mode (n);
AliasedExtern: mpfr_print_rnd_mode int mpfr_print_rnd_mode (n);

\ 5.12 Miscellaneous Functions

AliasedExtern: mpfr_nexttoward	void mpfr_nexttoward (a, a);
AliasedExtern: mpfr_nextabove	void mpfr_nextabove  (a);
AliasedExtern: mpfr_nextbelow	void mpfr_nextbelow  (a);

AliasedExtern: mpfr_min		int mpfr_min (a, a, a, n);
AliasedExtern: mpfr_max		int mpfr_max (a, a, a, n);

AliasedExtern: mpfr_urandomb	int mpfr_urandomb (a, a);
AliasedExtern: mpfr_urandom	int mpfr_urandom  (a, a, n);

\ see LIBMPFR MACROS
\ AliasedExtern: mpfr_get_exp	int mpfr_get_exp (a);
AliasedExtern: mpfr_set_exp	int mpfr_set_exp (a, n);

\ see LIBMPFR MACROS
\ AliasedExtern: mpfr_signbit	int mpfr_signbit (a);

AliasedExtern: mpfr_setsign	int mpfr_setsign (a, a, n, n);

\ see LIBMPFR MACROS
\ AliasedExtern: mpfr_copysign	int mpfr_copysign (a, a, a, n);

AliasedExtern: mpfr_get_version int mpfr_get_version (void);

\ 5.13 Exception Related Functions

AliasedExtern: mpfr_get_emin	int mpfr_get_emin (void);
AliasedExtern: mpfr_get_emax	int mpfr_get_emax (void);

AliasedExtern: mpfr_set_emin	int mpfr_set_emin (n);
AliasedExtern: mpfr_set_emax	int mpfr_set_emax (n);

AliasedExtern: mpfr_get_emin_min int mpfr_get_emin_min (void);
AliasedExtern: mpfr_get_emin_max int mpfr_get_emin_max (void);
AliasedExtern: mpfr_get_emax_min int mpfr_get_emax_min (void);
AliasedExtern: mpfr_get_emax_max int mpfr_get_emax_max (void);

AliasedExtern: mpfr_check_range	 int mpfr_check_range  (a, n, n);
AliasedExtern: mpfr_subnormalize int mpfr_subnormalize (a, n, n);

AliasedExtern: mpfr_clear_underflow  void mpfr_clear_underflow (void);
AliasedExtern: mpfr_clear_overflow   void mpfr_clear_overflow (void);
AliasedExtern: mpfr_clear_nanflag    void mpfr_clear_nanflag (void);
AliasedExtern: mpfr_clear_inexflag   void mpfr_clear_inexflag (void);
AliasedExtern: mpfr_clear_erangeflag void mpfr_clear_erangeflag (void);

AliasedExtern: mpfr_set_underflow  void mpfr_set_underflow (void);
AliasedExtern: mpfr_set_overflow   void mpfr_set_overflow (void);
AliasedExtern: mpfr_set_nanflag	   void mpfr_set_nanflag (void);
AliasedExtern: mpfr_set_inexflag   void mpfr_set_inexflag (void);
AliasedExtern: mpfr_set_erangeflag void mpfr_set_erangeflag (void);

AliasedExtern: mpfr_clear_flags	void mpfr_clear_flags (void);

AliasedExtern: mpfr_underflow_p	 int mpfr_underflow_p  (void);
AliasedExtern: mpfr_overflow_p	 int mpfr_overflow_p   (void);
AliasedExtern: mpfr_nanflag_p	 int mpfr_nanflag_p    (void);
AliasedExtern: mpfr_inexflag_p	 int mpfr_inexflag_p   (void);
AliasedExtern: mpfr_erangeflag_p int mpfr_erangeflag_p (void);

\ 5.14 Compatibility With MPF

AliasedExtern: mpfr_reldiff 	void mpfr_reldiff (a, a, a, n);

\ 5.15 Custom Interface
\ not implemented


\ libmpfr_macros

AliasedExtern: mpfr_init_set	int mpfr_init_set_macro	   (a, a, n);
AliasedExtern: mpfr_init_set_ui	int mpfr_init_set_ui_macro (a, n, n);
AliasedExtern: mpfr_init_set_si	int mpfr_init_set_si_macro (a, n, n);
AliasedExtern: mpfr_init_set_d	int mpfr_init_set_d_macro  (a, double op, n);
AliasedExtern: mpfr_init_set_z	int mpfr_init_set_z_macro  (a, a, n);
AliasedExtern: mpfr_init_set_q	int mpfr_init_set_q_macro  (a, a, n);
AliasedExtern: mpfr_init_set_f	int mpfr_init_set_f_macro  (a, a, n);

AliasedExtern: mpfr_get_prec	int mpfr_get_prec_macro	(a);
AliasedExtern: mpfr_get_exp	int mpfr_get_exp_macro	(a);

AliasedExtern: mpfr_set		int mpfr_set_macro	(a, a, n);
AliasedExtern: mpfr_copysign	int mpfr_copysign_macro	(a, a, a, n);
AliasedExtern: mpfr_signbit	int mpfr_signbit_macro	(a);

AliasedExtern: mpfr_nan_p	int mpfr_nan_p_macro	 (a);
AliasedExtern: mpfr_inf_p	int mpfr_inf_p_macro	 (a);
AliasedExtern: mpfr_number_p	int mpfr_number_p_macro	 (a);
AliasedExtern: mpfr_zero_p	int mpfr_zero_p_macro	 (a);
AliasedExtern: mpfr_regular_p	int mpfr_regular_p_macro (a);
AliasedExtern: mpfr_sgn		int mpfr_sgn_macro	 (a);

AliasedExtern: /MPFR		int sizeof_mpfr (void);

AliasedExtern: MPFR_PREC_MIN	 int MPFR_PREC_MIN_macro     (void);
AliasedExtern: MPFR_PREC_MAX	 int MPFR_PREC_MAX_macro     (void);
AliasedExtern: MPFR_EMAX_DEFAULT int MPFR_EMAX_DEFAULT_macro (void);
AliasedExtern: MPFR_EMIN_DEFAULT int MPFR_EMIN_DEFAULT_macro (void);

AliasedExtern: MPFR_RNDN	int MPFR_RNDN_macro  (void);
AliasedExtern: MPFR_RNDZ	int MPFR_RNDZ_macro  (void);
AliasedExtern: MPFR_RNDU	int MPFR_RNDU_macro  (void);
AliasedExtern: MPFR_RNDD	int MPFR_RNDD_macro  (void);
AliasedExtern: MPFR_RNDA	int MPFR_RNDA_macro  (void);
AliasedExtern: MPFR_RNDF	int MPFR_RNDF_macro  (void);
AliasedExtern: MPFR_RNDNA	int MPFR_RNDNA_macro (void);
