(   Title:  A utiilty for testing words that use the mfstack for
	    GMP multiple precision floating-point numbers
     File:  mftester.fs
 Log file:  gmpfr.log
   Author:  David N. Williams
  License:  Public Domain, John Hayes
  Version:  1.5.0
  Started:  March 2, 2011
  Revised:  April 13, 2011

For any code derived from John Hayes' tester program:
)
\ (C) 1995 JOHNS HOPKINS UNIVERSITY / APPLIED PHYSICS LABORATORY
\ MAY BE DISTRIBUTED FREELY AS LONG AS THIS COPYRIGHT NOTICE REMAINS.
(
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

The rest is public domain.
)
BASE @ DECIMAL

\ *** ERRORS

VARIABLE MFT-ERROR-XT
: MFT-ERROR MFT-ERROR-XT @ EXECUTE ;  \ vectored error reporting

VARIABLE mft-error-index  \ holds array index|-1 after error

: MFT-ERROR-DEFAULT   ( c-addr u -- )
(
Display an error message followed by the line that had the
error.
)
  TYPE SOURCE TYPE CR ;
' MFT-ERROR-DEFAULT MFT-ERROR-XT !

\ *** COMPARISONS

VARIABLE MFT-TEST=-XT

: MFT-TEST=  (mf: x y -- ) ( -- flag )  MFT-TEST=-XT @ EXECUTE ;

\ The sign of these tolerances must be plus.
mf" 0e0"   MFCONSTANT MFT-ABS-ERROR-DEFAULT
mf" 1e-12" MFCONSTANT MFT-REL-ERROR-DEFAULT
MFVARIABLE MFT-ABS-ERROR   MFT-ABS-ERROR-DEFAULT MFT-ABS-ERROR mf!
MFVARIABLE MFT-REL-ERROR   MFT-REL-ERROR-DEFAULT MFT-REL-ERROR mf!

: MFT-DATUM=  (mf: x y -- ) ( -- flag )
(
Leave TRUE if the two mfloats have equivalent internal
representations.  Else leave FALSE.
)
  mf= ;

\ |x - y| < eps
: MFT-ABS=  (mf: x y -- ) ( -- flag )
  mf- |mf| MFT-ABS-ERROR mf@ mf<  ;

\ |m - r| < eps * |r|
: MFT-REL=  (mf: meas ref -- ) ( -- flag )
  mfrel- MFT-REL-ERROR mf@ mf< ;

: MFT-REL0=  (mf: meas ref -- ) ( -- flag )
  mfdup mf0= IF MFT-ABS= ELSE MFT-REL= THEN ;

: SET-MFT-MODE-EXACT ( -- )  ['] MFT-DATUM= MFT-TEST=-XT ! ;
: SET-MFT-MODE-ABS   ( -- )  ['] MFT-ABS=   MFT-TEST=-XT ! ;
: SET-MFT-MODE-REL   ( -- )  ['] MFT-REL=   MFT-TEST=-XT ! ;
: SET-MFT-MODE-REL0  ( -- )  ['] MFT-REL0=  MFT-TEST=-XT ! ;

SET-MFT-MODE-EXACT


\ *** ARRAYS
(
The first array slot has index zero.
)

\ USER CONFIG DEFAULTS
[UNDEFINED] MAX-MFT-INITIALS [IF] 32 CONSTANT MAX-MFT-INITIALS [THEN]
[UNDEFINED] MAX-MFT-RESULTS  [IF] 32 CONSTANT MAX-MFT-RESULTS  [THEN]

\ Arrays begin with a cell-sized count.
CREATE mft-initials MAX-MFT-INITIALS 1+ CELLS ALLOT
CREATE mft-results  MAX-MFT-RESULTS  1+ CELLS ALLOT
CREATE mft-goals    MAX-MFT-RESULTS  1+ CELLS ALLOT

: 0MFT-A  ( +n 'array -- )  \ assume n<>0
(
Initialize array mf variables for garbage collection.
)
  swap 1+ cells over + swap DO 0 i ! 1 cells +LOOP ;

MAX-MFT-INITIALS mft-initials 0MFT-A
MAX-MFT-RESULTS  mft-results  0MFT-A
MAX-MFT-RESULTS  mft-goals    0MFT-A

\ Overflow checking to be done elsewhere.
: mft-a!   ( +n 'array -- ) (mf: f_[n-1] ... f_0 -- )
  2dup ! cell+ swap ( 'array n) dup
  IF   cells over + swap
       DO i mf! 1 cells +LOOP 
  ELSE 2drop THEN ;

: mft-a@   ( 'array -- +n ) (mf: -- f_[n-1] ... f_0 )
  dup @ dup >r ( 'array n r: n)
  IF   cell+  r@ 1- cells over + 
       DO i mf@ -1 cells +LOOP
  ELSE drop THEN r> ;

[UNDEFINED] NIP [IF] : NIP  ( a b -- b)  swap drop ; [THEN]

: mft-a<>  ( 'array1 'array2 -- [index+1]|-1|0 )
(
Nonzero return for errors:
  -1       unequal array size
  index+1  deepest unequal array elements
)
  dup @ swap cell+ >r >r  ( 'array1 r: 'f2 #left )
  dup @ r@ <> IF 2r> 2drop drop -1 EXIT THEN
  ( 'array1) dup cell+
  BEGIN  r@ ( 'array1 'f1 #left)
  WHILE  ( 'f1) dup mf@ cell+
         r> ( #left) 1- r>
         ( 'f2) dup mf@ cell+ >r ( #left) >r
         MFT-TEST=
  WHILE
  REPEAT ( 'array1 'f1) drop @ ( len) 2r> nip - ( index+1)
  ELSE   ( 'array1 'f1) 2drop 2r> 2drop 0
  THEN ;

\ *** TESTS

: MF{   ( <mfinitials> -- )
(
Save the mfstack <mfinitials> for restoration by }MF. 
Clear <mfresults> and <mfgoals> storage to put it in a defined
state for }MF in case MF-> is missing.
)
  mfdepth dup MAX-MFT-INITIALS > ABORT" TOO MANY MF INITIALS"
  mft-initials mft-a!   0 mft-results !   0 mft-goals ! ;

: MF->  ( <mfresults> -- )
(
Record the depth and contents of the mfstack.
)
  mfdepth dup MAX-MFT-RESULTS > ABORT" TOO MANY MF RESULTS"
  mft-results mft-a! ;

: }MF   ( <mfgoals> -- <mfinitials> )
(
Save the mfstack <mfgoals>, compare them with the saved
<mfresults>, and restore the <mfinitials>.
)
  mfdepth dup MAX-MFT-RESULTS > ABORT" TOO MANY MF GOALS"
  mft-goals mft-a!
  mft-results mft-goals mft-a<> dup mft-error-index !
  ?DUP IF -1 =
    IF   S" WRONG NUMBER OF MF RESULTS: "
    ELSE -1 mft-error-index +! S" INCORRECT MF RESULT: " THEN
    MFT-ERROR
  THEN
  mft-initials mft-a@ ( len) drop ;

BASE !

\ *** END of mftester.fs
