(   Title:  A utiilty for testing words that use the qstack for
	    GMP multiple precision rational numbers [quotients]
     File:  qtester.fs
 Log file:  gmpfr.log
   Author:  David N. Williams
  License:  Public Domain, John Hayes
  Version:  1.5.0
  Started:  March 10, 2011
  Revised:  March 19, 2011

For any code derived from John Hayes' tester program:
)
\ (C) 1995 JOHNS HOPKINS UNIVERSITY / APPLIED PHYSICS LABORATORY
\ MAY BE DISTRIBUTED FREELY AS LONG AS THIS COPYRIGHT NOTICE REMAINS.
(
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

The rest is public domain.
)
BASE @ DECIMAL

\ *** ERRORS

VARIABLE QT-ERROR-XT
: QT-ERROR QT-ERROR-XT @ EXECUTE ;  \ vectored error reporting

VARIABLE qt-error-index  \ holds array index|-1 after error

: QT-ERROR-DEFAULT   ( c-addr u -- )
(
Display an error message followed by the line that had the
error.
)
  TYPE SOURCE TYPE CR ;
' QT-ERROR-DEFAULT QT-ERROR-XT !

\ *** ARRAYS
(
The first array slot has index zero.
)

\ USER CONFIG DEFAULTS
[UNDEFINED] MAX-QT-INITIALS [IF] 32 CONSTANT MAX-QT-INITIALS [THEN]
[UNDEFINED] MAX-QT-RESULTS  [IF] 32 CONSTANT MAX-QT-RESULTS  [THEN]

\ Arrays begin with a cell-sized count.
CREATE qt-initials MAX-QT-INITIALS 1+ CELLS ALLOT
CREATE qt-results  MAX-QT-RESULTS  1+ CELLS ALLOT
CREATE qt-goals    MAX-QT-RESULTS  1+ CELLS ALLOT

: 0QT-A  ( +n 'array -- )  \ assume n<>0
(
Initialize array q variables for garbage collection.
)
  swap 1+ cells over + swap DO 0 i ! 1 cells +LOOP ;

MAX-QT-INITIALS qt-initials 0QT-A
MAX-QT-RESULTS  qt-results  0QT-A
MAX-QT-RESULTS  qt-goals    0QT-A

\ Overflow checking to be done elsewhere.
: qt-a!   ( +n 'array -- ) ( q: q_[n-1] ... q_0 -- )
  2dup ! cell+ swap ( 'array n) dup
  IF   cells over + swap
       DO i q! 1 cells +LOOP 
  ELSE 2drop THEN ;

: qt-a@   ( 'array -- +n ) (q: -- q_[n-1] ... q_0 )
  dup @ dup >r ( 'array n r: n)
  IF   cell+ r@ 1- cells over + 
       DO i q@ -1 cells +LOOP
  ELSE drop THEN r> ;

[UNDEFINED] NIP [IF] : NIP  ( a b -- b)  swap drop ; [THEN]

: qt-a<>  ( 'array1 'array2 -- [index+1]|-1|0 )
(
Nonzero return for errors:
  -1       unequal array size
  index+1  deepest unequal array elements
)
  dup @ swap cell+ >r >r  ( 'array1 r: 'q2 #left )
  dup @ r@ <> IF 2r> 2drop drop -1 EXIT THEN
  ( 'array1) dup cell+
  BEGIN  r@ ( 'array1 'q1 #left)
  WHILE  ( 'q1) dup q@ cell+
         r> ( #left) 1- r>
         ( 'q2) dup q@ cell+ >r ( #left) >r
         q=
  WHILE
  REPEAT ( 'array1 'q1) drop @ ( len) 2r> nip - ( index+1)
  ELSE   ( 'array1 'q1) 2drop 2r> 2drop 0
  THEN ;

\ *** TESTS

: Q{   ( <qinitials> -- )
(
Save the qstack <qinitials> for restoration by }Q.   Clear
<qresults> and <qgoals> storage to put it in a defined state for
}Q in case Q-> is missing.
)
  qdepth dup MAX-QT-INITIALS > ABORT" TOO MANY Q INITIALS"
  qt-initials qt-a!   0 qt-results !   0 qt-goals ! ;

: Q->  ( <qresults> -- )
(
Record the depth and contents of the qstack.
)
  qdepth dup MAX-QT-RESULTS > ABORT" TOO MANY Q RESULTS"
  qt-results qt-a! ;

: }Q   ( <qgoals> -- <qinitials> )
(
Save the qstack <qgoals>, compare them with the saved
<qresults>, and restore the <qinitials>.
)
  qdepth dup MAX-QT-RESULTS > ABORT" TOO MANY Q GOALS"
  qt-goals qt-a!
  qt-results qt-goals qt-a<> dup qt-error-index !
  ?DUP IF -1 =
    IF   S" WRONG NUMBER OF Q RESULTS: "
    ELSE -1 qt-error-index +! S" INCORRECT Q RESULT: " THEN
    QT-ERROR
  THEN
  qt-initials qt-a@ ( len) drop ;

BASE !

\ *** END of qtester.fs
