(   Title:  A utiilty for testing words that use the frstack for
	    MPFR multiple precision float-reliables
     File:  rftester.fs
 Log file:  gmpfr.log
   Author:  David N. Williams
  License:  Public Domain, John Hayes
  Version:  1.5.0
  Started:  March 26, 2011
  Revised:  April 13, 2011

For any code derived from John Hayes' tester program:
)
\ (C) 1995 JOHNS HOPKINS UNIVERSITY / APPLIED PHYSICS LABORATORY
\ MAY BE DISTRIBUTED FREELY AS LONG AS THIS COPYRIGHT NOTICE REMAINS.
(
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

The rest is public domain.
)
BASE @ DECIMAL

\ *** ERRORS

VARIABLE RFT-ERROR-XT
: RFT-ERROR RFT-ERROR-XT @ EXECUTE ;  \ vectored error reporting

VARIABLE rft-error-index  \ holds array index|-1 after error

: RFT-ERROR-DEFAULT   ( c-addr u -- )
(
Display an error message followed by the line that had the
error.
)
  TYPE SOURCE TYPE CR ;
' RFT-ERROR-DEFAULT RFT-ERROR-XT !

\ *** COMPARISONS

VARIABLE RFT-TEST=-XT

: RFT-TEST=  (rf: x y -- ) ( -- flag )  RFT-TEST=-XT @ EXECUTE ;

\ The sign of these tolerances must be plus.
rf" 0e0"   RFCONSTANT RFT-ABS-ERROR-DEFAULT
rf" 1e-12" RFCONSTANT RFT-REL-ERROR-DEFAULT
RFVARIABLE RFT-ABS-ERROR   RFT-ABS-ERROR-DEFAULT RFT-ABS-ERROR rf!
RFVARIABLE RFT-REL-ERROR   RFT-REL-ERROR-DEFAULT RFT-REL-ERROR rf!


: RFT-DATUM=  (rf: x y -- ) ( -- flag )
(
Leave TRUE if the two rffloats have equivalent internal
representations.  Else leave FALSE.
)
  rf= ;

\ |x - y| < eps
: RFT-ABS=  (rf: x y -- ) ( -- flag )
  rf- |rf| RFT-ABS-ERROR rf@ rf<  ;

\ |m - r| < eps * |r|
: RFT-REL=  (rf: meas ref -- ) ( -- flag )
  rf-reldiff RFT-REL-ERROR rf@ rf< ;

: RFT-REL0=  (rf: meas ref -- ) ( -- flag )
  rfdup rf0= IF RFT-ABS= ELSE RFT-REL= THEN ;

: SET-RFT-MODE-EXACT ( -- )  ['] RFT-DATUM= RFT-TEST=-XT ! ;
: SET-RFT-MODE-ABS   ( -- )  ['] RFT-ABS=   RFT-TEST=-XT ! ;
: SET-RFT-MODE-REL   ( -- )  ['] RFT-REL=   RFT-TEST=-XT ! ;
: SET-RFT-MODE-REL0  ( -- )  ['] RFT-REL0=  RFT-TEST=-XT ! ;

SET-RFT-MODE-EXACT


\ *** ARRAYS
(
The first array slot has index zero.
)

\ USER CONFIG DEFAULTS
[UNDEFINED] MAX-RFT-INITIALS [IF] 32 CONSTANT MAX-RFT-INITIALS [THEN]
[UNDEFINED] MAX-RFT-RESULTS  [IF] 32 CONSTANT MAX-RFT-RESULTS  [THEN]

\ Arrays begin with a cell-sized count.
CREATE rft-initials MAX-RFT-INITIALS 1+ CELLS ALLOT
CREATE rft-results  MAX-RFT-RESULTS  1+ CELLS ALLOT
CREATE rft-goals    MAX-RFT-RESULTS  1+ CELLS ALLOT

: 0RFT-A  ( +n 'array -- )  \ assume n<>0
(
Initialize array rf variables for garbage collection.
)
  swap 1+ cells over + swap DO 0 i ! 1 cells +LOOP ;

MAX-RFT-INITIALS rft-initials 0RFT-A
MAX-RFT-RESULTS  rft-results  0RFT-A
MAX-RFT-RESULTS  rft-goals    0RFT-A

\ Overflow checking to be done elsewhere.
: rft-a!   ( +n 'array -- ) (rf: f_[n-1] ... f_0 -- )
  2dup ! cell+ swap ( 'array n) dup
  IF   cells over + swap
       DO i rf! 1 cells +LOOP 
  ELSE 2drop THEN ;

: rft-a@   ( 'array -- +n ) (rf: -- f_[n-1] ... f_0 )
  dup @ dup >r ( 'array n r: n)
  IF   cell+  r@ 1- cells over + 
       DO i rf@ -1 cells +LOOP
  ELSE drop THEN r> ;

[UNDEFINED] NIP [IF] : NIP  ( a b -- b)  swap drop ; [THEN]

: rft-a<>  ( 'array1 'array2 -- [index+1]|-1|0 )
(
Nonzero return for errors:
  -1       unequal array size
  index+1  deepest unequal array elements
)
  dup @ swap cell+ >r >r  ( 'array1 r: 'f2 #left )
  dup @ r@ <> IF 2r> 2drop drop -1 EXIT THEN
  ( 'array1) dup cell+
  BEGIN  r@ ( 'array1 'f1 #left)
  WHILE  ( 'f1) dup rf@ cell+
         r> ( #left) 1- r>
         ( 'f2) dup rf@ cell+ >r ( #left) >r
         RFT-TEST=
  WHILE
  REPEAT ( 'array1 'f1) drop @ ( len) 2r> nip - ( index+1)
  ELSE   ( 'array1 'f1) 2drop 2r> 2drop 0
  THEN ;

\ *** TESTS

: RF{   ( <rfinitials> -- )
(
Save the rfstack <rfinitials> for restoration by }RF. 
Clear <rfresults> and <rfgoals> storage to put it in a defined
state for }RF in case RF-> is missing.
)
  rfdepth dup MAX-RFT-INITIALS > ABORT" TOO MANY RF INITIALS"
  rft-initials rft-a!   0 rft-results !   0 rft-goals ! ;

: RF->  ( <rfresults> -- )
(
Record the depth and contents of the rfstack.
)
  rfdepth dup MAX-RFT-RESULTS > ABORT" TOO MANY RF RESULTS"
  rft-results rft-a! ;

: }RF   ( <rfgoals> -- <rfinitials> )
(
Save the rfstack <rfgoals>, compare them with the saved
<rfresults>, and restore the <rfinitials>.
)
  rfdepth dup MAX-RFT-RESULTS > ABORT" TOO MANY RF GOALS"
  rft-goals rft-a!
  rft-results rft-goals rft-a<> dup rft-error-index !
  ?DUP IF -1 =
    IF   S" WRONG NUMBER OF RF RESULTS: "
    ELSE -1 rft-error-index +! S" INCORRECT RF RESULT: " THEN
    RFT-ERROR
  THEN
  rft-initials rft-a@ ( len) drop ;

BASE !

\ *** END of rftester.fs
